@@ -1,9 +1,13 @@
 """
-This is an object-orient plotting library.
+This is an object-oriented plotting library.
 
-A procedural interface is provided by the companion pylab module,
+A procedural interface is provided by the companion pyplot module,
 which may be imported directly, e.g::
 
+    from matplotlib.pyplot import *
+
+To include numpy functions too, use::
+
     from pylab import *
 
 or using ipython::
@@ -11,9 +15,11 @@ or using ipython::
     ipython -pylab
 
 For the most part, direct use of the object-oriented library is
-encouraged when programming rather than working interactively.  The
-exceptions are the pylab commands :func:`~matplotlib.pyplot.figure`,
+encouraged when programming; pyplot is primarily for working
+interactively.  The
+exceptions are the pyplot commands :func:`~matplotlib.pyplot.figure`,
 :func:`~matplotlib.pyplot.subplot`,
+:func:`~matplotlib.pyplot.subplots`,
 :func:`~matplotlib.backends.backend_qt4agg.show`, and
 :func:`~pyplot.savefig`, which can greatly simplify scripting.
 
@@ -36,7 +42,7 @@ Modules include:
         defines the :class:`~matplotlib.lines.Line2D` class for
         drawing lines and markers
 
-    :mod`matplotlib.patches`
+    :mod:`matplotlib.patches`
         defines classes for drawing polygons
 
     :mod:`matplotlib.text`
@@ -84,22 +90,27 @@ The base matplotlib namespace includes:
         for the first time.  In particular, it must be called
         **before** importing pylab (if pylab is imported).
 
-matplotlib is written by John D. Hunter (jdh2358 at gmail.com) and a
-host of others.
-"""
-from __future__ import generators
+matplotlib was initially written by John D. Hunter (1968-2012) and is now
+developed and maintained by a host of others.
 
-__version__  = '0.98.5.2'
-__revision__ = '$Revision: 6660 $'
-__date__     = '$Date: 2008-12-18 06:10:51 -0600 (Thu, 18 Dec 2008) $'
+Occasionally the internal documentation (python docstrings) will refer
+to MATLAB&reg;, a registered trademark of The MathWorks, Inc.
+
+"""
+from __future__ import print_function
+
+__version__  = '1.2.1'
+__version__numpy__ = '1.4' # minimum required numpy version
 
 import os, re, shutil, subprocess, sys, warnings
 import distutils.sysconfig
 import distutils.version
 
-
-NEWCONFIG = False
-
+try:
+    reload
+except NameError:
+    # Python 3
+    from imp import reload
 
 # Needed for toolkit setuptools support
 if 0:
@@ -111,6 +122,20 @@ if 0:
 if not hasattr(sys, 'argv'):  # for modpython
     sys.argv = ['modpython']
 
+
+class MatplotlibDeprecationWarning(UserWarning):
+    """
+    A class for issuing deprecation warnings for Matplotlib users.
+
+    In light of the fact that Python builtin DeprecationWarnings are ignored
+    by default as of Python 2.7 (see link below), this class was put in to
+    allow for the signaling of deprecation, but via UserWarnings which are not
+    ignored by default.
+
+    http://docs.python.org/dev/whatsnew/2.7.html#the-future-for-python-2-x
+    """
+    pass
+
 """
 Manage user customizations through a rc file.
 
@@ -126,11 +151,23 @@ The default file location is given in the following order
 
 import sys, os, tempfile
 
-from rcsetup import defaultParams, validate_backend, validate_toolbar
-from rcsetup import validate_cairo_format
+if sys.version_info[0] >= 3:
+    def ascii(s): return bytes(s, 'ascii')
+
+    def byte2str(b): return b.decode('ascii')
+
+else:
+    ascii = str
+
+    def byte2str(b): return b
+
+
+from matplotlib.rcsetup import (defaultParams,
+                                validate_backend,
+                                validate_toolbar)
 
 major, minor1, minor2, s, tmp = sys.version_info
-_python24 = major>=2 and minor1>=4
+_python24 = (major == 2 and minor1 >= 4) or major >= 3
 
 # the havedate check was a legacy from old matplotlib which preceeded
 # datetime support
@@ -144,11 +181,16 @@ _havedate = True
 if not _python24:
     raise ImportError('matplotlib requires Python 2.4 or later')
 
+
 import numpy
-nn = numpy.__version__.split('.')
-if not (int(nn[0]) >= 1 and int(nn[1]) >= 1):
+from distutils import version
+expected_version = version.LooseVersion(__version__numpy__)
+found_version = version.LooseVersion(numpy.__version__)
+if not found_version >= expected_version:
     raise ImportError(
-            'numpy 1.1 or later is required; you have %s' % numpy.__version__)
+        'numpy %s or later is required; you have %s' % (
+            __version__numpy__, numpy.__version__))
+del version
 
 def is_string_like(obj):
     if hasattr(obj, 'shape'): return 0
@@ -166,8 +208,10 @@ def _is_writable_dir(p):
     except TypeError: return False
     try:
         t = tempfile.TemporaryFile(dir=p)
-        t.write('1')
-        t.close()
+        try:
+            t.write(ascii('1'))
+        finally:
+            t.close()
     except OSError: return False
     else: return True
 
@@ -184,8 +228,14 @@ class Verbose:
     _commandLineVerbose = None
 
     for arg in sys.argv[1:]:
-        if not arg.startswith('--verbose-'): continue
-        _commandLineVerbose = arg[10:]
+        if not arg.startswith('--verbose-'):
+            continue
+        level_str = arg[10:]
+        # If it doesn't match one of ours, then don't even
+        # bother noting it, we are just a 3rd-party library
+        # to somebody else's script.
+        if level_str in levels:
+            _commandLineVerbose = level_str
 
     def __init__(self):
         self.set_level('silent')
@@ -197,8 +247,10 @@ class Verbose:
         if self._commandLineVerbose is not None:
             level = self._commandLineVerbose
         if level not in self.levels:
-            raise ValueError('Illegal verbose string "%s".  Legal values are %s'%(level, self.levels))
-        self.level = level
+            warnings.warn('matplotlib: unrecognized --verbose-* string "%s".'
+                          ' Legal values are %s' % (level, self.levels))
+        else:
+            self.level = level
 
     def set_fileo(self, fname):
         std = {
@@ -209,7 +261,7 @@ class Verbose:
             self.fileo = std[fname]
         else:
             try:
-                fileo = file(fname, 'w')
+                fileo = open(fname, 'w')
             except IOError:
                 raise ValueError('Verbose object could not open log file "%s" for writing.\nCheck your matplotlibrc verbose.fileo setting'%fname)
             else:
@@ -222,7 +274,7 @@ class Verbose:
 
         """
         if self.ge(level):
-            print >>self.fileo, s
+            print(s, file=self.fileo)
             return True
         return False
 
@@ -255,12 +307,13 @@ class Verbose:
 verbose=Verbose()
 
 
+
 def checkdep_dvipng():
     try:
         s = subprocess.Popen(['dvipng','-version'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
         line = s.stdout.readlines()[1]
-        v = line.split()[-1]
+        v = byte2str(line.split()[-1])
         return v
     except (IndexError, ValueError, OSError):
         return None
@@ -273,7 +326,7 @@ def checkdep_ghostscript():
             command_args = ['gs', '--version']
         s = subprocess.Popen(command_args, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
-        v = s.stdout.read()[:-1]
+        v = byte2str(s.stdout.read()[:-1])
         return v
     except (IndexError, ValueError, OSError):
         return None
@@ -282,7 +335,7 @@ def checkdep_tex():
     try:
         s = subprocess.Popen(['tex','-version'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
-        line = s.stdout.readlines()[0]
+        line = byte2str(s.stdout.readlines()[0])
         pattern = '3\.1\d+'
         match = re.search(pattern, line)
         v = match.group(0)
@@ -295,8 +348,32 @@ def checkdep_pdftops():
         s = subprocess.Popen(['pdftops','-v'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
         for line in s.stderr:
-            if 'version' in line:
-                v = line.split()[-1]
+            if b'version' in line:
+                v = byte2str(line.split()[-1])
+        return v
+    except (IndexError, ValueError, UnboundLocalError, OSError):
+        return None
+
+def checkdep_inkscape():
+    try:
+        s = subprocess.Popen(['inkscape','-V'], stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+        for line in s.stdout:
+            if b'Inkscape' in line:
+                v = byte2str(line.split()[1])
+                break
+        return v
+    except (IndexError, ValueError, UnboundLocalError, OSError):
+        return None
+
+def checkdep_xmllint():
+    try:
+        s = subprocess.Popen(['xmllint','--version'], stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+        for line in s.stderr:
+            if b'version' in line:
+                v = byte2str(line.split()[-1])
+                break
         return v
     except (IndexError, ValueError, UnboundLocalError, OSError):
         return None
@@ -411,21 +488,41 @@ def _get_home():
         raise RuntimeError('please define environment variable $HOME')
 
 
+def _create_tmp_config_dir():
+    """
+    If the config directory can not be created, create a temporary
+    directory.
+    """
+    import getpass
+    import tempfile
+
+    tempdir = os.path.join(
+        tempfile.gettempdir(), 'matplotlib-%s' % getpass.getuser())
+    os.environ['MPLCONFIGDIR'] = tempdir
+
+    return tempdir
+
 
 get_home = verbose.wrap('$HOME=%s', _get_home, always=False)
 
 def _get_configdir():
     """
-    Return the string representing the configuration dir.
+    Return the string representing the configuration directory.
 
-    default is HOME/.matplotlib.  you can override this with the
-    MPLCONFIGDIR environment variable
+    Default is HOME/.matplotlib.  You can override this with the
+    MPLCONFIGDIR environment variable.  If the default is not
+    writable, and MPLCONFIGDIR is not set, then
+    tempfile.gettempdir() is used to provide a directory in
+    which a matplotlib subdirectory is created as the configuration
+    directory.
     """
 
     configdir = os.environ.get('MPLCONFIGDIR')
     if configdir is not None:
+        if not os.path.exists(configdir):
+            os.makedirs(configdir)
         if not _is_writable_dir(configdir):
-            raise RuntimeError('Could not write to MPLCONFIGDIR="%s"'%configdir)
+            return _create_tmp_config_dir()
         return configdir
 
     h = get_home()
@@ -433,12 +530,12 @@ def _get_configdir():
 
     if os.path.exists(p):
         if not _is_writable_dir(p):
-            raise RuntimeError("'%s' is not a writable dir; you must set %s/.matplotlib to be a writable dir.  You can also set environment variable MPLCONFIGDIR to any writable directory where you want matplotlib data stored "% (h, h))
+            return _create_tmp_config_dir()
     else:
         if not _is_writable_dir(h):
-            raise RuntimeError("Failed to create %s/.matplotlib; consider setting MPLCONFIGDIR to a writable directory for matplotlib configuration data"%h)
-
-        os.mkdir(p)
+            return _create_tmp_config_dir()
+        from matplotlib.cbook import mkdirs
+        mkdirs(p)
 
     return p
 get_configdir = verbose.wrap('CONFIGDIR=%s', _get_configdir, always=False)
@@ -454,27 +551,32 @@ def _get_data_path():
         return path
 
     path = os.sep.join([os.path.dirname(__file__), 'mpl-data'])
-    if os.path.isdir(path): return path
+    if os.path.isdir(path):
+        return path
 
     # setuptools' namespace_packages may highjack this init file
     # so need to try something known to be in matplotlib, not basemap
     import matplotlib.afm
     path = os.sep.join([os.path.dirname(matplotlib.afm.__file__), 'mpl-data'])
-    if os.path.isdir(path): return path
+    if os.path.isdir(path):
+        return path
 
     # py2exe zips pure python, so still need special check
     if getattr(sys,'frozen',None):
-        path = os.path.join(os.path.split(sys.path[0])[0], 'mpl-data')
-        if os.path.isdir(path): return path
-        else:
-            # Try again assuming we need to step up one more directory
-            path = os.path.join(os.path.split(os.path.split(sys.path[0])[0])[0],
-                                'mpl-data')
-        if os.path.isdir(path): return path
-        else:
-            # Try again assuming sys.path[0] is a dir not a exe
-            path = os.path.join(sys.path[0], 'mpl-data')
-            if os.path.isdir(path): return path
+        exe_path = os.path.dirname(sys.executable)
+        path = os.path.join(exe_path, 'mpl-data')
+        if os.path.isdir(path):
+            return path
+
+        # Try again assuming we need to step up one more directory
+        path = os.path.join(os.path.split(exe_path)[0], 'mpl-data')
+        if os.path.isdir(path):
+            return path
+
+        # Try again assuming sys.path[0] is a dir not a exe
+        path = os.path.join(sys.path[0], 'mpl-data')
+        if os.path.isdir(path):
+            return path
 
     raise RuntimeError('Could not find the matplotlib data files')
 
@@ -487,19 +589,12 @@ get_data_path = verbose.wrap('matplotlib data path %s', _get_data_path_cached,
                              always=False)
 
 
+
 def get_example_data(fname):
     """
-    return a filehandle to one of the example files in mpl-data/example
-
-    *fname*
-        the name of one of the files in mpl-data/example
+    get_example_data is deprecated -- use matplotlib.cbook.get_sample_data instead
     """
-    datadir = os.path.join(get_data_path(), 'example')
-    fullpath = os.path.join(datadir, fname)
-    if not os.path.exists(fullpath):
-        raise IOError('could not find matplotlib example file "%s" in data directory "%s"'%(
-            fname, datadir))
-    return file(fullpath, 'rb')
+    raise NotImplementedError('get_example_data is deprecated -- use matplotlib.cbook.get_sample_data instead')
 
 
 def get_py2exe_datafiles():
@@ -515,7 +610,7 @@ def get_py2exe_datafiles():
         root = root.replace(tail, 'mpl-data')
         root = root[root.index('mpl-data'):]
         d[root] = files
-    return d.items()
+    return list(d.items())
 
 
 def matplotlib_fname():
@@ -534,10 +629,10 @@ def matplotlib_fname():
 
     oldname = os.path.join( os.getcwd(), '.matplotlibrc')
     if os.path.exists(oldname):
-        print >> sys.stderr, """\
+        print("""\
 WARNING: Old rc filename ".matplotlibrc" found in working dir
   and and renamed to new default rc file name "matplotlibrc"
-  (no leading"dot"). """
+  (no leading"dot"). """, file=sys.stderr)
         shutil.move('.matplotlibrc', 'matplotlibrc')
 
     home = get_home()
@@ -545,9 +640,9 @@ WARNING: Old rc filename ".matplotlibrc" found in working dir
     if os.path.exists(oldname):
         configdir = get_configdir()
         newname = os.path.join(configdir, 'matplotlibrc')
-        print >> sys.stderr, """\
+        print("""\
 WARNING: Old rc filename "%s" found and renamed to
-  new default rc file name "%s"."""%(oldname, newname)
+  new default rc file name "%s"."""%(oldname, newname), file=sys.stderr)
 
         shutil.move(oldname, newname)
 
@@ -580,6 +675,16 @@ _deprecated_map = {
     'text.fontweight':  'font.weight',
     'text.fontsize':    'font.size',
     'tick.size' :       'tick.major.size',
+    'svg.embed_char_paths' : 'svg.fonttype',
+    'savefig.extension' : 'savefig.format'
+    }
+
+_deprecated_ignore_map = {
+    'legend.pad' :       'legend.borderpad',
+    'legend.labelsep' :       'legend.labelspacing',
+    'legend.handlelen' :       'legend.handlelength',
+    'legend.handletextsep' :       'legend.handletextpad',
+    'legend.axespad' :       'legend.borderaxespad',
     }
 
 
@@ -594,20 +699,49 @@ class RcParams(dict):
 
     validate = dict([ (key, converter) for key, (default, converter) in \
                      defaultParams.iteritems() ])
+    msg_depr = "%s is deprecated and replaced with %s; please use the latter."
+    msg_depr_ignore = "%s is deprecated and ignored. Use %s"
 
     def __setitem__(self, key, val):
         try:
-            if key in _deprecated_map.keys():
+            if key in _deprecated_map:
                 alt = _deprecated_map[key]
-                warnings.warn('%s is deprecated in matplotlibrc. Use %s \
-instead.'% (key, alt))
+                warnings.warn(self.msg_depr % (key, alt))
                 key = alt
+            elif key in _deprecated_ignore_map:
+                alt = _deprecated_ignore_map[key]
+                warnings.warn(self.msg_depr_ignore % (key, alt))
+                return
             cval = self.validate[key](val)
             dict.__setitem__(self, key, cval)
         except KeyError:
             raise KeyError('%s is not a valid rc parameter.\
-See rcParams.keys() for a list of valid parameters.'%key)
+See rcParams.keys() for a list of valid parameters.' % (key,))
 
+    def __getitem__(self, key):
+        if key in _deprecated_map:
+            alt = _deprecated_map[key]
+            warnings.warn(self.msg_depr % (key, alt))
+            key = alt
+        elif key in _deprecated_ignore_map:
+            alt = _deprecated_ignore_map[key]
+            warnings.warn(self.msg_depr_ignore % (key, alt))
+            key = alt
+        return dict.__getitem__(self, key)
+
+    def keys(self):
+        """
+        Return sorted list of keys.
+        """
+        k = list(dict.keys(self))
+        k.sort()
+        return k
+
+    def values(self):
+        """
+        Return values in order of sorted keys.
+        """
+        return [self[k] for k in self.iterkeys()]
 
 def rc_params(fail_on_error=False):
     'Return the default params updated from the values in the rc file'
@@ -621,23 +755,30 @@ def rc_params(fail_on_error=False):
         warnings.warn(message)
         return ret
 
+    return rc_params_from_file(fname, fail_on_error)
+
+
+def rc_params_from_file(fname, fail_on_error=False):
+    """Load and return params from fname."""
+
     cnt = 0
     rc_temp = {}
-    for line in file(fname):
-        cnt += 1
-        strippedline = line.split('#',1)[0].strip()
-        if not strippedline: continue
-        tup = strippedline.split(':',1)
-        if len(tup) !=2:
-            warnings.warn('Illegal line #%d\n\t%s\n\tin file "%s"'%\
-                          (cnt, line, fname))
-            continue
-        key, val = tup
-        key = key.strip()
-        val = val.strip()
-        if key in rc_temp:
-            warnings.warn('Duplicate key in file "%s", line #%d'%(fname,cnt))
-        rc_temp[key] = (val, line, cnt)
+    with open(fname) as fd:
+        for line in fd:
+            cnt += 1
+            strippedline = line.split('#',1)[0].strip()
+            if not strippedline: continue
+            tup = strippedline.split(':',1)
+            if len(tup) !=2:
+                warnings.warn('Illegal line #%d\n\t%s\n\tin file "%s"'%\
+                              (cnt, line, fname))
+                continue
+            key, val = tup
+            key = key.strip()
+            val = val.strip()
+            if key in rc_temp:
+                warnings.warn('Duplicate key in file "%s", line #%d'%(fname,cnt))
+            rc_temp[key] = (val, line, cnt)
 
     ret = RcParams([ (key, default) for key, (default, converter) in \
                     defaultParams.iteritems() ])
@@ -649,7 +790,7 @@ def rc_params(fail_on_error=False):
                 ret[key] = val # try to convert to proper type or raise
             else:
                 try: ret[key] = val # try to convert to proper type or skip
-                except Exception, msg:
+                except Exception as msg:
                     warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file \
 "%s"\n\t%s' % (val, cnt, line, fname, msg))
 
@@ -662,16 +803,19 @@ def rc_params(fail_on_error=False):
                 ret[key] = val # try to convert to proper type or raise
             else:
                 try: ret[key] = val # try to convert to proper type or skip
-                except Exception, msg:
+                except Exception as msg:
                     warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file \
 "%s"\n\t%s' % (val, cnt, line, fname, msg))
+        elif key in _deprecated_ignore_map:
+            warnings.warn('%s is deprecated. Update your matplotlibrc to use %s instead.'% (key, _deprecated_ignore_map[key]))
+
         else:
-            print >> sys.stderr, """
+            print("""
 Bad key "%s" on line %d in
 %s.
 You probably need to get an updated matplotlibrc file from
 http://matplotlib.sf.net/_static/matplotlibrc or from the matplotlib source
-distribution""" % (key, cnt, fname)
+distribution""" % (key, cnt, fname), file=sys.stderr)
 
     if ret['datapath'] is None:
         ret['datapath'] = get_data_path()
@@ -693,12 +837,32 @@ Please do not ask for support with these customizations active.
 # this is the instance used by the matplotlib classes
 rcParams = rc_params()
 
+if rcParams['examples.directory']:
+    # paths that are intended to be relative to matplotlib_fname()
+    # are allowed for the examples.directory parameter.
+    # However, we will need to fully qualify the path because
+    # Sphinx requires absolute paths.
+    if not os.path.isabs(rcParams['examples.directory']):
+        _basedir, _fname = os.path.split(matplotlib_fname())
+        # Sometimes matplotlib_fname() can return relative paths,
+        # Also, using realpath() guarentees that Sphinx will use
+        # the same path that matplotlib sees (in case of weird symlinks).
+        _basedir = os.path.realpath(_basedir)
+        _fullpath = os.path.join(_basedir, rcParams['examples.directory'])
+        rcParams['examples.directory'] = _fullpath
+
+rcParamsOrig = rcParams.copy()
+
 rcParamsDefault = RcParams([ (key, default) for key, (default, converter) in \
                     defaultParams.iteritems() ])
 
 rcParams['ps.usedistiller'] = checkdep_ps_distiller(rcParams['ps.usedistiller'])
 rcParams['text.usetex'] = checkdep_usetex(rcParams['text.usetex'])
 
+if rcParams['axes.formatter.use_locale']:
+    import locale
+    locale.setlocale(locale.LC_ALL, '')
+
 def rc(group, **kwargs):
     """
     Set the current rc params.  Group is the grouping for the rc, eg.
@@ -762,30 +926,75 @@ def rc(group, **kwargs):
     if is_string_like(group):
         group = (group,)
     for g in group:
-        for k,v in kwargs.items():
+        for k,v in kwargs.iteritems():
             name = aliases.get(k) or k
             key = '%s.%s' % (g, name)
-            if key not in rcParams:
+            try:
+                rcParams[key] = v
+            except KeyError:
                 raise KeyError('Unrecognized key "%s" for group "%s" and name "%s"' %
                                (key, g, name))
 
-            rcParams[key] = v
-
 def rcdefaults():
     """
-    Restore the default rc params - the ones that were created at
-    matplotlib load time.
+    Restore the default rc params.  These are not the params loaded by
+    the rc file, but mpl's internal params.  See rc_file_defaults for
+    reloading the default params from the rc file
     """
     rcParams.update(rcParamsDefault)
 
-if NEWCONFIG:
-    #print "importing from reorganized config system!"
-    try:
-        from config import rcParams, rcdefaults, mplConfig, save_config
-        verbose.set_level(rcParams['verbose.level'])
-        verbose.set_fileo(rcParams['verbose.fileo'])
-    except:
-        from config import rcParams, rcdefaults
+
+def rc_file(fname):
+    """
+    Update rc params from file.
+    """
+    rcParams.update(rc_params_from_file(fname))
+
+
+class rc_context(object):
+    """
+    Return a context manager for managing rc settings.
+
+    This allows one to do::
+
+    >>> with mpl.rc_context(fname='screen.rc'):
+    >>>     plt.plot(x, a)
+    >>>     with mpl.rc_context(fname='print.rc'):
+    >>>         plt.plot(x, b)
+    >>>     plt.plot(x, c)
+
+    The 'a' vs 'x' and 'c' vs 'x' plots would have settings from
+    'screen.rc', while the 'b' vs 'x' plot would have settings from
+    'print.rc'.
+
+    A dictionary can also be passed to the context manager::
+
+    >>> with mpl.rc_context(rc={'text.usetex': True}, fname='screen.rc'):
+    >>>     plt.plot(x, a)
+
+    The 'rc' dictionary takes precedence over the settings loaded from
+    'fname'.  Passing a dictionary only is also valid.
+    """
+
+    def __init__(self, rc=None, fname=None):
+        self.rcdict = rc
+        self.fname = fname
+    def __enter__(self):
+        self._rcparams = rcParams.copy()
+        if self.fname:
+            rc_file(self.fname)
+        if self.rcdict:
+            rcParams.update(self.rcdict)
+    def __exit__(self, type, value, tb):
+        rcParams.update(self._rcparams)
+
+
+def rc_file_defaults():
+    """
+    Restore the default rc params from the original matplotlib rc that
+    was loaded
+    """
+    rcParams.update(rcParamsOrig)
 
 _use_error_msg = """ This call to matplotlib.use() has no effect
 because the the backend has already been chosen;
@@ -793,42 +1002,60 @@ matplotlib.use() must be called *before* pylab, matplotlib.pyplot,
 or matplotlib.backends is imported for the first time.
 """
 
-def use(arg, warn=True):
+def use(arg, warn=True, force=False):
     """
     Set the matplotlib backend to one of the known backends.
 
-    The argument is case-insensitive.  For the Cairo backend,
-    the argument can have an extension to indicate the type of
-    output.  Example:
+    The argument is case-insensitive. *warn* specifies whether a
+    warning should be issued if a backend has already been set up.
+    *force* is an **experimental** flag that tells matplotlib to
+    attempt to initialize a new backend by reloading the backend
+    module.
 
-        use('cairo.pdf')
+    .. note::
 
-    will specify a default of pdf output generated by Cairo.
+        This function must be called *before* importing pyplot for
+        the first time; or, if you are not using pyplot, it must be called
+        before importing matplotlib.backends.  If warn is True, a warning
+        is issued if you try and call this after pylab or pyplot have been
+        loaded.  In certain black magic use cases, e.g.
+        :func:`pyplot.switch_backend`, we are doing the reloading necessary to
+        make the backend switch work (in some cases, e.g. pure image
+        backends) so one can set warn=False to suppress the warnings.
+
+    To find out which backend is currently set, see
+    :func:`matplotlib.get_backend`.
 
-    Note: this function must be called *before* importing pylab for
-    the first time; or, if you are not using pylab, it must be called
-    before importing matplotlib.backends.  If warn is True, a warning
-    is issued if you try and callthis after pylab or pyplot have been
-    loaded.  In certain black magic use cases, eg
-    pyplot.switch_backends, we are doing the reloading necessary to
-    make the backend switch work (in some cases, eg pure image
-    backends) so one can set warn=False to supporess the warnings
     """
+    # Check if we've already set up a backend
     if 'matplotlib.backends' in sys.modules:
-        if warn: warnings.warn(_use_error_msg)
-        return
-    arg = arg.lower()
+        if warn:
+            warnings.warn(_use_error_msg)
+
+        # Unless we've been told to force it, just return
+        if not force:
+            return
+        need_reload = True
+    else:
+        need_reload = False
+
+    # Set-up the proper backend name
     if arg.startswith('module://'):
         name = arg
     else:
-        be_parts = arg.split('.')
-        name = validate_backend(be_parts[0])
+        # Lowercase only non-module backend names (modules are case-sensitive)
+        arg = arg.lower()
+        name = validate_backend(arg)
+
     rcParams['backend'] = name
-    if name == 'cairo' and len(be_parts) > 1:
-        rcParams['cairo.format'] = validate_cairo_format(be_parts[1])
+
+    # If needed we reload here because a lot of setup code is triggered on
+    # module import. See backends/__init__.py for more detail.
+    if need_reload:
+        reload(sys.modules['matplotlib.backends'])
 
 def get_backend():
-    "Returns the current backend"
+    "Returns the current backend."
     return rcParams['backend']
 
 def interactive(b):
@@ -854,7 +1081,7 @@ def tk_window_focus():
 
 # Now allow command line to override
 
-# Allow command line access to the backend with -d (matlab compatible
+# Allow command line access to the backend with -d (MATLAB compatible
 # flag)
 
 for s in sys.argv[1:]:
@@ -865,10 +1092,71 @@ for s in sys.argv[1:]:
             pass
         # we don't want to assume all -d flags are backends, eg -debug
 
+default_test_modules = [
+    'matplotlib.tests.test_agg',
+    'matplotlib.tests.test_artist',
+    'matplotlib.tests.test_axes',
+    'matplotlib.tests.test_backend_svg',
+    'matplotlib.tests.test_backend_pgf',
+    'matplotlib.tests.test_basic',
+    'matplotlib.tests.test_cbook',
+    'matplotlib.tests.test_colorbar',
+    'matplotlib.tests.test_colors',
+    'matplotlib.tests.test_dates',
+    'matplotlib.tests.test_delaunay',
+    'matplotlib.tests.test_figure',
+    'matplotlib.tests.test_image',
+    'matplotlib.tests.test_legend',
+    'matplotlib.tests.test_mathtext',
+    'matplotlib.tests.test_mlab',
+    'matplotlib.tests.test_patches',
+    'matplotlib.tests.test_pickle',
+    'matplotlib.tests.test_rcparams',
+    'matplotlib.tests.test_scale',
+    'matplotlib.tests.test_simplification',
+    'matplotlib.tests.test_spines',
+    'matplotlib.tests.test_subplots',
+    'matplotlib.tests.test_text',
+    'matplotlib.tests.test_ticker',
+    'matplotlib.tests.test_tightlayout',
+    'matplotlib.tests.test_triangulation',
+    'matplotlib.tests.test_transforms',
+    'matplotlib.tests.test_arrow_patches',
+    'matplotlib.tests.test_backend_qt4',
+    ]
+
+
+def test(verbosity=1):
+    """run the matplotlib test suite"""
+    old_backend = rcParams['backend']
+    try:
+        use('agg')
+        import nose
+        import nose.plugins.builtin
+        from .testing.noseclasses import KnownFailure
+        from nose.plugins.manager import PluginManager
+
+        # store the old values before overriding
+        plugins = []
+        plugins.append( KnownFailure() )
+        plugins.extend( [plugin() for plugin in nose.plugins.builtin.plugins] )
+
+        manager = PluginManager(plugins=plugins)
+        config = nose.config.Config(verbosity=verbosity, plugins=manager)
+
+        success = nose.run( defaultTest=default_test_modules,
+                            config=config,
+                            )
+    finally:
+        if old_backend.lower() != 'agg':
+            use(old_backend)
+
+    return success
+
+test.__test__ = False # nose: this function is not a test
 
 verbose.report('matplotlib version %s'%__version__)
 verbose.report('verbose.level %s'%verbose.level)
 verbose.report('interactive is %s'%rcParams['interactive'])
-verbose.report('units is %s'%rcParams['units'])
 verbose.report('platform is %s'%sys.platform)
-verbose.report('loaded modules: %s'%sys.modules.keys(), 'debug')
+verbose.report('loaded modules: %s'%sys.modules.iterkeys(), 'debug')
