@@ -1,8 +1,11 @@
-from __future__ import division
-import re, warnings
+from __future__ import division, print_function
+import re
+import warnings
 import matplotlib
 import matplotlib.cbook as cbook
-from transforms import Bbox, IdentityTransform, TransformedBbox, TransformedPath
+from matplotlib import docstring, rcParams
+from transforms import Bbox, IdentityTransform, TransformedBbox, \
+                       TransformedPath, Transform
 from path import Path
 
 ## Note, matplotlib artists use the doc strings for set and get
@@ -22,6 +25,43 @@ from path import Path
 # http://groups.google.com/groups?hl=en&lr=&threadm=mailman.5090.1098044946.5135.python-list%40python.org&rnum=1&prev=/groups%3Fq%3D__doc__%2Bauthor%253Ajdhunter%2540ace.bsd.uchicago.edu%26hl%3Den%26btnG%3DGoogle%2BSearch
 
 
+def allow_rasterization(draw):
+    """
+    Decorator for Artist.draw method. Provides routines
+    that run before and after the draw call. The before and after functions
+    are useful for changing artist-dependant renderer attributes or making
+    other setup function calls, such as starting and flushing a mixed-mode
+    renderer.
+    """
+    def before(artist, renderer):
+        if artist.get_rasterized():
+            renderer.start_rasterizing()
+
+        if artist.get_agg_filter() is not None:
+            renderer.start_filter()
+
+    def after(artist, renderer):
+
+        if artist.get_agg_filter() is not None:
+            renderer.stop_filter(artist.get_agg_filter())
+
+        if artist.get_rasterized():
+            renderer.stop_rasterizing()
+
+    # the axes class has a second argument inframe for its draw method.
+    def draw_wrapper(artist, renderer, *args, **kwargs):
+        before(artist, renderer)
+        draw(artist, renderer, *args, **kwargs)
+        after(artist, renderer)
+
+    # "safe wrapping" to exactly replicate anything we haven't overridden above
+    draw_wrapper.__name__ = draw.__name__
+    draw_wrapper.__dict__ = draw.__dict__
+    draw_wrapper.__doc__ = draw.__doc__
+    draw_wrapper._supports_rasterization = True
+    return draw_wrapper
+
+
 class Artist(object):
     """
     Abstract base class for someone who renders into a
@@ -30,6 +70,7 @@ class Artist(object):
 
     aname = 'Artist'
     zorder = 0
+
     def __init__(self):
         self.figure = None
 
@@ -37,7 +78,7 @@ class Artist(object):
         self._transformSet = False
         self._visible = True
         self._animated = False
-        self._alpha = 1.0
+        self._alpha = None
         self.clipbox = None
         self._clippath = None
         self._clipon = True
@@ -45,17 +86,29 @@ class Artist(object):
         self._label = ''
         self._picker = None
         self._contains = None
+        self._rasterized = None
+        self._agg_filter = None
 
         self.eventson = False  # fire events only if eventson
         self._oid = 0  # an observer id
-        self._propobservers = {} # a dict from oids to funcs
-        self.axes = None
+        self._propobservers = {}  # a dict from oids to funcs
+        try:
+            self.axes = None
+        except AttributeError:
+            # Handle self.axes as a read-only property, as in Figure.
+            pass
         self._remove_method = None
         self._url = None
-        self.x_isdata = True  # False to avoid updating Axes.dataLim with x
-        self.y_isdata = True  #                                      with y
+        self._gid = None
         self._snap = None
 
+    def __getstate__(self):
+        d = self.__dict__.copy()
+        # remove the unpicklable remove method, this will get re-added on load
+        # (by the axes) if the artist lives on an axes.
+        d['_remove_method'] = None
+        return d
+
     def remove(self):
         """
         Remove the artist from the figure if possible.  The effect
@@ -71,18 +124,18 @@ class Artist(object):
         Note: there is no support for removing the artist's legend entry.
         """
 
-        # There is no method to set the callback.  Instead the parent should set
-        # the _remove_method attribute directly.  This would be a protected
-        # attribute if Python supported that sort of thing.  The callback
-        # has one parameter, which is the child to be removed.
-        if self._remove_method != None:
+        # There is no method to set the callback.  Instead the parent should
+        # set the _remove_method attribute directly.  This would be a
+        # protected attribute if Python supported that sort of thing.  The
+        # callback has one parameter, which is the child to be removed.
+        if self._remove_method is not None:
             self._remove_method(self)
         else:
             raise NotImplementedError('cannot remove artist')
         # TODO: the fix for the collections relim problem is to move the
-        # limits calculation into the artist itself, including the property
-        # of whether or not the artist should affect the limits.  Then there
-        # will be no distinction between axes.add_line, axes.add_patch, etc.
+        # limits calculation into the artist itself, including the property of
+        # whether or not the artist should affect the limits.  Then there will
+        # be no distinction between axes.add_line, axes.add_patch, etc.
         # TODO: add legend support
 
     def have_units(self):
@@ -107,7 +160,8 @@ class Artist(object):
         convert *y* using yaxis unit type
         """
         ax = getattr(self, 'axes', None)
-        if ax is None or ax.yaxis is None: return y
+        if ax is None or ax.yaxis is None:
+            return y
         return ax.yaxis.convert_units(y)
 
     def set_axes(self, axes):
@@ -144,17 +198,22 @@ class Artist(object):
         Remove a callback based on its *id*.
 
         .. seealso::
+
             :meth:`add_callback`
+               For adding callbacks
+
         """
-        try: del self._propobservers[oid]
-        except KeyError: pass
+        try:
+            del self._propobservers[oid]
+        except KeyError:
+            pass
 
     def pchanged(self):
         """
         Fire an event when property changed, calling all of the
         registered callbacks.
         """
-        for oid, func in self._propobservers.items():
+        for oid, func in self._propobservers.iteritems():
             func(self)
 
     def is_transform_set(self):
@@ -182,22 +241,24 @@ class Artist(object):
         """
         if self._transform is None:
             self._transform = IdentityTransform()
+        elif (not isinstance(self._transform, Transform)
+              and hasattr(self._transform, '_as_mpl_transform')):
+            self._transform = self._transform._as_mpl_transform(self.axes)
         return self._transform
 
     def hitlist(self, event):
         """
         List the children of the artist which contain the mouse event *event*.
         """
-        import traceback
         L = []
         try:
-            hascursor,info = self.contains(event)
+            hascursor, info = self.contains(event)
             if hascursor:
                 L.append(self)
         except:
+            import traceback
             traceback.print_exc()
-            print "while checking",self.__class__
-
+            print("while checking", self.__class__)
 
         for a in self.get_children():
             L.extend(a.hitlist(event))
@@ -217,12 +278,12 @@ class Artist(object):
         selection, such as which points are contained in the pick radius.  See
         individual artists for details.
         """
-        if callable(self._contains): return self._contains(self,mouseevent)
-        #raise NotImplementedError,str(self.__class__)+" needs 'contains' method"
+        if callable(self._contains):
+            return self._contains(self, mouseevent)
         warnings.warn("'%s' needs 'contains' method" % self.__class__.__name__)
-        return False,{}
+        return False, {}
 
-    def set_contains(self,picker):
+    def set_contains(self, picker):
         """
         Replace the contains test used by this artist. The new picker
         should be a callable function which determines whether the
@@ -263,15 +324,22 @@ class Artist(object):
         if self.pickable():
             picker = self.get_picker()
             if callable(picker):
-                inside,prop = picker(self,mouseevent)
+                inside, prop = picker(self, mouseevent)
             else:
-                inside,prop = self.contains(mouseevent)
+                inside, prop = self.contains(mouseevent)
             if inside:
                 self.figure.canvas.pick_event(mouseevent, self, **prop)
 
         # Pick children
         for a in self.get_children():
-            a.pick(mouseevent)
+            # make sure the event happened in the same axes
+            ax = getattr(a, 'axes', None)
+            if mouseevent.inaxes is None or mouseevent.inaxes == ax:
+                # we need to check if mouseevent.inaxes is None
+                # because some objects associated with an axes (eg a
+                # tick label) can be outside the bounding box of the
+                # axes and inaxes will be None
+                a.pick(mouseevent)
 
     def set_picker(self, picker):
         """
@@ -327,9 +395,25 @@ class Artist(object):
     def set_url(self, url):
         """
         Sets the url for the artist
+
+        ACCEPTS: a url string
         """
         self._url = url
 
+    def get_gid(self):
+        """
+        Returns the group id
+        """
+        return self._gid
+
+    def set_gid(self, gid):
+        """
+        Sets the (group) id for the artist
+
+        ACCEPTS: an id string
+        """
+        self._gid = gid
+
     def get_snap(self):
         """
         Returns the snap setting which may be:
@@ -341,9 +425,12 @@ class Artist(object):
           * None: (auto) If the path contains only rectilinear line
             segments, round to the nearest pixel center
 
-        Only supported by the Agg backends.
+        Only supported by the Agg and MacOSX backends.
         """
-        return self._snap
+        if rcParams['path.snap']:
+            return self._snap
+        else:
+            return False
 
     def set_snap(self, snap):
         """
@@ -356,7 +443,7 @@ class Artist(object):
           * None: (auto) If the path contains only rectilinear line
             segments, round to the nearest pixel center
 
-        Only supported by the Agg backends.
+        Only supported by the Agg and MacOSX backends.
         """
         self._snap = snap
 
@@ -407,12 +494,13 @@ class Artist(object):
         :class:`~matplotlib.transforms.Transform`) |
         :class:`~matplotlib.patches.Patch` | None ]
         """
-        from patches import Patch, Rectangle
+        from matplotlib.patches import Patch, Rectangle
 
         success = False
         if transform is None:
             if isinstance(path, Rectangle):
-                self.clipbox = TransformedBbox(Bbox.unit(), path.get_transform())
+                self.clipbox = TransformedBbox(Bbox.unit(),
+                                              path.get_transform())
                 self._clippath = None
                 success = True
             elif isinstance(path, Patch):
@@ -420,6 +508,8 @@ class Artist(object):
                     path.get_path(),
                     path.get_transform())
                 success = True
+            elif isinstance(path, tuple):
+                path, transform = path
 
         if path is None:
             self._clippath = None
@@ -427,9 +517,12 @@ class Artist(object):
         elif isinstance(path, Path):
             self._clippath = TransformedPath(path, transform)
             success = True
+        elif isinstance(path, TransformedPath):
+            self._clippath = path
+            success = True
 
         if not success:
-            print type(path), type(transform)
+            print(type(path), type(transform))
             raise TypeError("Invalid arguments to set_clip_path")
 
         self.pchanged()
@@ -490,14 +583,43 @@ class Artist(object):
             gc.set_clip_rectangle(None)
             gc.set_clip_path(None)
 
+    def get_rasterized(self):
+        "return True if the artist is to be rasterized"
+        return self._rasterized
+
+    def set_rasterized(self, rasterized):
+        """
+        Force rasterized (bitmap) drawing in vector backend output.
+
+        Defaults to None, which implies the backend's default behavior
+
+        ACCEPTS: [True | False | None]
+        """
+        if rasterized and not hasattr(self.draw, "_supports_rasterization"):
+            warnings.warn("Rasterization of '%s' will be ignored" % self)
+
+        self._rasterized = rasterized
+
+    def get_agg_filter(self):
+        "return filter function to be used for agg filter"
+        return self._agg_filter
+
+    def set_agg_filter(self, filter_func):
+        """
+        set agg_filter fuction.
+
+        """
+        self._agg_filter = filter_func
+
     def draw(self, renderer, *args, **kwargs):
         'Derived classes drawing method'
-        if not self.get_visible(): return
+        if not self.get_visible():
+            return
 
     def set_alpha(self, alpha):
         """
         Set the alpha value used for blending - not supported on
-        all backends
+        all backends.
 
         ACCEPTS: float (0.0 transparent through 1.0 opaque)
         """
@@ -524,7 +646,6 @@ class Artist(object):
         self._visible = b
         self.pchanged()
 
-
     def set_animated(self, b):
         """
         Set the artist's animation state.
@@ -542,15 +663,16 @@ class Artist(object):
         store = self.eventson
         self.eventson = False
         changed = False
-        for k,v in props.items():
-            func = getattr(self, 'set_'+k, None)
+            
+        for k, v in props.iteritems():
+            func = getattr(self, 'set_' + k, None)
             if func is None or not callable(func):
-                raise AttributeError('Unknown property %s'%k)
+                raise AttributeError('Unknown property %s' % k)
             func(v)
             changed = True
         self.eventson = store
-        if changed: self.pchanged()
-
+        if changed:
+            self.pchanged()
 
     def get_label(self):
         """
@@ -562,13 +684,14 @@ class Artist(object):
         """
         Set the label to *s* for auto legend.
 
-        ACCEPTS: any string
+        ACCEPTS: string or anything printable with '%s' conversion.
         """
-        self._label = s
+        if s is not None:
+            self._label = '%s' % (s, )
+        else:
+            self._label = None
         self.pchanged()
 
-
-
     def get_zorder(self):
         """
         Return the :class:`Artist`'s zorder.
@@ -598,65 +721,75 @@ class Artist(object):
         self._label = other._label
         self.pchanged()
 
+    def properties(self):
+        """
+        return a dictionary mapping property name -> value for all Artist props
+        """
+        return ArtistInspector(self).properties()
 
     def set(self, **kwargs):
         """
         A tkstyle set command, pass *kwargs* to set properties
         """
         ret = []
-        for k,v in kwargs.items():
+        for k, v in kwargs.iteritems():
             k = k.lower()
-            funcName = "set_%s"%k
-            func = getattr(self,funcName)
-            ret.extend( [func(v)] )
+            funcName = "set_%s" % k
+            func = getattr(self, funcName)
+            ret.extend([func(v)])
         return ret
 
-    def findobj(self, match=None):
+    def findobj(self, match=None, include_self=True):
         """
+        Find artist objects.
+
         pyplot signature:
-          findobj(o=gcf(), match=None)
+          findobj(o=gcf(), match=None, include_self=True)
 
         Recursively find all :class:matplotlib.artist.Artist instances
         contained in self.
 
         *match* can be
 
-          - None: return all objects contained in artist (including artist)
+          - None: return all objects contained in artist.
 
-          - function with signature ``boolean = match(artist)`` used to filter matches
+          - function with signature ``boolean = match(artist)``
+            used to filter matches
 
-          - class instance: eg Line2D.  Only return artists of class type
+          - class instance: eg Line2D.  Only return artists of class type.
+
+        If *include_self* is True (default), include self in the list to be
+        checked for a match.
 
         .. plot:: mpl_examples/pylab_examples/findobj_demo.py
         """
 
-        if match is None: # always return True
-            def matchfunc(x): return True
+        if match is None:  # always return True
+            def matchfunc(x):
+                return True
         elif cbook.issubclass_safe(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
         elif callable(match):
             matchfunc = match
         else:
-            raise ValueError('match must be None, an matplotlib.artist.Artist subclass, or a callable')
-
+            raise ValueError('match must be None, a matplotlib.artist.Artist '
+                             'subclass, or a callable')
 
         artists = []
 
         for c in self.get_children():
             if matchfunc(c):
                 artists.append(c)
-            artists.extend([thisc for thisc in c.findobj(matchfunc) if matchfunc(thisc)])
+            artists.extend([thisc for thisc in
+                                c.findobj(matchfunc, include_self=False)
+                                                     if matchfunc(thisc)])
 
-        if matchfunc(self):
+        if include_self and matchfunc(self):
             artists.append(self)
         return artists
 
 
-
-
-
-
 class ArtistInspector:
     """
     A helper class to inspect an :class:`~matplotlib.artist.Artist`
@@ -666,12 +799,13 @@ class ArtistInspector:
     def __init__(self, o):
         """
         Initialize the artist inspector with an
-        :class:`~matplotlib.artist.Artist` or sequence of
-        :class:`Artists`.  If a sequence is used, we assume it is a
-        homogeneous sequence (all :class:`Artists` are of the same
-        type) and it is your responsibility to make sure this is so.
+        :class:`~matplotlib.artist.Artist` or sequence of :class:`Artists`.
+        If a sequence is used, we assume it is a homogeneous sequence (all
+        :class:`Artists` are of the same type) and it is your responsibility
+        to make sure this is so.
         """
-        if cbook.iterable(o) and len(o): o = o[0]
+        if cbook.iterable(o) and len(o):
+            o = o[0]
 
         self.oorig = o
         if not isinstance(o, type):
@@ -694,17 +828,20 @@ class ArtistInspector:
         """
         names = [name for name in dir(self.o) if
                  (name.startswith('set_') or name.startswith('get_'))
-                 and callable(getattr(self.o,name))]
+                 and callable(getattr(self.o, name))]
         aliases = {}
         for name in names:
             func = getattr(self.o, name)
-            if not self.is_alias(func): continue
+            if not self.is_alias(func):
+                continue
             docstring = func.__doc__
             fullname = docstring[10:]
             aliases.setdefault(fullname[4:], {})[name[4:]] = None
         return aliases
 
-    _get_valid_values_regex = re.compile(r"\n\s*ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))")
+    _get_valid_values_regex = re.compile(
+                r"\n\s*ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))")
+
     def get_valid_values(self, attr):
         """
         Get the legal arguments for the setter associated with *attr*.
@@ -713,16 +850,17 @@ class ArtistInspector:
         for a line that begins with ACCEPTS:
 
         Eg., for a line linestyle, return
-        [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
+        "[ ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'steps'`` | ``'None'`` ]"
         """
 
-        name = 'set_%s'%attr
+        name = 'set_%s' % attr
         if not hasattr(self.o, name):
-            raise AttributeError('%s has no function %s'%(self.o,name))
+            raise AttributeError('%s has no function %s' % (self.o, name))
         func = getattr(self.o, name)
 
         docstring = func.__doc__
-        if docstring is None: return 'unknown'
+        if docstring is None:
+            return 'unknown'
 
         if docstring.startswith('alias for '):
             return None
@@ -740,11 +878,14 @@ class ArtistInspector:
 
         setters = []
         for name in dir(self.o):
-            if not name.startswith('set_'): continue
+            if not name.startswith('set_'):
+                continue
             o = getattr(self.o, name)
-            if not callable(o): continue
+            if not callable(o):
+                continue
             func = o
-            if self.is_alias(func): continue
+            if self.is_alias(func):
+                continue
             source_class = self.o.__module__ + "." + self.o.__name__
             for cls in self.o.mro():
                 if name in cls.__dict__:
@@ -767,7 +908,8 @@ class ArtistInspector:
         function.
         """
         ds = o.__doc__
-        if ds is None: return False
+        if ds is None:
+            return False
         return ds.startswith('alias for ')
 
     def aliased_name(self, s):
@@ -781,11 +923,12 @@ class ArtistInspector:
         """
 
         if s in self.aliasd:
-            return s + ''.join([' or %s' % x for x in self.aliasd[s].keys()])
+            return s + ''.join([' or %s' % x
+                                for x
+                                in self.aliasd[s].iterkeys()])
         else:
             return s
 
-
     def aliased_name_rest(self, s, target):
         """
         return 'PROPNAME or alias' if *s* has an alias, else return
@@ -797,12 +940,13 @@ class ArtistInspector:
         """
 
         if s in self.aliasd:
-            aliases = ''.join([' or %s' % x for x in self.aliasd[s].keys()])
+            aliases = ''.join([' or %s' % x
+                               for x
+                               in self.aliasd[s].iterkeys()])
         else:
             aliases = ''
         return ':meth:`%s <%s>`%s' % (s, target, aliases)
 
-
     def pprint_setters(self, prop=None, leadingspace=2):
         """
         If *prop* is *None*, return a list of strings of all settable properies
@@ -813,12 +957,12 @@ class ArtistInspector:
         values.
         """
         if leadingspace:
-            pad = ' '*leadingspace
+            pad = ' ' * leadingspace
         else:
-            pad  = ''
+            pad = ''
         if prop is not None:
             accepts = self.get_valid_values(prop)
-            return '%s%s: %s' %(pad, prop, accepts)
+            return '%s%s: %s' % (pad, prop, accepts)
 
         attrs = self._get_setters_and_targets()
         attrs.sort()
@@ -828,7 +972,7 @@ class ArtistInspector:
             accepts = self.get_valid_values(prop)
             name = self.aliased_name(prop)
 
-            lines.append('%s%s: %s' %(pad, name, accepts))
+            lines.append('%s%s: %s' % (pad, name, accepts))
         return lines
 
     def pprint_setters_rest(self, prop=None, leadingspace=2):
@@ -841,32 +985,34 @@ class ArtistInspector:
         values.
         """
         if leadingspace:
-            pad = ' '*leadingspace
+            pad = ' ' * leadingspace
         else:
-            pad  = ''
+            pad = ''
         if prop is not None:
             accepts = self.get_valid_values(prop)
-            return '%s%s: %s' %(pad, prop, accepts)
+            return '%s%s: %s' % (pad, prop, accepts)
 
         attrs = self._get_setters_and_targets()
         attrs.sort()
         lines = []
 
         ########
-        names = [self.aliased_name_rest(prop, target) for prop, target in attrs]
+        names = [self.aliased_name_rest(prop, target)
+                 for prop, target
+                 in attrs]
         accepts = [self.get_valid_values(prop) for prop, target in attrs]
 
         col0_len = max([len(n) for n in names])
         col1_len = max([len(a) for a in accepts])
-        table_formatstr = pad + '='*col0_len + '   ' + '='*col1_len
+        table_formatstr = pad + '=' * col0_len + '   ' + '=' * col1_len
 
         lines.append('')
         lines.append(table_formatstr)
-        lines.append(pad + 'Property'.ljust(col0_len+3) + \
+        lines.append(pad + 'Property'.ljust(col0_len + 3) + \
                      'Description'.ljust(col1_len))
         lines.append(table_formatstr)
 
-        lines.extend([pad + n.ljust(col0_len+3) + a.ljust(col1_len)
+        lines.extend([pad + n.ljust(col0_len + 3) + a.ljust(col1_len)
                       for n, a in zip(names, accepts)])
 
         lines.append(table_formatstr)
@@ -878,40 +1024,56 @@ class ArtistInspector:
             accepts = self.get_valid_values(prop)
             name = self.aliased_name_rest(prop, path)
 
-            lines.append('%s%s: %s' %(pad, name, accepts))
+            lines.append('%s%s: %s' % (pad, name, accepts))
         return lines
 
-    def pprint_getters(self):
+    def properties(self):
         """
-        Return the getters and actual values as list of strings.
+        return a dictionary mapping property name -> value
         """
-
         o = self.oorig
         getters = [name for name in dir(o)
                    if name.startswith('get_')
                    and callable(getattr(o, name))]
         #print getters
         getters.sort()
-        lines = []
+        d = dict()
         for name in getters:
             func = getattr(o, name)
-            if self.is_alias(func): continue
+            if self.is_alias(func):
+                continue
 
-            try: val = func()
-            except: continue
-            if getattr(val, 'shape', ()) != () and len(val)>6:
+            try:
+                val = func()
+            except:
+                continue
+            else:
+                d[name[4:]] = val
+
+        return d
+
+    def pprint_getters(self):
+        """
+        Return the getters and actual values as list of strings.
+        """
+
+        d = self.properties()
+        names = d.keys()
+        names.sort()
+        lines = []
+        for name in names:
+            val = d[name]
+            if getattr(val, 'shape', ()) != () and len(val) > 6:
                 s = str(val[:6]) + '...'
             else:
                 s = str(val)
             s = s.replace('\n', ' ')
-            if len(s)>50:
+            if len(s) > 50:
                 s = s[:50] + '...'
-            name = self.aliased_name(name[4:])
-            lines.append('    %s = %s' %(name, s))
+            name = self.aliased_name(name)
+            lines.append('    %s = %s' % (name, s))
         return lines
 
-
-
     def findobj(self, match=None):
         """
         Recursively find all :class:`matplotlib.artist.Artist`
@@ -925,53 +1087,53 @@ class ArtistInspector:
 
         used to filter matches.
         """
-
-        if match is None: # always return True
-            def matchfunc(x): return True
+        if match is None:  # always return True
+            def matchfunc(x):
+                return True
         elif issubclass(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
         elif callable(match):
             matchfunc = func
         else:
-            raise ValueError('match must be None, an matplotlib.artist.Artist subclass, or a callable')
-
+            raise ValueError('match must be None, an '
+                             'matplotlib.artist.Artist '
+                             'subclass, or a callable')
 
         artists = []
 
         for c in self.get_children():
             if matchfunc(c):
                 artists.append(c)
-            artists.extend([thisc for thisc in c.findobj(matchfunc) if matchfunc(thisc)])
+            artists.extend([thisc
+                            for thisc
+                            in c.findobj(matchfunc)
+                            if matchfunc(thisc)])
 
         if matchfunc(self):
             artists.append(self)
         return artists
 
 
-
-
-
-
-def getp(o, property=None):
+def getp(obj, property=None):
     """
-    Return the value of handle property.  property is an optional string
+    Return the value of object's property.  *property* is an optional string
     for the property you want to return
 
     Example usage::
 
-        getp(o)  # get all the object properties
-        getp(o, 'linestyle')  # get the linestyle property
+        getp(obj)  # get all the object properties
+        getp(obj, 'linestyle')  # get the linestyle property
 
-    *o* is a :class:`Artist` instance, eg
+    *obj* is a :class:`Artist` instance, eg
     :class:`~matplotllib.lines.Line2D` or an instance of a
     :class:`~matplotlib.axes.Axes` or :class:`matplotlib.text.Text`.
     If the *property* is 'somename', this function returns
 
-      o.get_somename()
+      obj.get_somename()
 
     :func:`getp` can be used to query all the gettable properties with
-    ``getp(o)``. Many properties have aliases for shorter typing, e.g.
+    ``getp(obj)``. Many properties have aliases for shorter typing, e.g.
     'lw' is an alias for 'linewidth'.  In the output, aliases and full
     property names will be listed as:
 
@@ -981,23 +1143,23 @@ def getp(o, property=None):
 
       linewidth or lw = 2
     """
-
-    insp = ArtistInspector(o)
-
     if property is None:
+        insp = ArtistInspector(obj)
         ret = insp.pprint_getters()
-        print '\n'.join(ret)
+        print('\n'.join(ret))
         return
 
-    func = getattr(o, 'get_' + property)
-
+    func = getattr(obj, 'get_' + property)
     return func()
 
 # alias
 get = getp
 
-def setp(h, *args, **kwargs):
+
+def setp(obj, *args, **kwargs):
     """
+    Set a property on an artist object.
+
     matplotlib supports the use of :func:`setp` ("set property") and
     :func:`getp` to set and get object properties, as well as to do
     introspection on the object.  For example, to set the linestyle of a
@@ -1031,51 +1193,53 @@ def setp(h, *args, **kwargs):
       >>> lines = plot(x, y1, x, y2)
       >>> setp(lines, linewidth=2, color='r')
 
-    :func:`setp` works with the matlab(TM) style string/value pairs or
+    :func:`setp` works with the MATLAB style string/value pairs or
     with python kwargs.  For example, the following are equivalent::
 
-      >>> setp(lines, 'linewidth', 2, 'color', r')  # matlab style
+      >>> setp(lines, 'linewidth', 2, 'color', r')  # MATLAB style
 
       >>> setp(lines, linewidth=2, color='r')       # python style
     """
 
-    insp = ArtistInspector(h)
+    insp = ArtistInspector(obj)
 
-    if len(kwargs)==0 and len(args)==0:
-        print '\n'.join(insp.pprint_setters())
+    if len(kwargs) == 0 and len(args) == 0:
+        print('\n'.join(insp.pprint_setters()))
         return
 
-    if len(kwargs)==0 and len(args)==1:
-        print insp.pprint_setters(prop=args[0])
+    if len(kwargs) == 0 and len(args) == 1:
+        print(insp.pprint_setters(prop=args[0]))
         return
 
-    if not cbook.iterable(h): h = [h]
-    else: h = cbook.flatten(h)
+    if not cbook.iterable(obj):
+        objs = [obj]
+    else:
+        objs = cbook.flatten(obj)
 
-
-    if len(args)%2:
+    if len(args) % 2:
         raise ValueError('The set args must be string, value pairs')
 
     funcvals = []
-    for i in range(0, len(args)-1, 2):
-        funcvals.append((args[i], args[i+1]))
-    funcvals.extend(kwargs.items())
+    for i in range(0, len(args) - 1, 2):
+        funcvals.append((args[i], args[i + 1]))
+    funcvals.extend(kwargs.iteritems())
 
     ret = []
-    for o in h:
+    for o in objs:
         for s, val in funcvals:
             s = s.lower()
-            funcName = "set_%s"%s
-            func = getattr(o,funcName)
-            ret.extend( [func(val)] )
+            funcName = "set_%s" % s
+            func = getattr(o, funcName)
+            ret.extend([func(val)])
     return [x for x in cbook.flatten(ret)]
 
+
 def kwdoc(a):
     hardcopy = matplotlib.rcParams['docstring.hardcopy']
     if hardcopy:
-        return '\n'.join(ArtistInspector(a).pprint_setters_rest(leadingspace=2))
+        return '\n'.join(ArtistInspector(a).pprint_setters_rest(
+                                            leadingspace=2))
     else:
         return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=2))
 
-kwdocd = dict()
-kwdocd['Artist'] = kwdoc(Artist)
+docstring.interpd.update(Artist=kwdoc(Artist))
