@@ -21,8 +21,8 @@ TODO:
   * integrate screen dpi w/ ppi and text
 """
 from __future__ import division
-
-import numpy as npy
+import threading
+import numpy as np
 
 from matplotlib import verbose, rcParams
 from matplotlib.backend_bases import RendererBase,\
@@ -30,56 +30,108 @@ from matplotlib.backend_bases import RendererBase,\
 from matplotlib.cbook import is_string_like, maxdict
 from matplotlib.figure import Figure
 from matplotlib.font_manager import findfont
-from matplotlib.ft2font import FT2Font, LOAD_FORCE_AUTOHINT
+from matplotlib.ft2font import FT2Font, LOAD_FORCE_AUTOHINT, LOAD_NO_HINTING, \
+     LOAD_DEFAULT, LOAD_NO_AUTOHINT
 from matplotlib.mathtext import MathTextParser
 from matplotlib.path import Path
-from matplotlib.transforms import Bbox
+from matplotlib.transforms import Bbox, BboxBase
 
-from _backend_agg import RendererAgg as _RendererAgg
+from matplotlib.backends._backend_agg import RendererAgg as _RendererAgg
 from matplotlib import _png
 
 backend_version = 'v2.2'
 
+def get_hinting_flag():
+    mapping = {
+        True: LOAD_FORCE_AUTOHINT,
+        False: LOAD_NO_HINTING,
+        'either': LOAD_DEFAULT,
+        'native': LOAD_NO_AUTOHINT,
+        'auto': LOAD_FORCE_AUTOHINT,
+        'none': LOAD_NO_HINTING
+        }
+    return mapping[rcParams['text.hinting']]
+
+
 class RendererAgg(RendererBase):
     """
     The renderer handles all the drawing primitives using a graphics
     context instance that controls the colors/styles
     """
     debug=1
-    texd = maxdict(50)  # a cache of tex image rasters
+
+    # we want to cache the fonts at the class level so that when
+    # multiple figures are created we can reuse them.  This helps with
+    # a bug on windows where the creation of too many figures leads to
+    # too many open file handles.  However, storing them at the class
+    # level is not thread safe.  The solution here is to let the
+    # FigureCanvas acquire a lock on the fontd at the start of the
+    # draw, and release it when it is done.  This allows multiple
+    # renderers to share the cached fonts, but only one figure can
+    # draw at at time and so the font cache is used by only one
+    # renderer at a time
+
+    lock = threading.RLock()
     _fontd = maxdict(50)
     def __init__(self, width, height, dpi):
         if __debug__: verbose.report('RendererAgg.__init__', 'debug-annoying')
         RendererBase.__init__(self)
+        self.texd = maxdict(50)  # a cache of tex image rasters
+
         self.dpi = dpi
         self.width = width
         self.height = height
         if __debug__: verbose.report('RendererAgg.__init__ width=%s, height=%s'%(width, height), 'debug-annoying')
         self._renderer = _RendererAgg(int(width), int(height), dpi, debug=False)
+        self._filter_renderers = []
+
         if __debug__: verbose.report('RendererAgg.__init__ _RendererAgg done',
                                      'debug-annoying')
-        #self.draw_path = self._renderer.draw_path  # see below
-        self.draw_markers = self._renderer.draw_markers
-        self.draw_path_collection = self._renderer.draw_path_collection
-        self.draw_quad_mesh = self._renderer.draw_quad_mesh
-        self.draw_image = self._renderer.draw_image
-        self.copy_from_bbox = self._renderer.copy_from_bbox
-        self.restore_region = self._renderer.restore_region
-        self.tostring_rgba_minimized = self._renderer.tostring_rgba_minimized
+
+        self._update_methods()
         self.mathtext_parser = MathTextParser('Agg')
 
         self.bbox = Bbox.from_bounds(0, 0, self.width, self.height)
         if __debug__: verbose.report('RendererAgg.__init__ done',
                                      'debug-annoying')
 
+    def _get_hinting_flag(self):
+        if rcParams['text.hinting']:
+            return LOAD_FORCE_AUTOHINT
+        else:
+            return LOAD_NO_HINTING
+
+    # for filtering to work with rasterization, methods needs to be wrapped.
+    # maybe there is better way to do it.
+    def draw_markers(self, *kl, **kw):
+        return self._renderer.draw_markers(*kl, **kw)
+
+    def draw_path_collection(self, *kl, **kw):
+        return self._renderer.draw_path_collection(*kl, **kw)
+
+    def _update_methods(self):
+        #self.draw_path = self._renderer.draw_path  # see below
+        #self.draw_markers = self._renderer.draw_markers
+        #self.draw_path_collection = self._renderer.draw_path_collection
+        self.draw_quad_mesh = self._renderer.draw_quad_mesh
+        self.draw_gouraud_triangle = self._renderer.draw_gouraud_triangle
+        self.draw_gouraud_triangles = self._renderer.draw_gouraud_triangles
+        self.draw_image = self._renderer.draw_image
+        self.copy_from_bbox = self._renderer.copy_from_bbox
+        self.tostring_rgba_minimized = self._renderer.tostring_rgba_minimized
+
     def draw_path(self, gc, path, transform, rgbFace=None):
+        """
+        Draw the path
+        """
         nmax = rcParams['agg.path.chunksize'] # here at least for testing
         npts = path.vertices.shape[0]
-        if nmax > 100 and npts > nmax and path.should_simplify and rgbFace is None:
-            nch = npy.ceil(npts/float(nmax))
-            chsize = int(npy.ceil(npts/nch))
-            i0 = npy.arange(0, npts, chsize)
-            i1 = npy.zeros_like(i0)
+        if (nmax > 100 and npts > nmax and path.should_simplify and
+            rgbFace is None and gc.get_hatch() is None):
+            nch = np.ceil(npts/float(nmax))
+            chsize = int(np.ceil(npts/nch))
+            i0 = np.arange(0, npts, chsize)
+            i1 = np.zeros_like(i0)
             i1[:-1] = i0[1:] - 1
             i1[-1] = npts
             for ii0, ii1 in zip(i0, i1):
@@ -93,7 +145,6 @@ class RendererAgg(RendererBase):
         else:
             self._renderer.draw_path(gc, path, transform, rgbFace)
 
-
     def draw_mathtext(self, gc, x, y, s, prop, angle):
         """
         Draw the math text using matplotlib.mathtext
@@ -103,8 +154,8 @@ class RendererAgg(RendererBase):
         ox, oy, width, height, descent, font_image, used_characters = \
             self.mathtext_parser.parse(s, self.dpi, prop)
 
-        x = int(x) + ox
-        y = int(y) - oy
+        x = np.round(x + ox)
+        y = np.round(y - oy)
         self._renderer.draw_text_image(font_image, x, y + 1, angle, gc)
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
@@ -116,19 +167,20 @@ class RendererAgg(RendererBase):
         if ismath:
             return self.draw_mathtext(gc, x, y, s, prop, angle)
 
+        flags = get_hinting_flag()
         font = self._get_agg_font(prop)
         if font is None: return None
         if len(s) == 1 and ord(s) > 127:
-            font.load_char(ord(s), flags=LOAD_FORCE_AUTOHINT)
+            font.load_char(ord(s), flags=flags)
         else:
             # We pass '0' for angle here, since it will be rotated (in raster
             # space) in the following call to draw_text_image).
-            font.set_text(s, 0, flags=LOAD_FORCE_AUTOHINT)
-        font.draw_glyphs_to_bitmap()
+            font.set_text(s, 0, flags=flags)
+        font.draw_glyphs_to_bitmap(antialiased=rcParams['text.antialiased'])
 
-        #print x, y, int(x), int(y)
-
-        self._renderer.draw_text_image(font.get_image(), int(x), int(y) + 1, angle, gc)
+        #print x, y, int(x), int(y), s
+        self._renderer.draw_text_image(
+            font.get_image(), np.round(x), np.round(y) + 1, angle, gc)
 
     def get_text_width_height_descent(self, s, prop, ismath):
         """
@@ -139,21 +191,23 @@ class RendererAgg(RendererBase):
         # texmanager more efficient.  It is not meant to be used
         # outside the backend
         """
-        if ismath=='TeX':
+        if rcParams['text.usetex']:
             # todo: handle props
             size = prop.get_size_in_points()
             texmanager = self.get_texmanager()
-            Z = texmanager.get_grey(s, size, self.dpi)
-            m,n = Z.shape
-            # TODO: descent of TeX text (I am imitating backend_ps here -JKS)
-            return n, m, 0
+            fontsize = prop.get_size_in_points()
+            w, h, d = texmanager.get_text_width_height_descent(s, fontsize,
+                                                               renderer=self)
+            return w, h, d
 
         if ismath:
             ox, oy, width, height, descent, fonts, used_characters = \
                 self.mathtext_parser.parse(s, self.dpi, prop)
             return width, height, descent
+
+        flags = get_hinting_flag()
         font = self._get_agg_font(prop)
-        font.set_text(s, 0.0, flags=LOAD_FORCE_AUTOHINT)  # the width and height of unrotated string
+        font.set_text(s, 0.0, flags=flags)  # the width and height of unrotated string
         w, h = font.get_width_height()
         d = font.get_descent()
         w /= 64.0  # convert from subpixels
@@ -161,6 +215,7 @@ class RendererAgg(RendererBase):
         d /= 64.0
         return w, h, d
 
+
     def draw_tex(self, gc, x, y, s, prop, angle):
         # todo, handle props, angle, origins
         size = prop.get_size_in_points()
@@ -170,7 +225,7 @@ class RendererAgg(RendererBase):
         im = self.texd.get(key)
         if im is None:
             Z = texmanager.get_grey(s, size, self.dpi)
-            Z = npy.array(Z * 255.0, npy.uint8)
+            Z = np.array(Z * 255.0, np.uint8)
 
         self._renderer.draw_text_image(Z, x, y, angle, gc)
 
@@ -186,15 +241,17 @@ class RendererAgg(RendererBase):
                                      'debug-annoying')
 
         key = hash(prop)
-        font = self._fontd.get(key)
+        font = RendererAgg._fontd.get(key)
 
         if font is None:
             fname = findfont(prop)
-            font = self._fontd.get(fname)
+            font = RendererAgg._fontd.get(fname)
             if font is None:
-                font = FT2Font(str(fname))
-                self._fontd[fname] = font
-            self._fontd[key] = font
+                font = FT2Font(
+                    str(fname),
+                    hinting_factor=rcParams['text.hinting_factor'])
+                RendererAgg._fontd[fname] = font
+            RendererAgg._fontd[key] = font
 
         font.clear()
         size = prop.get_size_in_points()
@@ -221,10 +278,10 @@ class RendererAgg(RendererBase):
                                      'debug-annoying')
         return self._renderer.tostring_argb()
 
-    def buffer_rgba(self,x,y):
+    def buffer_rgba(self):
         if __debug__: verbose.report('RendererAgg.buffer_rgba',
                                      'debug-annoying')
-        return self._renderer.buffer_rgba(x,y)
+        return self._renderer.buffer_rgba()
 
     def clear(self):
         self._renderer.clear()
@@ -235,6 +292,99 @@ class RendererAgg(RendererBase):
         # with the Agg backend
         return True
 
+    def option_scale_image(self):
+        """
+        agg backend support arbitrary scaling of image.
+        """
+        return True
+
+    def restore_region(self, region, bbox=None, xy=None):
+        """
+        Restore the saved region. If bbox (instance of BboxBase, or
+        its extents) is given, only the region specified by the bbox
+        will be restored. *xy* (a tuple of two floasts) optionally
+        specifies the new position (the LLC of the original region,
+        not the LLC of the bbox) where the region will be restored.
+
+        >>> region = renderer.copy_from_bbox()
+        >>> x1, y1, x2, y2 = region.get_extents()
+        >>> renderer.restore_region(region, bbox=(x1+dx, y1, x2, y2),
+                                    xy=(x1-dx, y1))
+
+        """
+        if bbox is not None or xy is not None:
+            if bbox is None:
+                x1, y1, x2, y2 = region.get_extents()
+            elif isinstance(bbox, BboxBase):
+                x1, y1, x2, y2 = bbox.extents
+            else:
+                x1, y1, x2, y2 = bbox
+
+            if xy is None:
+                ox, oy = x1, y1
+            else:
+                ox, oy = xy
+
+            self._renderer.restore_region2(region, x1, y1, x2, y2, ox, oy)
+
+        else:
+            self._renderer.restore_region(region)
+
+    def start_filter(self):
+        """
+        Start filtering. It simply create a new canvas (the old one is saved).
+        """
+        self._filter_renderers.append(self._renderer)
+        self._renderer = _RendererAgg(int(self.width), int(self.height),
+                                      self.dpi)
+        self._update_methods()
+
+    def stop_filter(self, post_processing):
+        """
+        Save the plot in the current canvas as a image and apply
+        the *post_processing* function.
+
+           def post_processing(image, dpi):
+             # ny, nx, depth = image.shape
+             # image (numpy array) has RGBA channels and has a depth of 4.
+             ...
+             # create a new_image (numpy array of 4 channels, size can be
+             # different). The resulting image may have offsets from
+             # lower-left corner of the original image
+             return new_image, offset_x, offset_y
+
+        The saved renderer is restored and the returned image from
+        post_processing is plotted (using draw_image) on it.
+        """
+
+        # WARNING.
+        # For agg_filter to work, the rendere's method need
+        # to overridden in the class. See draw_markers, and draw_path_collections
+
+        from matplotlib._image import fromarray
+
+        width, height = int(self.width), int(self.height)
+
+        buffer, bounds = self._renderer.tostring_rgba_minimized()
+
+        l, b, w, h = bounds
+
+
+        self._renderer = self._filter_renderers.pop()
+        self._update_methods()
+
+        if w > 0 and h > 0:
+            img = np.fromstring(buffer, np.uint8)
+            img, ox, oy = post_processing(img.reshape((h, w, 4)) / 255.,
+                                          self.dpi)
+            image = fromarray(img, 1)
+            image.flipud_out()
+
+            gc = self.new_gc()
+            self._renderer.draw_image(gc,
+                                      l+ox, height - b - h +oy,
+                                      image)
+
 
 def new_figure_manager(num, *args, **kwargs):
     """
@@ -246,7 +396,14 @@ def new_figure_manager(num, *args, **kwargs):
 
     FigureClass = kwargs.pop('FigureClass', Figure)
     thisFig = FigureClass(*args, **kwargs)
-    canvas = FigureCanvasAgg(thisFig)
+    return new_figure_manager_given_figure(num, thisFig)
+
+
+def new_figure_manager_given_figure(num, figure):
+    """
+    Create a new figure manager instance for the given figure.
+    """
+    canvas = FigureCanvasAgg(figure)
     manager = FigureManagerBase(canvas, num)
     return manager
 
@@ -265,9 +422,9 @@ class FigureCanvasAgg(FigureCanvasBase):
         renderer = self.get_renderer()
         return renderer.copy_from_bbox(bbox)
 
-    def restore_region(self, region):
+    def restore_region(self, region, bbox=None, xy=None):
         renderer = self.get_renderer()
-        return renderer.restore_region(region)
+        return renderer.restore_region(region, bbox, xy)
 
     def draw(self):
         """
@@ -276,7 +433,15 @@ class FigureCanvasAgg(FigureCanvasBase):
         if __debug__: verbose.report('FigureCanvasAgg.draw', 'debug-annoying')
 
         self.renderer = self.get_renderer()
-        self.figure.draw(self.renderer)
+        # acquire a lock on the shared font cache
+        RendererAgg.lock.acquire()
+
+        try:
+            self.figure.draw(self.renderer)
+        finally:
+            RendererAgg.lock.release()
+
+
 
     def get_renderer(self):
         l, b, w, h = self.figure.bbox.bounds
@@ -300,13 +465,10 @@ class FigureCanvasAgg(FigureCanvasBase):
                                      'debug-annoying')
         return self.renderer.tostring_argb()
 
-    def buffer_rgba(self,x,y):
+    def buffer_rgba(self):
         if __debug__: verbose.report('FigureCanvasAgg.buffer_rgba',
                                      'debug-annoying')
-        return self.renderer.buffer_rgba(x,y)
-
-    def get_default_filetype(self):
-        return 'png'
+        return self.renderer.buffer_rgba()
 
     def print_raw(self, filename_or_obj, *args, **kwargs):
         FigureCanvasAgg.draw(self)
@@ -314,8 +476,15 @@ class FigureCanvasAgg(FigureCanvasBase):
         original_dpi = renderer.dpi
         renderer.dpi = self.figure.dpi
         if is_string_like(filename_or_obj):
-            filename_or_obj = file(filename_or_obj, 'wb')
-        renderer._renderer.write_rgba(filename_or_obj)
+            filename_or_obj = open(filename_or_obj, 'wb')
+            close = True
+        else:
+            close = False
+        try:
+            renderer._renderer.write_rgba(filename_or_obj)
+        finally:
+            if close:
+                filename_or_obj.close()
         renderer.dpi = original_dpi
     print_rgba = print_raw
 
@@ -325,8 +494,25 @@ class FigureCanvasAgg(FigureCanvasBase):
         original_dpi = renderer.dpi
         renderer.dpi = self.figure.dpi
         if is_string_like(filename_or_obj):
-            filename_or_obj = file(filename_or_obj, 'wb')
-        _png.write_png(renderer._renderer.buffer_rgba(0, 0),
-                       renderer.width, renderer.height,
-                       filename_or_obj, self.figure.dpi)
+            filename_or_obj = open(filename_or_obj, 'wb')
+            close = True
+        else:
+            close = False
+        try:
+            _png.write_png(renderer._renderer.buffer_rgba(),
+                           renderer.width, renderer.height,
+                           filename_or_obj, self.figure.dpi)
+        finally:
+            if close:
+                filename_or_obj.close()
         renderer.dpi = original_dpi
+
+    def print_to_buffer(self):
+        FigureCanvasAgg.draw(self)
+        renderer = self.get_renderer()
+        original_dpi = renderer.dpi
+        renderer.dpi = self.figure.dpi
+        result = (renderer._renderer.buffer_rgba(),
+                  (int(renderer.width), int(renderer.height)))
+        renderer.dpi = original_dpi
+        return result
