@@ -8,7 +8,7 @@ This code is released under the matplotlib license
 
 """
 
-from __future__ import division
+from __future__ import division, print_function
 
 import os, sys, math
 
@@ -20,29 +20,18 @@ import os.path
 import matplotlib
 
 from matplotlib import rcParams, verbose
-from matplotlib.cbook import is_string_like,  enumerate
+from matplotlib.cbook import is_string_like
 from matplotlib.backend_bases import \
      RendererBase, GraphicsContextBase, FigureManagerBase, FigureCanvasBase,\
      NavigationToolbar2, cursors
+from matplotlib.backend_bases import ShowBase
+
+
 from matplotlib.figure import Figure
 from matplotlib._pylab_helpers import Gcf
-from matplotlib.numerix import asarray
-import matplotlib.windowing as windowing
+import matplotlib.backends.windowing as windowing
 from matplotlib.widgets import SubplotTool
 
-
-import thread,time
-
-Fl_running=thread.allocate_lock()
-def Fltk_run_interactive():
-    global Fl_running
-    if Fl_running.acquire(0):
-      while True:
-        Fltk.Fl.check()
-        time.sleep(0.005)
-    else:
-        print "fl loop already running"
-
 # the true dots per inch on the screen; should be display dependent
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
 PIXELS_PER_INCH = 75
@@ -76,35 +65,11 @@ def draw_if_interactive():
         if figManager is not None:
             figManager.canvas.draw()
 
-
-def ishow():
-    """
-    Show all the figures and enter the fltk mainloop in another thread
-    This allows to keep hand in interractive python session
-    Warning: does not work under windows
-    This should be the last line of your script
-    """
-    for manager in Gcf.get_all_fig_managers():
-        manager.show()
-    if show._needmain:
-        thread.start_new_thread(Fltk_run_interactive,())
-    show._needmain = False
-
-def show():
-    """
-    Show all the figures and enter the fltk mainloop
-
-    This should be the last line of your script
-    """
-    for manager in Gcf.get_all_fig_managers():
-        manager.show()
-    #mainloop, if an fltk program exist no need to call that
-    #threaded (and interractive) version
-    if show._needmain:
+class Show(ShowBase):
+    def mainloop(self):
         Fltk.Fl.run()
-        show._needmain = False
 
-show._needmain = True
+show = Show()
 
 
 def new_figure_manager(num, *args, **kwargs):
@@ -113,9 +78,17 @@ def new_figure_manager(num, *args, **kwargs):
     """
     FigureClass = kwargs.pop('FigureClass', Figure)
     figure = FigureClass(*args, **kwargs)
+    return new_figure_manager_given_figure(num, figure)
+
+
+def new_figure_manager_given_figure(num, figure):
+    """
+    Create a new figure manager instance for the given figure.
+    """
     window = Fltk.Fl_Double_Window(10,10,30,30)
     canvas = FigureCanvasFltkAgg(figure)
     window.end()
+    #Fltk.Fl.visual(Fltk.FL_DOUBLE)
     window.show()
     window.make_current()
     figManager = FigureManagerFltkAgg(canvas, num, window)
@@ -141,24 +114,24 @@ class FltkCanvas(Fltk.Fl_Widget):
             self._oldsize =newsize
             self._source.resize(newsize)
             self._source.draw()
-        t1,t2,w,h = self._source.figure.bbox.get_bounds()
-        Fltk.fl_draw_image(self._source.buffer_rgba(0,0),0,0,int(w),int(h),4,0)
+        t1,t2,w,h = self._source.figure.bbox.bounds
+        Fltk.fl_draw_image(self._source.buffer_rgba(),0,0,int(w),int(h),4,0)
         self.redraw()
 
     def blit(self,bbox=None):
         if bbox is None:
-            t1,t2,w,h = self._source.figure.bbox.get_bounds()
+            t1,t2,w,h = self._source.figure.bbox.bounds
         else:
-           t1o,t2o,wo,ho = self._source.figure.bbox.get_bounds()
-           t1,t2,w,h = bbox.get_bounds()
+           t1o,t2o,wo,ho = self._source.figure.bbox.bounds
+           t1,t2,w,h = bbox.bounds
         x,y=int(t1),int(t2)
-        Fltk.fl_draw_image(self._source.buffer_rgba(x,y),x,y,int(w),int(h),4,int(wo)*4)
+        Fltk.fl_draw_image(self._source.buffer_rgba(),x,y,int(w),int(h),4,int(wo)*4)
         #self.redraw()
 
     def handle(self, event):
         x=Fltk.Fl.event_x()
         y=Fltk.Fl.event_y()
-        yf=self._source.figure.bbox.height() - y
+        yf=self._source.figure.bbox.height - y
         if event == Fltk.FL_FOCUS or event == Fltk.FL_UNFOCUS:
             return 1
         elif event == Fltk.FL_KEYDOWN:
@@ -170,6 +143,8 @@ class FltkCanvas(Fltk.Fl_Widget):
                     self._key=special_key[ikey]
                 except:
                     self._key=None
+
+            # TODO: Handle ctrl, alt, super modifiers.
             FigureCanvasBase.key_press_event(self._source, self._key)
             return 1
         elif event == Fltk.FL_KEYUP:
@@ -189,8 +164,8 @@ class FltkCanvas(Fltk.Fl_Widget):
             if self._draw_overlay:
                 self._oldx=x
                 self._oldy=y
-            if Fltk.Fl.event_clicks():
-                FigureCanvasBase.button_press_event(self._source, x, yf, self._button)
+            if Fltk.Fl.event_clicks():  # according to docs, event_clicks() returns nonzero if this is a double click.
+                FigureCanvasBase.button_press_event(self._source, x, yf, self._button, dblclick=True)
                 return 1
             else:
                 FigureCanvasBase.button_press_event(self._source, x, yf, self._button)
@@ -223,7 +198,7 @@ class FltkCanvas(Fltk.Fl_Widget):
 class FigureCanvasFltkAgg(FigureCanvasAgg):
     def __init__(self, figure):
         FigureCanvasAgg.__init__(self,figure)
-        t1,t2,w,h = self.figure.bbox.get_bounds()
+        t1,t2,w,h = self.figure.bbox.bounds
         w, h = int(w), int(h)
         self.canvas=FltkCanvas(0, 0, w, h, "canvas",self)
         #self.draw()
@@ -231,7 +206,7 @@ class FigureCanvasFltkAgg(FigureCanvasAgg):
     def resize(self,size):
         w, h = size
         # compute desired figure size in inches
-        dpival = self.figure.dpi.get()
+        dpival = self.figure.dpi
         winch = w/dpival
         hinch = h/dpival
         self.figure.set_size_inches(winch,hinch)
@@ -243,22 +218,22 @@ class FigureCanvasFltkAgg(FigureCanvasAgg):
     def blit(self,bbox):
         self.canvas.blit(bbox)
 
-
     show = draw
 
-    def print_figure(self, filename, dpi=None, facecolor='w', edgecolor='w',
-                     orientation='portrait', **kwargs):
-        if dpi is None: dpi = matplotlib.rcParams['savefig.dpi']
-        agg = self.switch_backends(FigureCanvasAgg)
-        agg.print_figure(filename, dpi, facecolor, edgecolor, orientation,
-                         **kwargs)
-        self.figure.set_canvas(self)
-
     def widget(self):
         return self.canvas
 
-def destroy_figure(ptr,figman):
+    def start_event_loop(self,timeout):
+        FigureCanvasBase.start_event_loop_default(self,timeout)
+    start_event_loop.__doc__=FigureCanvasBase.start_event_loop_default.__doc__
+
+    def stop_event_loop(self):
+        FigureCanvasBase.stop_event_loop_default(self)
+    stop_event_loop.__doc__=FigureCanvasBase.stop_event_loop_default.__doc__
+
+def destroy_figure(ptr, figman):
     figman.window.hide()
+    Fltk.Fl.wait(0)         # This is needed to make the last figure vanish.
     Gcf.destroy(figman._num)
 
 class FigureManagerFltkAgg(FigureManagerBase):
@@ -273,7 +248,7 @@ class FigureManagerFltkAgg(FigureManagerBase):
     def __init__(self, canvas, num, window):
         FigureManagerBase.__init__(self, canvas, num)
         #Fltk container window
-        t1,t2,w,h = canvas.figure.bbox.get_bounds()
+        t1,t2,w,h = canvas.figure.bbox.bounds
         w, h = int(w), int(h)
         self.window = window
         self.window.size(w,h+30)
@@ -309,6 +284,14 @@ class FigureManagerFltkAgg(FigureManagerBase):
         self.canvas.draw()
         self.window.redraw()
 
+    def destroy(self):
+        self.window.hide()
+        Fltk.Fl.wait(0)         # This is needed to make the last figure vanish.
+        Gcf.destroy(self._num)
+
+    def set_window_title(self, title):
+        self.window_title=title
+        self.window.label(title)
 
 class AxisMenu:
     def __init__(self, toolbar):
@@ -404,7 +387,7 @@ class NavigationToolbar:
     """
 
     def __init__(self, canvas, figman):
-        #xmin, xmax = canvas.figure.bbox.intervalx().get_bounds()
+        #xmin, xmax = canvas.figure.bbox.intervalx
         #height, width = 50, xmax-xmin
         self.canvas = canvas
         self.figman = figman
@@ -412,7 +395,7 @@ class NavigationToolbar:
         Fltk.Fl_File_Icon.load_system_icons()
         self._fc = Fltk.Fl_File_Chooser( ".", "*", Fltk.Fl_File_Chooser.CREATE, "Save Figure" )
         self._fc.hide()
-        t1,t2,w,h = canvas.figure.bbox.get_bounds()
+        t1,t2,w,h = canvas.figure.bbox.bounds
         w, h = int(w), int(h)
         self._group = Fltk.Fl_Pack(0,h+2,1000,26)
         self._group.type(Fltk.FL_HORIZONTAL)
@@ -494,7 +477,23 @@ def zoom(ptr, arg):
 
 
 def save_figure(ptr,base):
+    filetypes = base.canvas.get_supported_filetypes()
+    default_filetype = base.canvas.get_default_filetype()
+    sorted_filetypes = filetypes.items()
+    sorted_filetypes.sort()
+
+    selected_filter = 0
+    filters = []
+    for i, (ext, name) in enumerate(sorted_filetypes):
+        filter = '%s (*.%s)' % (name, ext)
+        filters.append(filter)
+        if ext == default_filetype:
+            selected_filter = i
+    filters = '\t'.join(filters)
+
     file_chooser=base._fc
+    file_chooser.filter(filters)
+    file_chooser.filter_value(selected_filter)
     file_chooser.show()
     while file_chooser.visible() :
         Fltk.Fl.wait()
@@ -508,10 +507,11 @@ def save_figure(ptr,base):
     #start from last directory
     lastDir = os.path.dirname(fname)
     file_chooser.directory(lastDir)
+    format = sorted_filetypes[file_chooser.filter_value()][0]
 
     try:
-        base.canvas.print_figure(fname)
-    except IOError, msg:
+        base.canvas.print_figure(fname, format=format)
+    except IOError as msg:
         err = '\n'.join(map(str, msg))
         msg = 'Failed to save %s: Error msg was\n\n%s' % (
             fname, err)
@@ -581,7 +581,7 @@ class NavigationToolbar2FltkAgg(NavigationToolbar2):
         Fltk.Fl_File_Icon.load_system_icons()
         self._fc = Fltk.Fl_File_Chooser( ".", "*", Fltk.Fl_File_Chooser.CREATE, "Save Figure" )
         self._fc.hide()
-        t1,t2,w,h = self.canvas.figure.bbox.get_bounds()
+        t1,t2,w,h = self.canvas.figure.bbox.bounds
         w, h = int(w), int(h)
         self._group = Fltk.Fl_Pack(0,h+2,1000,26)
         self._group.type(Fltk.FL_HORIZONTAL)
