@@ -1,8 +1,13 @@
-from __future__ import division
+from __future__ import division, print_function
 
-import os, sys
+import os, sys, warnings
 def fn_name(): return sys._getframe(1).f_code.co_name
 
+if sys.version_info[0] >= 3:
+    warnings.warn(
+        "The gtk* backends have not been tested with Python 3.x",
+        ImportWarning)
+
 try:
     import gobject
     import gtk; gdk = gtk.gdk
@@ -10,18 +15,21 @@ try:
 except ImportError:
     raise ImportError("Gtk* backend requires pygtk to be installed.")
 
-pygtk_version_required = (2,2,0)
+pygtk_version_required = (2,4,0)
 if gtk.pygtk_version < pygtk_version_required:
     raise ImportError ("PyGTK %d.%d.%d is installed\n"
                       "PyGTK %d.%d.%d or later is required"
                       % (gtk.pygtk_version + pygtk_version_required))
 del pygtk_version_required
 
+_new_tooltip_api =  (gtk.pygtk_version[1] >= 12)
+
 import matplotlib
-from matplotlib import verbose
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
-     FigureManagerBase, FigureCanvasBase, NavigationToolbar2, cursors
+     FigureManagerBase, FigureCanvasBase, NavigationToolbar2, cursors, TimerBase
+from matplotlib.backend_bases import ShowBase
+
 from matplotlib.backends.backend_gdk import RendererGDK, FigureCanvasGDK
 from matplotlib.cbook import is_string_like, is_writable_file_like
 from matplotlib.colors import colorConverter
@@ -29,7 +37,10 @@ from matplotlib.figure import Figure
 from matplotlib.widgets import SubplotTool
 
 from matplotlib import lines
+from matplotlib import markers
 from matplotlib import cbook
+from matplotlib import verbose
+from matplotlib import rcParams
 
 backend_version = "%d.%d.%d" % gtk.pygtk_version
 
@@ -40,6 +51,9 @@ _debug = False
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
 PIXELS_PER_INCH = 96
 
+# Hide the benign warning that it can't stat a file that doesn't
+warnings.filterwarnings('ignore', '.*Unable to retrieve the file info for.*', gtk.Warning)
+
 cursord = {
     cursors.MOVE          : gdk.Cursor(gdk.FLEUR),
     cursors.HAND          : gdk.Cursor(gdk.HAND2),
@@ -60,20 +74,15 @@ def draw_if_interactive():
     if matplotlib.is_interactive():
         figManager =  Gcf.get_active()
         if figManager is not None:
-            figManager.canvas.draw()
+            figManager.canvas.draw_idle()
 
 
-def show(mainloop=True):
-    """
-    Show all the figures and enter the gtk main loop
-    This should be the last line of your script
-    """
-    for manager in Gcf.get_all_fig_managers():
-        manager.window.show()
+class Show(ShowBase):
+    def mainloop(self):
+        if gtk.main_level() == 0:
+            gtk.main()
 
-    if mainloop and gtk.main_level() == 0 and \
-                            len(Gcf.get_all_fig_managers())>0:
-        gtk.main()
+show = Show()
 
 def new_figure_manager(num, *args, **kwargs):
     """
@@ -81,13 +90,60 @@ def new_figure_manager(num, *args, **kwargs):
     """
     FigureClass = kwargs.pop('FigureClass', Figure)
     thisFig = FigureClass(*args, **kwargs)
-    canvas = FigureCanvasGTK(thisFig)
+    return new_figure_manager_given_figure(num, thisFig)
+
+
+def new_figure_manager_given_figure(num, figure):
+    """
+    Create a new figure manager instance for the given figure.
+    """
+    canvas = FigureCanvasGTK(figure)
     manager = FigureManagerGTK(canvas, num)
-    # equals:
-    #manager = FigureManagerGTK(FigureCanvasGTK(Figure(*args, **kwargs), num)
     return manager
 
 
+class TimerGTK(TimerBase):
+    '''
+    Subclass of :class:`backend_bases.TimerBase` that uses GTK for timer events.
+
+    Attributes:
+    * interval: The time between timer events in milliseconds. Default
+        is 1000 ms.
+    * single_shot: Boolean flag indicating whether this timer should
+        operate as single shot (run once and then stop). Defaults to False.
+    * callbacks: Stores list of (func, args) tuples that will be called
+        upon timer events. This list can be manipulated directly, or the
+        functions add_callback and remove_callback can be used.
+    '''
+    def _timer_start(self):
+        # Need to stop it, otherwise we potentially leak a timer id that will
+        # never be stopped.
+        self._timer_stop()
+        self._timer = gobject.timeout_add(self._interval, self._on_timer)
+
+    def _timer_stop(self):
+        if self._timer is not None:
+            gobject.source_remove(self._timer)
+            self._timer = None
+
+    def _timer_set_interval(self):
+        # Only stop and restart it if the timer has already been started
+        if self._timer is not None:
+            self._timer_stop()
+            self._timer_start()
+
+    def _on_timer(self):
+        TimerBase._on_timer(self)
+
+        # Gtk timeout_add() requires that the callback returns True if it
+        # is to be called again.
+        if len(self.callbacks) > 0 and not self._single:
+            return True
+        else:
+            self._timer = None
+            return False
+
+
 class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
     keyvald = {65507 : 'control',
                65505 : 'shift',
@@ -138,6 +194,10 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                65455 : '/',
                65439 : 'dec',
                65421 : 'enter',
+               65511 : 'super',
+               65512 : 'super',
+               65406 : 'alt',
+               65289 : 'tab',
                }
 
     # Setting this as a static constant prevents
@@ -153,7 +213,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                   gdk.POINTER_MOTION_HINT_MASK)
 
     def __init__(self, figure):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         FigureCanvasBase.__init__(self, figure)
         gtk.DrawingArea.__init__(self)
 
@@ -182,14 +242,17 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
 
         self._idle_event_id = gobject.idle_add(self.idle_event)
 
+        self.last_downclick = {}
+
     def destroy(self):
         #gtk.DrawingArea.destroy(self)
+        self.close_event()
         gobject.source_remove(self._idle_event_id)
         if self._idle_draw_id != 0:
             gobject.source_remove(self._idle_draw_id)
 
     def scroll_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
@@ -197,41 +260,57 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
             step = 1
         else:
             step = -1
-        FigureCanvasBase.scroll_event(self, x, y, step)
+        FigureCanvasBase.scroll_event(self, x, y, step, guiEvent=event)
         return False  # finish event propagation?
 
     def button_press_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
-        FigureCanvasBase.button_press_event(self, x, y, event.button)
+        dblclick = (event.type == gdk._2BUTTON_PRESS)
+        if not dblclick:
+            # GTK is the only backend that generates a DOWN-UP-DOWN-DBLCLICK-UP  event
+            # sequence for a double click.  All other backends have a DOWN-UP-DBLCLICK-UP
+            # sequence.  In order to provide consistency to matplotlib users, we will
+            # eat the extra DOWN event in the case that we detect it is part of a double
+            # click.
+            # first, get the double click time in milliseconds.
+            current_time  = event.get_time()
+            last_time     = self.last_downclick.get(event.button,0)
+            dblclick_time = gtk.settings_get_for_screen(gdk.screen_get_default()).get_property('gtk-double-click-time')
+            delta_time    = current_time-last_time
+            if delta_time < dblclick_time:
+                del self.last_downclick[event.button] # we do not want to eat more than one event.
+                return False                          # eat.
+            self.last_downclick[event.button] = current_time
+        FigureCanvasBase.button_press_event(self, x, y, event.button, dblclick=dblclick, guiEvent=event)
         return False  # finish event propagation?
 
     def button_release_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - event.y
-        FigureCanvasBase.button_release_event(self, x, y, event.button)
+        FigureCanvasBase.button_release_event(self, x, y, event.button, guiEvent=event)
         return False  # finish event propagation?
 
     def key_press_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         key = self._get_key(event)
-        if _debug: print "hit", key
-        FigureCanvasBase.key_press_event(self, key)
+        if _debug: print("hit", key)
+        FigureCanvasBase.key_press_event(self, key, guiEvent=event)
         return False  # finish event propagation?
 
     def key_release_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         key = self._get_key(event)
-        if _debug: print "release", key
-        FigureCanvasBase.key_release_event(self, key)
+        if _debug: print("release", key)
+        FigureCanvasBase.key_release_event(self, key, guiEvent=event)
         return False  # finish event propagation?
 
     def motion_notify_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         if event.is_hint:
             x, y, state = event.window.get_pointer()
         else:
@@ -239,30 +318,35 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
 
         # flipy so y=0 is bottom of canvas
         y = self.allocation.height - y
-        FigureCanvasBase.motion_notify_event(self, x, y)
+        FigureCanvasBase.motion_notify_event(self, x, y, guiEvent=event)
         return False  # finish event propagation?
 
     def leave_notify_event(self, widget, event):
         FigureCanvasBase.leave_notify_event(self, event)
 
     def enter_notify_event(self, widget, event):
-        FigureCanvasBase.enter_notify_event(self, event)
+        x, y, state = event.window.get_pointer()
+        FigureCanvasBase.enter_notify_event(self, event, xy=(x,y))
 
     def _get_key(self, event):
         if event.keyval in self.keyvald:
             key = self.keyvald[event.keyval]
-        elif event.keyval <256:
+        elif event.keyval < 256:
             key = chr(event.keyval)
         else:
             key = None
 
-        ctrl  = event.state & gdk.CONTROL_MASK
-        shift = event.state & gdk.SHIFT_MASK
+        for key_mask, prefix in (
+                                 [gdk.MOD4_MASK, 'super'],
+                                 [gdk.MOD1_MASK, 'alt'],
+                                 [gdk.CONTROL_MASK, 'ctrl'],):
+            if event.state & key_mask:
+                key = '{0}+{1}'.format(prefix, key)
+
         return key
 
-
     def configure_event(self, widget, event):
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
         if widget.window is None:
             return
         w, h = event.width, event.height
@@ -313,7 +397,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         Make sure _._pixmap is at least width, height,
         create new pixmap if necessary
         """
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
 
         create_pixmap = False
         if width > self._pixmap_width:
@@ -343,7 +427,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
     def expose_event(self, widget, event):
         """Expose_event for all GTK backends. Should not be overridden.
         """
-        if _debug: print 'FigureCanvasGTK.%s' % fn_name()
+        if _debug: print('FigureCanvasGTK.%s' % fn_name())
 
         if GTK_WIDGET_DRAWABLE(self):
             if self._need_redraw:
@@ -389,7 +473,7 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
         if is_string_like(filename):
             try:
                 pixbuf.save(filename, format)
-            except gobject.GError, exc:
+            except gobject.GError as exc:
                 error_msg_gtk('Save figure failure:\n%s' % (exc,), parent=self)
         elif is_writable_file_like(filename):
             if hasattr(pixbuf, 'save_to_callback'):
@@ -397,16 +481,28 @@ class FigureCanvasGTK (gtk.DrawingArea, FigureCanvasBase):
                     data.write(buf)
                 try:
                     pixbuf.save_to_callback(save_callback, format, user_data=filename)
-                except gobject.GError, exc:
+                except gobject.GError as exc:
                     error_msg_gtk('Save figure failure:\n%s' % (exc,), parent=self)
             else:
                 raise ValueError("Saving to a Python file-like object is only supported by PyGTK >= 2.8")
         else:
             raise ValueError("filename must be a path or a file-like object")
 
-    def get_default_filetype(self):
-        return 'png'
+    def new_timer(self, *args, **kwargs):
+        """
+        Creates a new backend-specific subclass of :class:`backend_bases.Timer`.
+        This is useful for getting periodic events through the backend's native
+        event loop. Implemented only for backends with GUIs.
 
+        optional arguments:
+
+        *interval*
+          Timer interval in milliseconds
+        *callbacks*
+          Sequence of (func, args, kwargs) where func(*args, **kwargs) will
+          be executed by the timer every *interval*.
+        """
+        return TimerGTK(*args, **kwargs)
 
     def flush_events(self):
         gtk.gdk.threads_enter()
@@ -434,11 +530,20 @@ class FigureManagerGTK(FigureManagerBase):
     window      : The gtk.Window   (gtk only)
     """
     def __init__(self, canvas, num):
-        if _debug: print 'FigureManagerGTK.%s' % fn_name()
+        if _debug: print('FigureManagerGTK.%s' % fn_name())
         FigureManagerBase.__init__(self, canvas, num)
 
         self.window = gtk.Window()
-        self.window.set_title("Figure %d" % num)
+        self.set_window_title("Figure %d" % num)
+        if (window_icon):
+            try:
+                self.window.set_icon_from_file(window_icon)
+            except:
+                # some versions of gtk throw a glib.GError but not
+                # all, so I am not sure how to catch it.  I am unhappy
+                # diong a blanket catch here, but an not sure what a
+                # better way is - JDH
+                verbose.report('Could not load matplotlib icon: %s' % sys.exc_info()[1])
 
         self.vbox = gtk.VBox()
         self.window.add(self.vbox)
@@ -446,9 +551,6 @@ class FigureManagerGTK(FigureManagerBase):
 
         self.canvas.show()
 
-        # attach a show method to the figure  for pylab ease of use
-        self.canvas.figure.show = lambda *args: self.window.show()
-
         self.vbox.pack_start(self.canvas, True, True)
 
         self.toolbar = self._get_toolbar(canvas)
@@ -480,12 +582,16 @@ class FigureManagerGTK(FigureManagerBase):
         self.canvas.grab_focus()
 
     def destroy(self, *args):
-        if _debug: print 'FigureManagerGTK.%s' % fn_name()
-        self.vbox.destroy()
-        self.window.destroy()
-        self.canvas.destroy()
-        self.toolbar.destroy()
-        self.__dict__.clear()
+        if _debug: print('FigureManagerGTK.%s' % fn_name())
+        if hasattr(self, 'toolbar') and self.toolbar is not None:
+            self.toolbar.destroy()
+        if hasattr(self, 'vbox'):
+            self.vbox.destroy()
+        if hasattr(self, 'window'):
+            self.window.destroy()
+        if hasattr(self, 'canvas'):
+            self.canvas.destroy()
+        self.__dict__.clear()   #Is this needed? Other backends don't have it.
 
         if Gcf.get_num_fig_managers()==0 and \
                not matplotlib.is_interactive() and \
@@ -496,7 +602,7 @@ class FigureManagerGTK(FigureManagerBase):
         # show the figure window
         self.window.show()
 
-    def full_screen_toggle (self):
+    def full_screen_toggle(self):
         self._full_screen_flag = not self._full_screen_flag
         if self._full_screen_flag:
             self.window.fullscreen()
@@ -508,14 +614,17 @@ class FigureManagerGTK(FigureManagerBase):
     def _get_toolbar(self, canvas):
         # must be inited after the window, drawingArea and figure
         # attrs are set
-        if matplotlib.rcParams['toolbar'] == 'classic':
+        if rcParams['toolbar'] == 'classic':
             toolbar = NavigationToolbar (canvas, self.window)
-        elif matplotlib.rcParams['toolbar'] == 'toolbar2':
+        elif rcParams['toolbar'] == 'toolbar2':
             toolbar = NavigationToolbar2GTK (canvas, self.window)
         else:
             toolbar = None
         return toolbar
 
+    def get_window_title(self):
+        return self.window.get_title()
+
     def set_window_title(self, title):
         self.window.set_title(title)
 
@@ -528,34 +637,19 @@ class FigureManagerGTK(FigureManagerBase):
 
 
 class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
-    # list of toolitems to add to the toolbar, format is:
-    # text, tooltip_text, image_file, callback(str)
-    toolitems = (
-        ('Home', 'Reset original view', 'home.png', 'home'),
-        ('Back', 'Back to  previous view','back.png', 'back'),
-        ('Forward', 'Forward to next view','forward.png', 'forward'),
-        ('Pan', 'Pan axes with left mouse, zoom with right', 'move.png','pan'),
-        ('Zoom', 'Zoom to rectangle','zoom_to_rect.png', 'zoom'),
-        (None, None, None, None),
-        ('Subplots', 'Configure subplots','subplots.png', 'configure_subplots'),
-        ('Save', 'Save the figure','filesave.png', 'save_figure'),
-        )
-
     def __init__(self, canvas, window):
         self.win = window
         gtk.Toolbar.__init__(self)
         NavigationToolbar2.__init__(self, canvas)
-        self._idle_draw_id = 0
 
     def set_message(self, s):
-        if self._idle_draw_id == 0:
-            self.message.set_label(s)
+        self.message.set_label(s)
 
     def set_cursor(self, cursor):
         self.canvas.window.set_cursor(cursord[cursor])
 
     def release(self, event):
-        try: del self._imageBack
+        try: del self._pixmapBack
         except AttributeError: pass
 
     def dynamic_update(self):
@@ -577,8 +671,9 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         w = abs(x1 - x0)
         h = abs(y1 - y0)
 
-        rect = [int(val)for val in min(x0,x1), min(y0, y1), w, h]
-        try: lastrect, imageBack = self._imageBack
+        rect = [int(val)for val in (min(x0,x1), min(y0, y1), w, h)]
+        try:
+            lastrect, pixmapBack = self._pixmapBack
         except AttributeError:
             #snap image back
             if event.inaxes is None:
@@ -588,67 +683,37 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
             l,b,w,h = [int(val) for val in ax.bbox.bounds]
             b = int(height)-(b+h)
             axrect = l,b,w,h
-            self._imageBack = axrect, drawable.get_image(*axrect)
-            drawable.draw_rectangle(gc, False, *rect)
-            self._idle_draw_id = 0
+            self._pixmapBack = axrect, gtk.gdk.Pixmap(drawable, w, h)
+            self._pixmapBack[1].draw_drawable(gc, drawable, l, b, 0, 0, w, h)
         else:
-            def idle_draw(*args):
-                drawable.draw_image(gc, imageBack, 0, 0, *lastrect)
-                drawable.draw_rectangle(gc, False, *rect)
-                self._idle_draw_id = 0
-                return False
-            if self._idle_draw_id == 0:
-                self._idle_draw_id = gobject.idle_add(idle_draw)
+            drawable.draw_drawable(gc, pixmapBack, 0, 0, *lastrect)
+        drawable.draw_rectangle(gc, False, *rect)
 
 
     def _init_toolbar(self):
         self.set_style(gtk.TOOLBAR_ICONS)
+        self._init_toolbar2_4()
 
-        if gtk.pygtk_version >= (2,4,0):
-            self._init_toolbar2_4()
-        else:
-            self._init_toolbar2_2()
-
-
-    def _init_toolbar2_2(self):
-        basedir = os.path.join(matplotlib.rcParams['datapath'],'images')
-
-        for text, tooltip_text, image_file, callback in self.toolitems:
-            if text is None:
-                 self.append_space()
-                 continue
-
-            fname = os.path.join(basedir, image_file)
-            image = gtk.Image()
-            image.set_from_file(fname)
-            w = self.append_item(text,
-                                 tooltip_text,
-                                 'Private',
-                                 image,
-                                 getattr(self, callback)
-                                 )
-
-        self.append_space()
-
-        self.message = gtk.Label()
-        self.append_widget(self.message, None, None)
-        self.message.show()
 
     def _init_toolbar2_4(self):
-        basedir = os.path.join(matplotlib.rcParams['datapath'],'images')
-        self.tooltips = gtk.Tooltips()
+        basedir = os.path.join(rcParams['datapath'],'images')
+        if not _new_tooltip_api:
+            self.tooltips = gtk.Tooltips()
 
         for text, tooltip_text, image_file, callback in self.toolitems:
             if text is None:
                 self.insert( gtk.SeparatorToolItem(), -1 )
                 continue
-            fname = os.path.join(basedir, image_file)
+            fname = os.path.join(basedir, image_file + '.png')
             image = gtk.Image()
             image.set_from_file(fname)
             tbutton = gtk.ToolButton(image, text)
             self.insert(tbutton, -1)
             tbutton.connect('clicked', getattr(self, callback))
-            tbutton.set_tooltip(self.tooltips, tooltip_text, 'Private')
+            if _new_tooltip_api:
+                tbutton.set_tooltip_text(tooltip_text)
+            else:
+                tbutton.set_tooltip(self.tooltips, tooltip_text, 'Private')
 
         toolitem = gtk.SeparatorToolItem()
         self.insert(toolitem, -1)
@@ -665,22 +730,20 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         self.show_all()
 
     def get_filechooser(self):
-        if gtk.pygtk_version >= (2,4,0):
-            return FileChooserDialog(
-                title='Save the figure',
-                parent=self.win,
-                filetypes=self.canvas.get_supported_filetypes(),
-                default_filetype=self.canvas.get_default_filetype())
-        else:
-            return FileSelection(title='Save the figure',
-                                 parent=self.win,)
+        fc = FileChooserDialog(
+            title='Save the figure',
+            parent=self.win,
+            filetypes=self.canvas.get_supported_filetypes(),
+            default_filetype=self.canvas.get_default_filetype())
+        fc.set_current_name(self.canvas.get_default_filename())
+        return fc
 
-    def save_figure(self, button):
+    def save_figure(self, *args):
         fname, format = self.get_filechooser().get_filename_from_user()
         if fname:
             try:
                 self.canvas.print_figure(fname, format=format)
-            except Exception, e:
+            except Exception as e:
                 error_msg_gtk(str(e), parent=self)
 
     def configure_subplots(self, button):
@@ -694,6 +757,12 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
 
 
         window = gtk.Window()
+        if (window_icon):
+            try: window.set_icon_from_file(window_icon)
+            except:
+                # we presumably already logged a message on the
+                # failure of the main plot, don't keep reporting
+                pass
         window.set_title("Subplot Configuration Tool")
         window.set_default_size(w, h)
         vbox = gtk.VBox()
@@ -763,26 +832,21 @@ class NavigationToolbar(gtk.Toolbar):
 
         self.set_style(gtk.TOOLBAR_ICONS)
 
-        if gtk.pygtk_version >= (2,4,0):
-            self._create_toolitems_2_4()
-            self.update = self._update_2_4
-            self.fileselect = FileChooserDialog(
-                title='Save the figure',
-                parent=self.win,
-                filetypes=self.canvas.get_supported_filetypes(),
-                default_filetype=self.canvas.get_default_filetype())
-        else:
-            self._create_toolitems_2_2()
-            self.update = self._update_2_2
-            self.fileselect = FileSelection(title='Save the figure',
-                                            parent=self.win)
+        self._create_toolitems_2_4()
+        self.update = self._update_2_4
+        self.fileselect = FileChooserDialog(
+            title='Save the figure',
+            parent=self.win,
+            filetypes=self.canvas.get_supported_filetypes(),
+            default_filetype=self.canvas.get_default_filetype())
         self.show_all()
         self.update()
 
     def _create_toolitems_2_4(self):
         # use the GTK+ 2.4 GtkToolbar API
         iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
-        self.tooltips = gtk.Tooltips()
+        if not _new_tooltip_api:
+            self.tooltips = gtk.Tooltips()
 
         for text, tooltip_text, image_num, callback, callback_arg, scroll \
                 in self.toolitems:
@@ -800,15 +864,22 @@ class NavigationToolbar(gtk.Toolbar):
                 tbutton.connect('clicked', getattr(self, callback))
             if scroll:
                 tbutton.connect('scroll_event', getattr(self, callback))
-            tbutton.set_tooltip(self.tooltips, tooltip_text, 'Private')
+            if _new_tooltip_api:
+                tbutton.set_tooltip_text(tooltip_text)
+            else:
+                tbutton.set_tooltip(self.tooltips, tooltip_text, 'Private')
 
         # Axes toolitem, is empty at start, update() adds a menu if >=2 axes
         self.axes_toolitem = gtk.ToolItem()
         self.insert(self.axes_toolitem, 0)
-        self.axes_toolitem.set_tooltip (
-            self.tooltips,
-            tip_text='Select axes that controls affect',
-            tip_private = 'Private')
+        if _new_tooltip_api:
+            self.axes_toolitem.set_tooltip_text(
+                                'Select axes that controls affect')
+        else:
+            self.axes_toolitem.set_tooltip (
+                self.tooltips,
+                tip_text='Select axes that controls affect',
+                tip_private = 'Private')
 
         align = gtk.Alignment (xalign=0.5, yalign=0.5, xscale=0.0, yscale=0.0)
         self.axes_toolitem.add(align)
@@ -855,46 +926,6 @@ class NavigationToolbar(gtk.Toolbar):
         self.set_active(range(len(self._axes)))
 
 
-    def _create_toolitems_2_2(self):
-        # use the GTK+ 2.2 (and lower) GtkToolbar API
-        iconSize = gtk.ICON_SIZE_SMALL_TOOLBAR
-
-        for text, tooltip_text, image_num, callback, callback_arg, scroll \
-                in self.toolitems:
-            if text is None:
-                self.append_space()
-                continue
-            image = gtk.Image()
-            image.set_from_stock(image_num, iconSize)
-            item = self.append_item(text, tooltip_text, 'Private', image,
-                                    getattr(self, callback), callback_arg)
-            if scroll:
-                item.connect("scroll_event", getattr(self, callback))
-
-        self.omenu = gtk.OptionMenu()
-        self.omenu.set_border_width(3)
-        self.insert_widget(
-            self.omenu,
-            'Select axes that controls affect',
-            'Private', 0)
-
-
-    def _update_2_2(self):
-        # for GTK+ 2.2 and lower
-        # called by __init__() and FigureManagerGTK
-
-        self._axes = self.canvas.figure.axes
-
-        if len(self._axes) >= 2:
-            # set up the axis menu
-            self.omenu.set_menu( self._make_axis_menu() )
-            self.omenu.show_all()
-        else:
-            self.omenu.hide()
-
-        self.set_active(range(len(self._axes)))
-
-
     def _make_axis_menu(self):
         # called by self._update*()
 
@@ -964,130 +995,96 @@ class NavigationToolbar(gtk.Toolbar):
         return True
 
     def get_filechooser(self):
-        if gtk.pygtk_version >= (2,4,0):
-            return FileChooserDialog(
-                title='Save the figure',
-                parent=self.win,
-                filetypes=self.canvas.get_supported_filetypes(),
-                default_filetype=self.canvas.get_default_filetype())
-        else:
-            return FileSelection(title='Save the figure',
-                                 parent=self.win)
+        return FileChooserDialog(
+            title='Save the figure',
+            parent=self.win,
+            filetypes=self.canvas.get_supported_filetypes(),
+            default_filetype=self.canvas.get_default_filetype())
 
-    def save_figure(self, button):
+    def save_figure(self, *args):
         fname, format = self.get_filechooser().get_filename_from_user()
         if fname:
             try:
                 self.canvas.print_figure(fname, format=format)
-            except Exception, e:
+            except Exception as e:
                 error_msg_gtk(str(e), parent=self)
 
 
-if gtk.pygtk_version >= (2,4,0):
-    class FileChooserDialog(gtk.FileChooserDialog):
-        """GTK+ 2.4 file selector which remembers the last file/directory
-        selected and presents the user with a menu of supported image formats
-        """
-        def __init__ (self,
-                      title   = 'Save file',
-                      parent  = None,
-                      action  = gtk.FILE_CHOOSER_ACTION_SAVE,
-                      buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                 gtk.STOCK_SAVE,   gtk.RESPONSE_OK),
-                      path    = None,
-                      filetypes = [],
-                      default_filetype = None
-                      ):
-            super (FileChooserDialog, self).__init__ (title, parent, action,
-                                                      buttons)
-            self.set_default_response (gtk.RESPONSE_OK)
+class FileChooserDialog(gtk.FileChooserDialog):
+    """GTK+ 2.4 file selector which remembers the last file/directory
+    selected and presents the user with a menu of supported image formats
+    """
+    def __init__ (self,
+                  title   = 'Save file',
+                  parent  = None,
+                  action  = gtk.FILE_CHOOSER_ACTION_SAVE,
+                  buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                             gtk.STOCK_SAVE,   gtk.RESPONSE_OK),
+                  path    = None,
+                  filetypes = [],
+                  default_filetype = None
+                  ):
+        super(FileChooserDialog, self).__init__ (title, parent, action,
+                                                 buttons)
+        super(FileChooserDialog, self).set_do_overwrite_confirmation(True)
+        self.set_default_response (gtk.RESPONSE_OK)
 
-            if not path: path = os.getcwd() + os.sep
+        if not path: path = os.getcwd() + os.sep
 
-            # create an extra widget to list supported image formats
-            self.set_current_folder (path)
-            self.set_current_name ('image.' + default_filetype)
+        # create an extra widget to list supported image formats
+        self.set_current_folder (path)
+        self.set_current_name ('image.' + default_filetype)
 
-            hbox = gtk.HBox (spacing=10)
-            hbox.pack_start (gtk.Label ("File Format:"), expand=False)
+        hbox = gtk.HBox (spacing=10)
+        hbox.pack_start (gtk.Label ("File Format:"), expand=False)
 
-            liststore = gtk.ListStore(gobject.TYPE_STRING)
-            cbox = gtk.ComboBox(liststore)
-            cell = gtk.CellRendererText()
-            cbox.pack_start(cell, True)
-            cbox.add_attribute(cell, 'text', 0)
-            hbox.pack_start (cbox)
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        cbox = gtk.ComboBox(liststore)
+        cell = gtk.CellRendererText()
+        cbox.pack_start(cell, True)
+        cbox.add_attribute(cell, 'text', 0)
+        hbox.pack_start (cbox)
 
-            self.filetypes = filetypes
-            self.sorted_filetypes = filetypes.items()
-            self.sorted_filetypes.sort()
-            default = 0
-            for i, (ext, name) in enumerate(self.sorted_filetypes):
-                cbox.append_text ("%s (*.%s)" % (name, ext))
-                if ext == default_filetype:
-                    default = i
-            cbox.set_active(default)
-            self.ext = default_filetype
+        self.filetypes = filetypes
+        self.sorted_filetypes = filetypes.items()
+        self.sorted_filetypes.sort()
+        default = 0
+        for i, (ext, name) in enumerate(self.sorted_filetypes):
+            cbox.append_text ("%s (*.%s)" % (name, ext))
+            if ext == default_filetype:
+                default = i
+        cbox.set_active(default)
+        self.ext = default_filetype
 
-            def cb_cbox_changed (cbox, data=None):
-                """File extension changed"""
-                head, filename = os.path.split(self.get_filename())
-                root, ext = os.path.splitext(filename)
-                ext = ext[1:]
-                new_ext = self.sorted_filetypes[cbox.get_active()][0]
-                self.ext = new_ext
+        def cb_cbox_changed (cbox, data=None):
+            """File extension changed"""
+            head, filename = os.path.split(self.get_filename())
+            root, ext = os.path.splitext(filename)
+            ext = ext[1:]
+            new_ext = self.sorted_filetypes[cbox.get_active()][0]
+            self.ext = new_ext
 
-                if ext in self.filetypes:
-                    filename = root + '.' + new_ext
-                elif ext == '':
-                    filename = filename.rstrip('.') + '.' + new_ext
+            if ext in self.filetypes:
+                filename = root + '.' + new_ext
+            elif ext == '':
+                filename = filename.rstrip('.') + '.' + new_ext
 
-                self.set_current_name (filename)
-            cbox.connect ("changed", cb_cbox_changed)
+            self.set_current_name (filename)
+        cbox.connect ("changed", cb_cbox_changed)
 
-            hbox.show_all()
-            self.set_extra_widget(hbox)
-
-        def get_filename_from_user (self):
-            while True:
-                filename = None
-                if self.run() != int(gtk.RESPONSE_OK):
-                    break
-                filename = self.get_filename()
-                break
-
-            self.hide()
-            return filename, self.ext
-else:
-    class FileSelection(gtk.FileSelection):
-        """GTK+ 2.2 and lower file selector which remembers the last
-        file/directory selected
-        """
-        def __init__(self, path=None, title='Select a file', parent=None):
-            super(FileSelection, self).__init__(title)
-
-            if path: self.path = path
-            else:    self.path = os.getcwd() + os.sep
-
-            if parent: self.set_transient_for(parent)
-
-        def get_filename_from_user(self, path=None, title=None):
-            if path:  self.path = path
-            if title: self.set_title(title)
-            self.set_filename(self.path)
+        hbox.show_all()
+        self.set_extra_widget(hbox)
 
+    def get_filename_from_user (self):
+        while True:
             filename = None
-            if self.run() == int(gtk.RESPONSE_OK):
-                self.path = filename = self.get_filename()
-            self.hide()
-
-            ext = None
-            if filename is not None:
-                ext = os.path.splitext(filename)[1]
-                if ext.startswith('.'):
-                    ext = ext[1:]
-            return filename, ext
+            if self.run() != int(gtk.RESPONSE_OK):
+                break
+            filename = self.get_filename()
+            break
 
+        self.hide()
+        return filename, self.ext
 
 class DialogLineprops:
     """
@@ -1107,7 +1104,7 @@ class DialogLineprops:
     linestyled = dict([ (s,i) for i,s in enumerate(linestyles)])
 
 
-    markers =  [m for m in lines.Line2D.markers if cbook.is_string_like(m)]
+    markers =  [m for m in markers.MarkerStyle.markers if cbook.is_string_like(m)]
 
     markerd = dict([(s,i) for i,s in enumerate(markers)])
 
@@ -1194,12 +1191,12 @@ class DialogLineprops:
 
         button = self.wtree.get_widget('colorbutton_linestyle')
         color = button.get_color()
-        r, g, b = [val/65535. for val in color.red, color.green, color.blue]
+        r, g, b = [val/65535. for val in (color.red, color.green, color.blue)]
         line.set_color((r,g,b))
 
         button = self.wtree.get_widget('colorbutton_markerface')
         color = button.get_color()
-        r, g, b = [val/65535. for val in color.red, color.green, color.blue]
+        r, g, b = [val/65535. for val in (color.red, color.green, color.blue)]
         line.set_markerfacecolor((r,g,b))
 
         line.figure.canvas.draw()
@@ -1221,12 +1218,12 @@ class DialogLineprops:
         self.cbox_markers.set_active(self.markerd[marker])
 
         r,g,b = colorConverter.to_rgb(line.get_color())
-        color = gtk.gdk.Color(*[int(val*65535) for val in r,g,b])
+        color = gtk.gdk.Color(*[int(val*65535) for val in (r,g,b)])
         button = self.wtree.get_widget('colorbutton_linestyle')
         button.set_color(color)
 
         r,g,b = colorConverter.to_rgb(line.get_markerfacecolor())
-        color = gtk.gdk.Color(*[int(val*65535) for val in r,g,b])
+        color = gtk.gdk.Color(*[int(val*65535) for val in (r,g,b)])
         button = self.wtree.get_widget('colorbutton_markerface')
         button.set_color(color)
         self._updateson = True
@@ -1256,16 +1253,15 @@ class DialogLineprops:
 # versions of pygtk, so we have to use a PNG file instead.
 try:
 
-    if gtk.pygtk_version < (2, 8, 0):
+    if gtk.pygtk_version < (2, 8, 0) or sys.platform == 'win32':
         icon_filename = 'matplotlib.png'
     else:
         icon_filename = 'matplotlib.svg'
-    gtk.window_set_default_icon_from_file (
-        os.path.join (matplotlib.rcParams['datapath'], 'images', icon_filename))
+    window_icon = os.path.join(rcParams['datapath'], 'images', icon_filename)
 except:
+    window_icon = None
     verbose.report('Could not load matplotlib icon: %s' % sys.exc_info()[1])
 
-
 def error_msg_gtk(msg, parent=None):
     if parent is not None: # find the toplevel gtk.Window
         parent = parent.get_toplevel()
