@@ -1,8 +1,5 @@
-from __future__ import division
+from __future__ import division, print_function
 """
-
- backend_wx.py
-
  A wxPython backend for matplotlib, based (very heavily) on
  backend_template.py and backend_gtk.py
 
@@ -17,86 +14,18 @@ from __future__ import division
  should be included with this source code.
 
 """
-"""
-KNOWN BUGS -
- - Mousewheel (on Windows) only works after menu button has been pressed
-   at least once
- - Mousewheel on Linux (wxGTK linked against GTK 1.2) does not work at all
- - Vertical text renders horizontally if you use a non TrueType font
-   on Windows. This is a known wxPython issue. Work-around is to ensure
-   that you use a TrueType font.
- - Pcolor demo puts chart slightly outside bounding box (approx 1-2 pixels
-   to the bottom left)
- - Outputting to bitmap more than 300dpi results in some text being incorrectly
-   scaled. Seems to be a wxPython bug on Windows or font point sizes > 60, as
-   font size is correctly calculated.
- - Performance poorer than for previous direct rendering version
- - TIFF output not supported on wxGTK. This is a wxGTK issue
- - Text is not anti-aliased on wxGTK. This is probably a platform
-   configuration issue.
- - If a second call is made to show(), no figure is generated (#866965)
 
-Not implemented:
- - Printing
+import sys
+import os
+import os.path
+import math
+import StringIO
+import weakref
+import warnings
 
-Fixed this release:
- - Bug #866967: Interactive operation issues fixed [JDH]
- - Bug #866969: Dynamic update does not function with backend_wx [JOD]
+import numpy as np
 
-Examples which work on this release:
- ---------------------------------------------------------------
-                         | Windows 2000    |  Linux            |
-                         | wxPython 2.3.3  |  wxPython 2.4.2.4 |
- --------------------------------------------------------------|
- - alignment_test.py     |     TBE         |     OK            |
- - arctest.py            |     TBE         |     (3)           |
- - axes_demo.py          |     OK          |     OK            |
- - axes_props.py         |     OK          |     OK            |
- - bar_stacked.py        |     TBE         |     OK            |
- - barchart_demo.py      |     OK          |     OK            |
- - color_demo.py         |     OK          |     OK            |
- - csd_demo.py           |     OK          |     OK            |
- - dynamic_demo.py       |     N/A         |     N/A           |
- - dynamic_demo_wx.py    |     TBE         |     OK            |
- - embedding_in_gtk.py   |     N/A         |     N/A           |
- - embedding_in_wx.py    |     OK          |     OK            |
- - errorbar_demo.py      |     OK          |     OK            |
- - figtext.py            |     OK          |     OK            |
- - histogram_demo.py     |     OK          |     OK            |
- - interactive.py        |     N/A (2)     |     N/A (2)       |
- - interactive2.py       |     N/A (2)     |     N/A (2)       |
- - legend_demo.py        |     OK          |     OK            |
- - legend_demo2.py       |     OK          |     OK            |
- - line_styles.py        |     OK          |     OK            |
- - log_demo.py           |     OK          |     OK            |
- - logo.py               |     OK          |     OK            |
- - mpl_with_glade.py     |     N/A (2)     |     N/A (2)       |
- - mri_demo.py           |     OK          |     OK            |
- - mri_demo_with_eeg.py  |     OK          |     OK            |
- - multiple_figs_demo.py |     OK          |     OK            |
- - pcolor_demo.py        |     OK          |     OK            |
- - psd_demo.py           |     OK          |     OK            |
- - scatter_demo.py       |     OK          |     OK            |
- - scatter_demo2.py      |     OK          |     OK            |
- - simple_plot.py        |     OK          |     OK            |
- - stock_demo.py         |     OK          |     OK            |
- - subplot_demo.py       |     OK          |     OK            |
- - system_monitor.py     |     N/A (2)     |     N/A (2)       |
- - text_handles.py       |     OK          |     OK            |
- - text_themes.py        |     OK          |     OK            |
- - vline_demo.py         |     OK          |     OK            |
- ---------------------------------------------------------------
-
- (2) - Script uses GTK-specific features - cannot not run,
-       but wxPython equivalent should be written.
- (3) - Clipping seems to be broken.
-"""
-
-cvs_id = '$Id: backend_wx.py 6484 2008-12-03 18:38:03Z jdh2358 $'
-
-
-import sys, os, os.path, math, StringIO, weakref, warnings
-import numpy as npy
+from matplotlib import MatplotlibDeprecationWarning as mplDeprecation
 
 # Debugging settings here...
 # Debug level set here. If the debug level is less than 5, information
@@ -108,12 +37,49 @@ if _DEBUG < 5:
     import traceback, pdb
 _DEBUG_lvls = {1 : 'Low ', 2 : 'Med ', 3 : 'High', 4 : 'Error' }
 
+if sys.version_info[0] >= 3:
+    warnings.warn(
+        "The wx and wxagg backends have not been tested with Python 3.x",
+        ImportWarning)
+
+missingwx = "Matplotlib backend_wx and backend_wxagg require wxPython >=2.8"
+missingwxversion = ("Matplotlib backend_wx and backend_wxagg "
+                    "require wxversion, which was not found.")
+
+if not hasattr(sys, 'frozen'): # i.e., not py2exe
+    try:
+        import wxversion
+    except ImportError:
+        raise ImportError(missingwxversion)
+
+    # Some early versions of wxversion lack AlreadyImportedError.
+    # It was added around 2.8.4?
+    try:
+        _wx_ensure_failed = wxversion.AlreadyImportedError
+    except AttributeError:
+        _wx_ensure_failed = wxversion.VersionError
+
+    try:
+        wxversion.ensureMinimal('2.8')
+    except _wx_ensure_failed:
+        pass
+    # We don't really want to pass in case of VersionError, but when
+    # AlreadyImportedError is not available, we have to.
 
 try:
     import wx
     backend_version = wx.VERSION_STRING
-except:
-    raise ImportError("Matplotlib backend_wx requires wxPython be installed")
+except ImportError:
+    raise ImportError(missingwx)
+
+# Extra version check in case wxversion lacks AlreadyImportedError;
+# then VersionError might have been raised and ignored when
+# there really *is* a problem with the version.
+major, minor = [int(n) for n in backend_version.split('.')[:2]]
+if major < 2 or (major < 3 and minor < 8):
+    print(" wxPython version %s was imported." % backend_version)
+    raise ImportError(missingwx)
+
 
 #!!! this is the call that is causing the exception swallowing !!!
 #wx.InitAllImageHandlers()
@@ -125,12 +91,12 @@ def DEBUG_MSG(string, lvl=3, o=None):
         # one below does.  I think WX is redefining stderr, damned
         # beast
         #print >>sys.stderr, "%s- %s in %s" % (_DEBUG_lvls[lvl], string, cls)
-        print "%s- %s in %s" % (_DEBUG_lvls[lvl], string, cls)
+        print("%s- %s in %s" % (_DEBUG_lvls[lvl], string, cls))
 
 def debug_on_error(type, value, tb):
     """Code due to Thomas Heller - published in Python Cookbook (O'Reilley)"""
     traceback.print_exc(type, value, tb)
-    print
+    print()
     pdb.pm()  # jdh uncomment
 
 class fake_stderr:
@@ -138,7 +104,7 @@ class fake_stderr:
     is probably no console. This redirects stderr to the console, since we know
     that there is one!"""
     def write(self, msg):
-        print "Stderr: %s\n\r" % msg
+        print("Stderr: %s\n\r" % msg)
 
 #if _DEBUG < 5:
 #    sys.excepthook = debug_on_error
@@ -160,7 +126,9 @@ import matplotlib
 from matplotlib import verbose
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureCanvasBase, FigureManagerBase, NavigationToolbar2, \
-     cursors
+     cursors, TimerBase
+from matplotlib.backend_bases import ShowBase
+
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.artist import Artist
 from matplotlib.cbook import exception_to_str, is_string_like, is_writable_file_like
@@ -196,6 +164,50 @@ def raise_msg_to_str(msg):
         msg = '\n'.join(map(str, msg))
     return msg
 
+
+class TimerWx(TimerBase):
+    '''
+    Subclass of :class:`backend_bases.TimerBase` that uses WxTimer events.
+
+    Attributes:
+    * interval: The time between timer events in milliseconds. Default
+        is 1000 ms.
+    * single_shot: Boolean flag indicating whether this timer should
+        operate as single shot (run once and then stop). Defaults to False.
+    * callbacks: Stores list of (func, args) tuples that will be called
+        upon timer events. This list can be manipulated directly, or the
+        functions add_callback and remove_callback can be used.
+    '''
+    def __init__(self, parent, *args, **kwargs):
+        TimerBase.__init__(self, *args, **kwargs)
+
+        # Create a new timer and connect the timer event to our handler.
+        # For WX, the events have to use a widget for binding.
+        self.parent = parent
+        self._timer = wx.Timer(self.parent, wx.NewId())
+        self.parent.Bind(wx.EVT_TIMER, self._on_timer, self._timer)
+
+     # Unbinding causes Wx to stop for some reason. Disabling for now.
+#    def __del__(self):
+#        TimerBase.__del__(self)
+#        self.parent.Bind(wx.EVT_TIMER, None, self._timer)
+
+    def _timer_start(self):
+        self._timer.Start(self._interval, self._single)
+
+    def _timer_stop(self):
+        self._timer.Stop()
+
+    def _timer_set_interval(self):
+        self._timer_start()
+
+    def _timer_set_single_shot(self):
+        self._timer.start()
+
+    def _on_timer(self, *args):
+        TimerBase._on_timer(self)
+
+
 class RendererWx(RendererBase):
     """
     The renderer handles all the drawing primitives using a graphics
@@ -303,10 +315,10 @@ class RendererWx(RendererBase):
                 gfx_ctx.Clip(new_bounds[0], self.height - new_bounds[1] - new_bounds[3],
                              new_bounds[2], new_bounds[3])
 
-    #@staticmethod
-    def convert_path(gfx_ctx, tpath):
+    @staticmethod
+    def convert_path(gfx_ctx, path, transform):
         wxpath = gfx_ctx.CreatePath()
-        for points, code in tpath.iter_segments():
+        for points, code in path.iter_segments(transform):
             if code == Path.MOVETO:
                 wxpath.MoveToPoint(*points)
             elif code == Path.LINETO:
@@ -318,15 +330,13 @@ class RendererWx(RendererBase):
             elif code == Path.CLOSEPOLY:
                 wxpath.CloseSubpath()
         return wxpath
-    convert_path = staticmethod(convert_path)
 
     def draw_path(self, gc, path, transform, rgbFace=None):
         gc.select()
         self.handle_clip_rectangle(gc)
         gfx_ctx = gc.gfx_ctx
         transform = transform + Affine2D().scale(1.0, -1.0).translate(0.0, self.height)
-        tpath = transform.transform_path(path)
-        wxpath = self.convert_path(gfx_ctx, tpath)
+        wxpath = self.convert_path(gfx_ctx, path, transform)
         if rgbFace is not None:
             gfx_ctx.SetBrush(wx.Brush(gc.get_wxcolour(rgbFace)))
             gfx_ctx.DrawPath(wxpath)
@@ -334,7 +344,8 @@ class RendererWx(RendererBase):
             gfx_ctx.StrokePath(wxpath)
         gc.unselect()
 
-    def draw_image(self, x, y, im, bbox, clippath=None, clippath_trans=None):
+    def draw_image(self, gc, x, y, im):
+        bbox = gc.get_clip_rectangle()
         if bbox != None:
             l,b,w,h = bbox.bounds
         else:
@@ -343,12 +354,12 @@ class RendererWx(RendererBase):
             w=self.width
             h=self.height
         rows, cols, image_str = im.as_rgba_str()
-        image_array = npy.fromstring(image_str, npy.uint8)
+        image_array = np.fromstring(image_str, np.uint8)
         image_array.shape = rows, cols, 4
         bitmap = wx.BitmapFromBufferRGBA(cols,rows,image_array)
         gc = self.get_gc()
         gc.select()
-        gc.gfx_ctx.DrawBitmap(bitmap,int(l),int(b),int(w),int(h))
+        gc.gfx_ctx.DrawBitmap(bitmap,int(l),int(self.height-b),int(w),int(-h))
         gc.unselect()
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
@@ -399,7 +410,6 @@ class RendererWx(RendererBase):
         assert self.gc != None, "gc must be defined"
         return self.gc
 
-
     def get_wx_font(self, s, prop):
         """
         Return a wx font.  Cache instances in a font dictionary for
@@ -437,7 +447,6 @@ class RendererWx(RendererBase):
 
         return font
 
-
     def points_to_pixels(self, points):
         """
         convert point measures to pixes using dpi and the pixels per
@@ -445,6 +454,7 @@ class RendererWx(RendererBase):
         """
         return points*(PIXELS_PER_INCH/72.0*self.dpi/72.0)
 
+
 class GraphicsContextWx(GraphicsContextBase):
     """
     The graphics context provides the color, line styles, etc...
@@ -616,6 +626,7 @@ class GraphicsContextWx(GraphicsContextBase):
             a *= 255
             return wx.Colour(red=int(r), green=int(g), blue=int(b), alpha=int(a))
 
+
 class FigureCanvasWx(FigureCanvasBase, wx.Panel):
     """
     The FigureCanvas contains the figure and does event handling.
@@ -735,21 +746,30 @@ class FigureCanvasWx(FigureCanvasBase, wx.Panel):
         bind(self, wx.EVT_KEY_DOWN, self._onKeyDown)
         bind(self, wx.EVT_KEY_UP, self._onKeyUp)
         bind(self, wx.EVT_RIGHT_DOWN, self._onRightButtonDown)
-        bind(self, wx.EVT_RIGHT_DCLICK, self._onRightButtonDown)
+        bind(self, wx.EVT_RIGHT_DCLICK, self._onRightButtonDClick)
         bind(self, wx.EVT_RIGHT_UP, self._onRightButtonUp)
         bind(self, wx.EVT_MOUSEWHEEL, self._onMouseWheel)
         bind(self, wx.EVT_LEFT_DOWN, self._onLeftButtonDown)
-        bind(self, wx.EVT_LEFT_DCLICK, self._onLeftButtonDown)
+        bind(self, wx.EVT_LEFT_DCLICK, self._onLeftButtonDClick)
         bind(self, wx.EVT_LEFT_UP, self._onLeftButtonUp)
         bind(self, wx.EVT_MOTION, self._onMotion)
         bind(self, wx.EVT_LEAVE_WINDOW, self._onLeave)
         bind(self, wx.EVT_ENTER_WINDOW, self._onEnter)
         bind(self, wx.EVT_IDLE, self._onIdle)
+        #Add middle button events
+        bind(self, wx.EVT_MIDDLE_DOWN, self._onMiddleButtonDown)
+        bind(self, wx.EVT_MIDDLE_DCLICK, self._onMiddleButtonDClick)
+        bind(self, wx.EVT_MIDDLE_UP, self._onMiddleButtonUp)
+
         self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
 
         self.macros = {} # dict from wx id to seq of macros
 
-        self.Printer_Init()
+        # printer attributes and methods deprecated, 2010/06/19
+        self._printerData = None
+        self._printerPageData = None
+        self.printer_width = 5.5
+        self.printer_margin = 0.5
 
     def Destroy(self, *args, **kwargs):
         wx.Panel.Destroy(self, *args, **kwargs)
@@ -759,12 +779,21 @@ class FigureCanvasWx(FigureCanvasBase, wx.Panel):
         "copy bitmap of canvas to system clipboard"
         bmp_obj = wx.BitmapDataObject()
         bmp_obj.SetBitmap(self.bitmap)
-        wx.TheClipboard.Open()
-        wx.TheClipboard.SetData(bmp_obj)
-        wx.TheClipboard.Close()
+
+        if not wx.TheClipboard.IsOpened():
+           open_success = wx.TheClipboard.Open()
+           if open_success:
+              wx.TheClipboard.SetData(bmp_obj)
+              wx.TheClipboard.Close()
+              wx.TheClipboard.Flush()
 
     def Printer_Init(self):
-        """initialize printer settings using wx methods"""
+        """
+        initialize printer settings using wx methods
+
+        Deprecated.
+        """
+        warnings.warn("Printer* methods will be removed", mplDeprecation)
         self.printerData = wx.PrintData()
         self.printerData.SetPaperId(wx.PAPER_LETTER)
         self.printerData.SetPrintMode(wx.PRINT_MODE_PRINTER)
@@ -776,14 +805,37 @@ class FigureCanvasWx(FigureCanvasBase, wx.Panel):
         self.printer_width = 5.5
         self.printer_margin= 0.5
 
+    def _get_printerData(self):
+        if self._printerData is None:
+            warnings.warn("Printer* methods will be removed", mplDeprecation)
+            self._printerData = wx.PrintData()
+            self._printerData.SetPaperId(wx.PAPER_LETTER)
+            self._printerData.SetPrintMode(wx.PRINT_MODE_PRINTER)
+        return self._printerData
+    printerData = property(_get_printerData)
+
+    def _get_printerPageData(self):
+        if self._printerPageData is None:
+            warnings.warn("Printer* methods will be removed", mplDeprecation)
+            self._printerPageData= wx.PageSetupDialogData()
+            self._printerPageData.SetMarginBottomRight((25,25))
+            self._printerPageData.SetMarginTopLeft((25,25))
+            self._printerPageData.SetPrintData(self.printerData)
+        return self._printerPageData
+    printerPageData = property(_get_printerPageData)
+
     def Printer_Setup(self, event=None):
-        """set up figure for printing.  The standard wx Printer
+        """
+        set up figure for printing.  The standard wx Printer
         Setup Dialog seems to die easily. Therefore, this setup
-        simply asks for image width and margin for printing. """
+        simply asks for image width and margin for printing.
+        Deprecated.
+        """
 
         dmsg = """Width of output figure in inches.
-The current aspect ration will be kept."""
+The current aspect ratio will be kept."""
 
+        warnings.warn("Printer* methods will be removed", mplDeprecation)
         dlg = wx.Dialog(self, -1, 'Page Setup for Printing' , (-1,-1))
         df = dlg.GetFont()
         df.SetWeight(wx.NORMAL)
@@ -839,9 +891,14 @@ The current aspect ration will be kept."""
         return
 
     def Printer_Setup2(self, event=None):
-        """set up figure for printing.  Using the standard wx Printer
-        Setup Dialog. """
+        """
+        set up figure for printing.  Using the standard wx Printer
+        Setup Dialog.
 
+        Deprecated.
+        """
+
+        warnings.warn("Printer* methods will be removed", mplDeprecation)
         if hasattr(self, 'printerData'):
             data = wx.PageSetupDialogData()
             data.SetPrintData(self.printerData)
@@ -860,13 +917,18 @@ The current aspect ration will be kept."""
         dlg.Destroy()
 
     def Printer_Preview(self, event=None):
-        """ generate Print Preview with wx Print mechanism"""
+        """
+        generate Print Preview with wx Print mechanism
+
+        Deprecated.
+        """
+        warnings.warn("Printer* methods will be removed", mplDeprecation)
         po1  = PrintoutWx(self, width=self.printer_width,
                           margin=self.printer_margin)
         po2  = PrintoutWx(self, width=self.printer_width,
                           margin=self.printer_margin)
         self.preview = wx.PrintPreview(po1,po2,self.printerData)
-        if not self.preview.Ok():  print "error with preview"
+        if not self.preview.Ok():  print("error with preview")
 
         self.preview.SetZoom(50)
         frameInst= self
@@ -881,7 +943,12 @@ The current aspect ration will be kept."""
         self.gui_repaint()
 
     def Printer_Print(self, event=None):
-        """ Print figure using wx Print mechanism"""
+        """
+        Print figure using wx Print mechanism
+
+        Deprecated.
+        """
+        warnings.warn("Printer* methods will be removed", mplDeprecation)
         pdd = wx.PrintDialogData()
         # SetPrintData for 2.4 combatibility
         pdd.SetPrintData(self.printerData)
@@ -904,7 +971,6 @@ The current aspect ration will be kept."""
         printout.Destroy()
         self.gui_repaint()
 
-
     def draw_idle(self):
         """
         Delay rendering until the GUI is idle.
@@ -944,6 +1010,22 @@ The current aspect ration will be kept."""
         self._isDrawn = True
         self.gui_repaint(drawDC=drawDC)
 
+    def new_timer(self, *args, **kwargs):
+        """
+        Creates a new backend-specific subclass of :class:`backend_bases.Timer`.
+        This is useful for getting periodic events through the backend's native
+        event loop. Implemented only for backends with GUIs.
+
+        optional arguments:
+
+        *interval*
+          Timer interval in milliseconds
+        *callbacks*
+          Sequence of (func, args, kwargs) where func(*args, **kwargs) will
+          be executed by the timer every *interval*.
+        """
+        return TimerWx(self, *args, **kwargs)
+
     def flush_events(self):
         wx.Yield()
 
@@ -1108,9 +1190,6 @@ The current aspect ration will be kept."""
             self.draw()
         self.Refresh()
 
-    def get_default_filetype(self):
-        return 'png'
-
     def _onPaint(self, evt):
         """
         Called when wxPaintEvt is generated
@@ -1156,19 +1235,27 @@ The current aspect ration will be kept."""
         # so no need to do anything here except to make sure
         # the whole background is repainted.
         self.Refresh(eraseBackground=False)
+        FigureCanvasBase.resize_event(self)
 
     def _get_key(self, evt):
 
         keyval = evt.m_keyCode
         if keyval in self.keyvald:
             key = self.keyvald[keyval]
-        elif keyval <256:
+        elif keyval < 256:
             key = chr(keyval)
+            # wx always returns an uppercase, so make it lowercase if the shift
+            # key is not depressed (NOTE: this will not handle Caps Lock)
+            if not evt.ShiftDown():
+                key = key.lower()
         else:
             key = None
 
-        # why is wx upcasing this?
-        if key is not None: key = key.lower()
+        for meth, prefix in (
+                             [evt.AltDown, 'alt'],
+                             [evt.ControlDown, 'ctrl'], ):
+            if meth():
+                key = '{0}+{1}'.format(prefix, key)
 
         return key
 
@@ -1198,6 +1285,13 @@ The current aspect ration will be kept."""
         self.CaptureMouse()
         FigureCanvasBase.button_press_event(self, x, y, 3, guiEvent=evt)
 
+    def _onRightButtonDClick(self, evt):
+        """Start measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height - evt.GetY()
+        evt.Skip()
+        self.CaptureMouse()
+        FigureCanvasBase.button_press_event(self, x, y, 3, dblclick=True,guiEvent=evt)
 
     def _onRightButtonUp(self, evt):
         """End measuring on an axis."""
@@ -1215,6 +1309,14 @@ The current aspect ration will be kept."""
         self.CaptureMouse()
         FigureCanvasBase.button_press_event(self, x, y, 1, guiEvent=evt)
 
+    def _onLeftButtonDClick(self, evt):
+        """Start measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height - evt.GetY()
+        evt.Skip()
+        self.CaptureMouse()
+        FigureCanvasBase.button_press_event(self, x, y, 1, dblclick=True, guiEvent=evt)
+
     def _onLeftButtonUp(self, evt):
         """End measuring on an axis."""
         x = evt.GetX()
@@ -1224,6 +1326,32 @@ The current aspect ration will be kept."""
         if self.HasCapture(): self.ReleaseMouse()
         FigureCanvasBase.button_release_event(self, x, y, 1, guiEvent=evt)
 
+    #Add middle button events
+    def _onMiddleButtonDown(self, evt):
+        """Start measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height - evt.GetY()
+        evt.Skip()
+        self.CaptureMouse()
+        FigureCanvasBase.button_press_event(self, x, y, 2, guiEvent=evt)
+
+    def _onMiddleButtonDClick(self, evt):
+        """Start measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height - evt.GetY()
+        evt.Skip()
+        self.CaptureMouse()
+        FigureCanvasBase.button_press_event(self, x, y, 2, dblclick=True, guiEvent=evt)
+
+    def _onMiddleButtonUp(self, evt):
+        """End measuring on an axis."""
+        x = evt.GetX()
+        y = self.figure.bbox.height - evt.GetY()
+        #print 'release button', 1
+        evt.Skip()
+        if self.HasCapture(): self.ReleaseMouse()
+        FigureCanvasBase.button_release_event(self, x, y, 2, guiEvent=evt)
+
     def _onMouseWheel(self, evt):
         """Translate mouse wheel events into matplotlib events"""
 
@@ -1306,35 +1434,17 @@ def draw_if_interactive():
 
         figManager = Gcf.get_active()
         if figManager is not None:
-            figManager.canvas.draw()
+            figManager.canvas.draw_idle()
 
-def show():
-    """
-    Current implementation assumes that matplotlib is executed in a PyCrust
-    shell. It appears to be possible to execute wxPython applications from
-    within a PyCrust without having to ensure that wxPython has been created
-    in a secondary thread (e.g. SciPy gui_thread).
-
-    Unfortunately, gui_thread seems to introduce a number of further
-    dependencies on SciPy modules, which I do not wish to introduce
-    into the backend at this point. If there is a need I will look
-    into this in a later release.
-    """
-    DEBUG_MSG("show()", 3, None)
-
-    for figwin in Gcf.get_all_fig_managers():
-        figwin.frame.Show()
-
-    if show._needmain and not matplotlib.is_interactive():
-        # start the wxPython gui event if there is not already one running
-        wxapp = wx.GetApp()
-        if wxapp is not None:
-            # wxPython 2.4 has no wx.App.IsMainLoopRunning() method
-            imlr = getattr(wxapp, 'IsMainLoopRunning', lambda: False)
-            if not imlr():
+class Show(ShowBase):
+    def mainloop(self):
+        needmain = not wx.App.IsMainLoopRunning()
+        if needmain:
+            wxapp = wx.GetApp()
+            if wxapp is not None:
                 wxapp.MainLoop()
-        show._needmain = False
-show._needmain = True
+
+show = Show()
 
 def new_figure_manager(num, *args, **kwargs):
     """
@@ -1347,6 +1457,14 @@ def new_figure_manager(num, *args, **kwargs):
 
     FigureClass = kwargs.pop('FigureClass', Figure)
     fig = FigureClass(*args, **kwargs)
+    return new_figure_manager_given_figure(num, fig)
+
+
+def new_figure_manager_given_figure(num, figure):
+    """
+    Create a new figure manager instance for the given figure.
+    """
+    fig = figure
     frame = FigureFrameWx(num, fig)
     figmgr = frame.get_figure_manager()
     if matplotlib.is_interactive():
@@ -1354,6 +1472,7 @@ def new_figure_manager(num, *args, **kwargs):
 
     return figmgr
 
+
 class FigureFrameWx(wx.Frame):
     def __init__(self, num, fig):
         # On non-Windows platform, explicitly set the position - fix
@@ -1373,6 +1492,7 @@ class FigureFrameWx(wx.Frame):
         self.SetStatusBar(statbar)
         self.canvas = self.get_canvas(fig)
         self.canvas.SetInitialSize(wx.Size(fig.bbox.width, fig.bbox.height))
+        self.canvas.SetFocus()
         self.sizer =wx.BoxSizer(wx.VERTICAL)
         self.sizer.Add(self.canvas, 1, wx.TOP | wx.LEFT | wx.EXPAND)
         # By adding toolbar in sizer, we are able to put it at the bottom
@@ -1400,14 +1520,25 @@ class FigureFrameWx(wx.Frame):
         self.SetSizer(self.sizer)
         self.Fit()
 
+        self.canvas.SetMinSize((2, 2))
+
+        # give the window a matplotlib icon rather than the stock one.
+        # This is not currently working on Linux and is untested elsewhere.
+        #icon_path = os.path.join(matplotlib.rcParams['datapath'],
+        #                         'images', 'matplotlib.png')
+        #icon = wx.IconFromBitmap(wx.Bitmap(icon_path))
+        # for xpm type icons try:
+        #icon = wx.Icon(icon_path, wx.BITMAP_TYPE_XPM)
+        #self.SetIcon(icon)
+
         self.figmgr = FigureManagerWx(self.canvas, num, self)
 
         bind(self, wx.EVT_CLOSE, self._onClose)
 
     def _get_toolbar(self, statbar):
-        if matplotlib.rcParams['toolbar']=='classic':
+        if rcParams['toolbar']=='classic':
             toolbar = NavigationToolbarWx(self.canvas, True)
-        elif matplotlib.rcParams['toolbar']=='toolbar2':
+        elif rcParams['toolbar']=='toolbar2':
             toolbar = NavigationToolbar2Wx(self.canvas)
             toolbar.set_status_bar(statbar)
         else:
@@ -1423,6 +1554,7 @@ class FigureFrameWx(wx.Frame):
 
     def _onClose(self, evt):
         DEBUG_MSG("onClose()", 1, self)
+        self.canvas.close_event()
         self.canvas.stop_event_loop()
         Gcf.destroy(self.num)
         #self.Destroy()
@@ -1432,6 +1564,11 @@ class FigureFrameWx(wx.Frame):
         return self.toolbar
 
     def Destroy(self, *args, **kwargs):
+        try:
+            self.canvas.mpl_disconnect(self.toolbar._idDrag)
+            # Rationale for line above: see issue 2941338.
+        except AttributeError:
+            pass # classic toolbar lacks the attribute
         wx.Frame.Destroy(self, *args, **kwargs)
         if self.toolbar is not None:
             self.toolbar.Destroy()
@@ -1440,6 +1577,7 @@ class FigureFrameWx(wx.Frame):
             wxapp.Yield()
         return True
 
+
 class FigureManagerWx(FigureManagerBase):
     """
     This class contains the FigureCanvas and GUI frame
@@ -1447,8 +1585,6 @@ class FigureManagerWx(FigureManagerBase):
     It is instantiated by GcfWx whenever a new figure is created. GcfWx is
     responsible for managing multiple instances of FigureManagerWx.
 
-    NB: FigureManagerBase is found in _pylab_helpers
-
     public attrs
 
     canvas - a FigureCanvasWx(wx.Panel) instance
@@ -1467,21 +1603,19 @@ class FigureManagerWx(FigureManagerBase):
             if self.tb != None: self.tb.update()
         self.canvas.figure.add_axobserver(notify_axes_change)
 
-        def showfig(*args):
-            frame.Show()
-
-        # attach a show method to the figure
-        self.canvas.figure.show = showfig
-
+    def show(self):
+        self.frame.Show()
 
     def destroy(self, *args):
         DEBUG_MSG("destroy()", 1, self)
         self.frame.Destroy()
         #if self.tb is not None: self.tb.Destroy()
-        import wx
         #wx.GetApp().ProcessIdle()
         wx.WakeUpIdle()
 
+    def get_window_title(self):
+        return self.window.GetTitle()
+
     def set_window_title(self, title):
         self.window.SetTitle(title)
 
@@ -1591,6 +1725,8 @@ class MenuButtonWx(wx.Button):
         else:
             new = True
         self._menu.Check(evt.GetId(), new)
+        # Lines above would be deleted based on svn tracker ID 2841525;
+        # not clear whether this matters or not.
         self._toolbar.set_active(self.getActiveAxes())
         evt.Skip()
 
@@ -1604,7 +1740,11 @@ class MenuButtonWx(wx.Button):
                 self._menu.Append(menuId, "Axis %d" % i, "Select axis %d" % i, True)
                 self._menu.Check(menuId, True)
                 bind(self, wx.EVT_MENU, self._onMenuItemSelected, id=menuId)
-        self._toolbar.set_active(range(len(self._axisId)))
+        elif maxAxis < len(self._axisId):
+            for menuId in self._axisId[maxAxis:]:
+                self._menu.Delete(menuId)
+            self._axisId = self._axisId[:maxAxis]
+        self._toolbar.set_active(range(maxAxis))
 
     def getActiveAxes(self):
         """Return a list of the selected axes."""
@@ -1623,8 +1763,6 @@ class MenuButtonWx(wx.Button):
         self.SetLabel("Axes: %s" % axis_txt[:-1])
 
 
-
-
 cursord = {
     cursors.MOVE : wx.CURSOR_HAND,
     cursors.HAND : wx.CURSOR_HAND,
@@ -1668,57 +1806,33 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
         DEBUG_MSG("_init_toolbar", 1, self)
 
         self._parent = self.canvas.GetParent()
-        _NTB2_HOME    =wx.NewId()
-        self._NTB2_BACK    =wx.NewId()
-        self._NTB2_FORWARD =wx.NewId()
-        self._NTB2_PAN     =wx.NewId()
-        self._NTB2_ZOOM    =wx.NewId()
-        _NTB2_SAVE    = wx.NewId()
-        _NTB2_SUBPLOT    =wx.NewId()
 
-        self.SetToolBitmapSize(wx.Size(24,24))
 
-        self.AddSimpleTool(_NTB2_HOME, _load_bitmap('home.png'),
-                           'Home', 'Reset original view')
-        self.AddSimpleTool(self._NTB2_BACK, _load_bitmap('back.png'),
-                           'Back', 'Back navigation view')
-        self.AddSimpleTool(self._NTB2_FORWARD, _load_bitmap('forward.png'),
-                           'Forward', 'Forward navigation view')
-        # todo: get new bitmap
-        self.AddCheckTool(self._NTB2_PAN, _load_bitmap('move.png'),
-                           shortHelp='Pan',
-                           longHelp='Pan with left, zoom with right')
-        self.AddCheckTool(self._NTB2_ZOOM, _load_bitmap('zoom_to_rect.png'),
-                           shortHelp='Zoom', longHelp='Zoom to rectangle')
-
-        self.AddSeparator()
-        self.AddSimpleTool(_NTB2_SUBPLOT, _load_bitmap('subplots.png'),
-                           'Configure subplots', 'Configure subplot parameters')
-
-        self.AddSimpleTool(_NTB2_SAVE, _load_bitmap('filesave.png'),
-                           'Save', 'Save plot contents to file')
-
-        bind(self, wx.EVT_TOOL, self.home, id=_NTB2_HOME)
-        bind(self, wx.EVT_TOOL, self.forward, id=self._NTB2_FORWARD)
-        bind(self, wx.EVT_TOOL, self.back, id=self._NTB2_BACK)
-        bind(self, wx.EVT_TOOL, self.zoom, id=self._NTB2_ZOOM)
-        bind(self, wx.EVT_TOOL, self.pan, id=self._NTB2_PAN)
-        bind(self, wx.EVT_TOOL, self.configure_subplot, id=_NTB2_SUBPLOT)
-        bind(self, wx.EVT_TOOL, self.save, id=_NTB2_SAVE)
+        self.wx_ids = {}
+        for text, tooltip_text, image_file, callback in self.toolitems:
+            if text is None:
+                self.AddSeparator()
+                continue
+            self.wx_ids[text] = wx.NewId()
+            if text in ['Pan', 'Zoom']:
+               self.AddCheckTool(self.wx_ids[text], _load_bitmap(image_file + '.png'),
+                                 shortHelp=text, longHelp=tooltip_text)
+            else:
+               self.AddSimpleTool(self.wx_ids[text], _load_bitmap(image_file + '.png'),
+                                  text, tooltip_text)
+            bind(self, wx.EVT_TOOL, getattr(self, callback), id=self.wx_ids[text])
 
         self.Realize()
 
-
     def zoom(self, *args):
-        self.ToggleTool(self._NTB2_PAN, False)
+        self.ToggleTool(self.wx_ids['Pan'], False)
         NavigationToolbar2.zoom(self, *args)
 
     def pan(self, *args):
-        self.ToggleTool(self._NTB2_ZOOM, False)
+        self.ToggleTool(self.wx_ids['Zoom'], False)
         NavigationToolbar2.pan(self, *args)
 
-
-    def configure_subplot(self, evt):
+    def configure_subplots(self, evt):
         frame = wx.Frame(None, -1, "Configure subplots")
 
         toolfig = Figure((6,3))
@@ -1736,13 +1850,13 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
         tool = SubplotTool(self.canvas.figure, toolfig)
         frame.Show()
 
-    def save(self, evt):
+    def save_figure(self, *args):
         # Fetch the required filename and file type.
         filetypes, exts, filter_index = self.canvas._get_imagesave_wildcards()
-        default_file = "image." + self.canvas.get_default_filetype()
+        default_file = self.canvas.get_default_filename()
         dlg = wx.FileDialog(self._parent, "Save to file", "", default_file,
                             filetypes,
-                            wx.SAVE|wx.OVERWRITE_PROMPT|wx.CHANGE_DIR)
+                            wx.SAVE|wx.OVERWRITE_PROMPT)
         dlg.SetFilterIndex(filter_index)
         if dlg.ShowModal() == wx.ID_OK:
             dirname  = dlg.GetDirectory()
@@ -1760,7 +1874,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             try:
                 self.canvas.print_figure(
                     os.path.join(dirname, filename), format=format)
-            except Exception, e:
+            except Exception as e:
                 error_msg_wx(str(e))
 
     def set_cursor(self, cursor):
@@ -1829,17 +1943,12 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
     def set_history_buttons(self):
         can_backward = (self._views._pos > 0)
         can_forward = (self._views._pos < len(self._views._elements) - 1)
-        self.EnableTool(self._NTB2_BACK, can_backward)
-        self.EnableTool(self._NTB2_FORWARD, can_forward)
+        self.EnableTool(self.wx_ids['Back'], can_backward)
+        self.EnableTool(self.wx_ids['Forward'], can_forward)
 
 
 class NavigationToolbarWx(wx.ToolBar):
     def __init__(self, canvas, can_kill=False):
-        """
-        figure is the Figure instance that the toolboar controls
-
-        win, if not None, is the wxWindow the Figure is embedded in
-        """
         wx.ToolBar.__init__(self, canvas.GetParent(), -1)
         DEBUG_MSG("__init__()", 1, self)
         self.canvas = canvas
@@ -1964,7 +2073,8 @@ class NavigationToolbarWx(wx.ToolBar):
 
     def update(self):
         """
-        Update the toolbar menu - called when (e.g.) a new subplot or axes are added
+        Update the toolbar menu - called when (e.g.) a new subplot
+        or axes are added
         """
         DEBUG_MSG("update()", 1, self)
         self._axes = self.canvas.figure.get_axes()
@@ -2029,13 +2139,12 @@ class NavigationToolbarWx(wx.ToolBar):
             direction = -1
         self.button_fn(direction)
 
-    _onSave = NavigationToolbar2Wx.save
+    _onSave = NavigationToolbar2Wx.save_figure
 
     def _onClose(self, evt):
         self.GetParent().Destroy()
 
 
-
 class StatusBarWx(wx.StatusBar):
     """
     A status bar is added to _FigureFrame to allow measurements and the
