@@ -2,6 +2,7 @@
 Contains a class for managing paths (polylines).
 """
 
+from __future__ import print_function
 import math
 from weakref import WeakValueDictionary
 
@@ -10,8 +11,10 @@ from numpy import ma
 
 from matplotlib._path import point_in_path, get_path_extents, \
     point_in_path_collection, get_path_collection_extents, \
-    path_in_path, path_intersects_path, convert_path_to_polygons
-from matplotlib.cbook import simple_linear_interpolation
+    path_in_path, path_intersects_path, convert_path_to_polygons, \
+    cleanup_path, points_in_path, clip_path_to_rect
+from matplotlib.cbook import simple_linear_interpolation, maxdict
+from matplotlib import rcParams
 
 class Path(object):
     """
@@ -57,25 +60,31 @@ class Path(object):
     at all, but have a default one provided for them by
     :meth:`iter_segments`.
 
-    Note also that the vertices and codes arrays should be treated as
-    immutable -- there are a number of optimizations and assumptions
-    made up front in the constructor that will not change when the
-    data changes.
+    .. note::
+
+        The vertices and codes arrays should be treated as
+        immutable -- there are a number of optimizations and assumptions
+        made up front in the constructor that will not change when the
+        data changes.
+
     """
 
     # Path codes
-    STOP      = 0 # 1 vertex
-    MOVETO    = 1 # 1 vertex
-    LINETO    = 2 # 1 vertex
-    CURVE3    = 3 # 2 vertices
-    CURVE4    = 4 # 3 vertices
-    CLOSEPOLY = 5 # 1 vertex
+    STOP      = 0    # 1 vertex
+    MOVETO    = 1    # 1 vertex
+    LINETO    = 2    # 1 vertex
+    CURVE3    = 3    # 2 vertices
+    CURVE4    = 4    # 3 vertices
+    CLOSEPOLY = 0x4f # 1 vertex
 
-    NUM_VERTICES = [1, 1, 1, 2, 3, 1]
+    NUM_VERTICES = [1, 1, 1, 2,
+                    3, 1, 1, 1,
+                    1, 1, 1, 1,
+                    1, 1, 1, 1]
 
     code_type = np.uint8
 
-    def __init__(self, vertices, codes=None):
+    def __init__(self, vertices, codes=None, _interpolation_steps=1, closed=False):
         """
         Create a new path with the given vertices and codes.
 
@@ -95,6 +104,11 @@ class Path(object):
         to NaNs which are then handled correctly by the Agg
         PathIterator and other consumers of path data, such as
         :meth:`iter_segments`.
+
+        *interpolation_steps* is used as a hint to certain projections,
+        such as Polar, that this path should be linearly interpolated
+        immediately before drawing.  This attribute is primarily an
+        implementation detail and is not intended for public use.
         """
         if ma.isMaskedArray(vertices):
             vertices = vertices.astype(np.float_).filled(np.nan)
@@ -105,39 +119,77 @@ class Path(object):
             codes = np.asarray(codes, self.code_type)
             assert codes.ndim == 1
             assert len(codes) == len(vertices)
+            if len(codes):
+                assert codes[0] == self.MOVETO
+        elif closed:
+            codes = np.empty(len(vertices), dtype=self.code_type)
+            codes[0] = self.MOVETO
+            codes[1:-1] = self.LINETO
+            codes[-1] = self.CLOSEPOLY
 
         assert vertices.ndim == 2
         assert vertices.shape[1] == 2
 
-        self.should_simplify = (len(vertices) >= 128 and
-                                (codes is None or np.all(codes <= Path.LINETO)))
+        self.should_simplify = (rcParams['path.simplify'] and
+                                (len(vertices) >= 128 and
+                                 (codes is None or np.all(codes <= Path.LINETO))))
+        self.simplify_threshold = rcParams['path.simplify_threshold']
         self.has_nonfinite = not np.isfinite(vertices).all()
         self.codes = codes
         self.vertices = vertices
+        self._interpolation_steps = _interpolation_steps
 
-    #@staticmethod
-    def make_compound_path(*args):
+    @classmethod
+    def make_compound_path_from_polys(cls, XY):
+        """
+        (static method) Make a compound path object to draw a number
+        of polygons with equal numbers of sides XY is a (numpolys x
+        numsides x 2) numpy array of vertices.  Return object is a
+        :class:`Path`
+
+        .. plot:: mpl_examples/api/histogram_path_demo.py
+
+        """
+
+        # for each poly: 1 for the MOVETO, (numsides-1) for the LINETO, 1 for the
+        # CLOSEPOLY; the vert for the closepoly is ignored but we still need
+        # it to keep the codes aligned with the vertices
+        numpolys, numsides, two = XY.shape
+        assert(two==2)
+        stride = numsides + 1
+        nverts = numpolys * stride
+        verts = np.zeros((nverts, 2))
+        codes = np.ones(nverts, int) * cls.LINETO
+        codes[0::stride] = cls.MOVETO
+        codes[numsides::stride] = cls.CLOSEPOLY
+        for i in range(numsides):
+            verts[i::stride] = XY[:,i]
+
+        return cls(verts, codes)
+
+    @classmethod
+    def make_compound_path(cls, *args):
         """
         (staticmethod) Make a compound path from a list of Path
-        objects.  Only polygons (not curves) are supported.
+        objects.
         """
-        for p in args:
-            assert p.codes is None
-
         lengths = [len(x) for x in args]
         total_length = sum(lengths)
 
         vertices = np.vstack([x.vertices for x in args])
         vertices.reshape((total_length, 2))
 
-        codes = Path.LINETO * np.ones(total_length)
+        codes = np.empty(total_length, dtype=cls.code_type)
         i = 0
-        for length in lengths:
-            codes[i] = Path.MOVETO
-            i += length
+        for path in args:
+            if path.codes is None:
+                codes[i] = cls.MOVETO
+                codes[i + 1:i + len(path.vertices)] = cls.LINETO
+            else:
+                codes[i:i + len(path.codes)] = path.codes
+            i += len(path.vertices)
 
-        return Path(vertices, codes)
-    make_compound_path = staticmethod(make_compound_path)
+        return cls(vertices, codes)
 
     def __repr__(self):
         return "Path(%s, %s)" % (self.vertices, self.codes)
@@ -145,31 +197,48 @@ class Path(object):
     def __len__(self):
         return len(self.vertices)
 
-    def iter_segments(self, simplify=None):
+    def iter_segments(self, transform=None, remove_nans=True, clip=None,
+                      snap=False, stroke_width=1.0, simplify=None,
+                      curves=True):
         """
         Iterates over all of the curve segments in the path.  Each
         iteration returns a 2-tuple (*vertices*, *code*), where
         *vertices* is a sequence of 1 - 3 coordinate pairs, and *code* is
         one of the :class:`Path` codes.
 
-        If *simplify* is provided, it must be a tuple (*width*,
-        *height*) defining the size of the figure, in native units
-        (e.g. pixels or points).  Simplification implies both removing
-        adjacent line segments that are very close to parallel, and
-        removing line segments outside of the figure.  The path will
-        be simplified *only* if :attr:`should_simplify` is True, which
-        is determined in the constructor by this criteria:
+        Additionally, this method can provide a number of standard
+        cleanups and conversions to the path.
 
-           - No curves
-           - More than 128 vertices
+        *transform*: if not None, the given affine transformation will
+         be applied to the path.
+
+        *remove_nans*: if True, will remove all NaNs from the path and
+         insert MOVETO commands to skip over them.
+
+        *clip*: if not None, must be a four-tuple (x1, y1, x2, y2)
+         defining a rectangle in which to clip the path.
+
+        *snap*: if None, auto-snap to pixels, to reduce
+         fuzziness of rectilinear lines.  If True, force snapping, and
+         if False, don't snap.
+
+        *stroke_width*: the width of the stroke being drawn.  Needed
+         as a hint for the snapping algorithm.
+
+        *simplify*: if True, perform simplification, to remove
+         vertices that do not affect the appearance of the path.  If
+         False, perform no simplification.  If None, use the
+         should_simplify member variable.
+
+        *curves*: If True, curve segments will be returned as curve
+         segments.  If False, all curves will be converted to line
+         segments.
         """
         vertices = self.vertices
         if not len(vertices):
             return
 
         codes        = self.codes
-        len_vertices = len(vertices)
-        isfinite     = np.isfinite
 
         NUM_VERTICES = self.NUM_VERTICES
         MOVETO       = self.MOVETO
@@ -177,69 +246,65 @@ class Path(object):
         CLOSEPOLY    = self.CLOSEPOLY
         STOP         = self.STOP
 
-        if simplify is not None and self.should_simplify:
-            polygons = self.to_polygons(None, *simplify)
-            for vertices in polygons:
-                yield vertices[0], MOVETO
-                for v in vertices[1:]:
-                    yield v, LINETO
-        elif codes is None:
-            if self.has_nonfinite:
-                next_code = MOVETO
-                for v in vertices:
-                    if np.isfinite(v).all():
-                        yield v, next_code
-                        next_code = LINETO
-                    else:
-                        next_code = MOVETO
+        vertices, codes = cleanup_path(self, transform, remove_nans, clip,
+                                       snap, stroke_width, simplify, curves)
+        len_vertices = len(vertices)
+
+        i = 0
+        while i < len_vertices:
+            code = codes[i]
+            if code == STOP:
+                return
             else:
-                yield vertices[0], MOVETO
-                for v in vertices[1:]:
-                    yield v, LINETO
-        else:
-            i = 0
-            was_nan = False
-            while i < len_vertices:
-                code = codes[i]
-                if code == CLOSEPOLY:
-                    yield [], code
-                    i += 1
-                elif code == STOP:
-                    return
-                else:
-                    num_vertices = NUM_VERTICES[int(code)]
-                    curr_vertices = vertices[i:i+num_vertices].flatten()
-                    if not isfinite(curr_vertices).all():
-                        was_nan = True
-                    elif was_nan:
-                        yield curr_vertices[-2:], MOVETO
-                        was_nan = False
-                    else:
-                        yield curr_vertices, code
-                    i += num_vertices
+                num_vertices = NUM_VERTICES[int(code) & 0xf]
+                curr_vertices = vertices[i:i+num_vertices].flatten()
+                yield curr_vertices, code
+                i += num_vertices
 
     def transformed(self, transform):
         """
         Return a transformed copy of the path.
 
         .. seealso::
-            :class:`matplotlib.transforms.TransformedPath`:
+
+            :class:`matplotlib.transforms.TransformedPath`
                 A specialized path class that will cache the
                 transformed result and automatically update when the
                 transform changes.
         """
-        return Path(transform.transform(self.vertices), self.codes)
+        return Path(transform.transform(self.vertices), self.codes,
+                    self._interpolation_steps)
 
-    def contains_point(self, point, transform=None):
+    def contains_point(self, point, transform=None, radius=0.0):
         """
         Returns *True* if the path contains the given point.
 
         If *transform* is not *None*, the path will be transformed
         before performing the test.
+
+        *radius* allows the path to be made slightly larger or
+        smaller.
         """
         if transform is not None:
             transform = transform.frozen()
-        return point_in_path(point[0], point[1], self, transform)
+        result = point_in_path(point[0], point[1], radius, self, transform)
+        return result
+
+    def contains_points(self, points, transform=None, radius=0.0):
+        """
+        Returns a bool array which is *True* if the path contains the
+        corresponding point.
+
+        If *transform* is not *None*, the path will be transformed
+        before performing the test.
+
+        *radius* allows the path to be made slightly larger or
+        smaller.
+        """
+        if transform is not None:
+            transform = transform.frozen()
+        result = points_in_path(points, radius, self, transform)
+        return result
 
     def contains_path(self, path, transform=None):
         """
@@ -262,9 +327,13 @@ class Path(object):
         control points appropriately.
         """
         from transforms import Bbox
+        path = self
         if transform is not None:
             transform = transform.frozen()
-        return Bbox(get_path_extents(self, transform))
+            if not transform.is_affine:
+                path = self.transformed(transform)
+                transform = None
+        return Bbox(get_path_extents(path, transform))
 
     def intersects_path(self, other, filled=True):
         """
@@ -296,6 +365,9 @@ class Path(object):
         Returns a new path resampled to length N x steps.  Does not
         currently handle interpolating curves.
         """
+        if steps == 1:
+            return self
+
         vertices = simple_linear_interpolation(self.vertices, steps)
         codes = self.codes
         if codes is not None:
@@ -334,7 +406,7 @@ class Path(object):
         return convert_path_to_polygons(self, transform, width, height)
 
     _unit_rectangle = None
-    #@classmethod
+    @classmethod
     def unit_rectangle(cls):
         """
         (staticmethod) Returns a :class:`Path` of the unit rectangle
@@ -342,12 +414,13 @@ class Path(object):
         """
         if cls._unit_rectangle is None:
             cls._unit_rectangle = \
-                Path([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0], [0.0, 0.0]])
+                cls([[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0], [0.0, 0.0]],
+                    [cls.MOVETO, cls.LINETO, cls.LINETO, cls.LINETO, cls.CLOSEPOLY])
         return cls._unit_rectangle
-    unit_rectangle = classmethod(unit_rectangle)
 
     _unit_regular_polygons = WeakValueDictionary()
-    #@classmethod
+
+    @classmethod
     def unit_regular_polygon(cls, numVertices):
         """
         (staticmethod) Returns a :class:`Path` for a unit regular
@@ -365,13 +438,18 @@ class Path(object):
             # "points-up"
             theta += np.pi / 2.0
             verts = np.concatenate((np.cos(theta), np.sin(theta)), 1)
-            path = Path(verts)
-            cls._unit_regular_polygons[numVertices] = path
+            codes = np.empty((numVertices + 1,))
+            codes[0] = cls.MOVETO
+            codes[1:-1] = cls.LINETO
+            codes[-1] = cls.CLOSEPOLY
+            path = cls(verts, codes)
+            if numVertices <= 16:
+                cls._unit_regular_polygons[numVertices] = path
         return path
-    unit_regular_polygon = classmethod(unit_regular_polygon)
 
     _unit_regular_stars = WeakValueDictionary()
-    #@classmethod
+
+    @classmethod
     def unit_regular_star(cls, numVertices, innerCircle=0.5):
         """
         (staticmethod) Returns a :class:`Path` for a unit regular star
@@ -391,12 +469,16 @@ class Path(object):
             r = np.ones(ns2 + 1)
             r[1::2] = innerCircle
             verts = np.vstack((r*np.cos(theta), r*np.sin(theta))).transpose()
-            path = Path(verts)
-            cls._unit_regular_polygons[(numVertices, innerCircle)] = path
+            codes = np.empty((ns2 + 1,))
+            codes[0] = cls.MOVETO
+            codes[1:-1] = cls.LINETO
+            codes[-1] = cls.CLOSEPOLY
+            path = cls(verts, codes)
+            if numVertices <= 16:
+                cls._unit_regular_polygons[(numVertices, innerCircle)] = path
         return path
-    unit_regular_star = classmethod(unit_regular_star)
 
-    #@classmethod
+    @classmethod
     def unit_regular_asterisk(cls, numVertices):
         """
         (staticmethod) Returns a :class:`Path` for a unit regular
@@ -404,10 +486,10 @@ class Path(object):
         centered at (0, 0).
         """
         return cls.unit_regular_star(numVertices, 0.0)
-    unit_regular_asterisk = classmethod(unit_regular_asterisk)
 
     _unit_circle = None
-    #@classmethod
+
+    @classmethod
     def unit_circle(cls):
         """
         (staticmethod) Returns a :class:`Path` of the unit circle.
@@ -465,11 +547,58 @@ class Path(object):
             codes[0] = cls.MOVETO
             codes[-1] = cls.CLOSEPOLY
 
-            cls._unit_circle = Path(vertices, codes)
+            cls._unit_circle = cls(vertices, codes)
         return cls._unit_circle
-    unit_circle = classmethod(unit_circle)
 
-    #@classmethod
+    _unit_circle_righthalf = None
+
+    @classmethod
+    def unit_circle_righthalf(cls):
+        """
+        (staticmethod) Returns a :class:`Path` of the right half
+        of a unit circle. The circle is approximated using cubic Bezier
+        curves.  This uses 4 splines around the circle using the approach
+        presented here:
+
+          Lancaster, Don.  `Approximating a Circle or an Ellipse Using Four
+          Bezier Cubic Splines <http://www.tinaja.com/glib/ellipse4.pdf>`_.
+        """
+        if cls._unit_circle_righthalf is None:
+            MAGIC = 0.2652031
+            SQRTHALF = np.sqrt(0.5)
+            MAGIC45 = np.sqrt((MAGIC*MAGIC) / 2.0)
+
+            vertices = np.array(
+                [[0.0, -1.0],
+
+                 [MAGIC, -1.0],
+                 [SQRTHALF-MAGIC45, -SQRTHALF-MAGIC45],
+                 [SQRTHALF, -SQRTHALF],
+
+                 [SQRTHALF+MAGIC45, -SQRTHALF+MAGIC45],
+                 [1.0, -MAGIC],
+                 [1.0, 0.0],
+
+                 [1.0, MAGIC],
+                 [SQRTHALF+MAGIC45, SQRTHALF-MAGIC45],
+                 [SQRTHALF, SQRTHALF],
+
+                 [SQRTHALF-MAGIC45, SQRTHALF+MAGIC45],
+                 [MAGIC, 1.0],
+                 [0.0, 1.0],
+
+                 [0.0, -1.0]],
+
+                np.float_)
+
+            codes = cls.CURVE4 * np.ones(14)
+            codes[0] = cls.MOVETO
+            codes[-1] = cls.CLOSEPOLY
+
+            cls._unit_circle_righthalf = cls(vertices, codes)
+        return cls._unit_circle_righthalf
+
+    @classmethod
     def arc(cls, theta1, theta2, n=None, is_wedge=False):
         """
         (staticmethod) Returns an arc on the unit circle from angle
@@ -523,18 +652,18 @@ class Path(object):
         if is_wedge:
             length = n * 3 + 4
             vertices = np.zeros((length, 2), np.float_)
-            codes = Path.CURVE4 * np.ones((length, ), Path.code_type)
+            codes = cls.CURVE4 * np.ones((length, ), cls.code_type)
             vertices[1] = [xA[0], yA[0]]
-            codes[0:2] = [Path.MOVETO, Path.LINETO]
-            codes[-2:] = [Path.LINETO, Path.CLOSEPOLY]
+            codes[0:2] = [cls.MOVETO, cls.LINETO]
+            codes[-2:] = [cls.LINETO, cls.CLOSEPOLY]
             vertex_offset = 2
             end = length - 2
         else:
             length = n * 3 + 1
-            vertices = np.zeros((length, 2), np.float_)
-            codes = Path.CURVE4 * np.ones((length, ), Path.code_type)
+            vertices = np.empty((length, 2), np.float_)
+            codes = cls.CURVE4 * np.ones((length, ), cls.code_type)
             vertices[0] = [xA[0], yA[0]]
-            codes[0] = Path.MOVETO
+            codes[0] = cls.MOVETO
             vertex_offset = 1
             end = length
 
@@ -545,10 +674,9 @@ class Path(object):
         vertices[vertex_offset+2:end:3, 0] = xB
         vertices[vertex_offset+2:end:3, 1] = yB
 
-        return Path(vertices, codes)
-    arc = classmethod(arc)
+        return cls(vertices, codes)
 
-    #@classmethod
+    @classmethod
     def wedge(cls, theta1, theta2, n=None):
         """
         (staticmethod) Returns a wedge of the unit circle from angle
@@ -559,15 +687,93 @@ class Path(object):
         determined based on the delta between *theta1* and *theta2*.
         """
         return cls.arc(theta1, theta2, n, True)
-    wedge = classmethod(wedge)
+
+    _hatch_dict = maxdict(8)
+
+    @classmethod
+    def hatch(cls, hatchpattern, density=6):
+        """
+        Given a hatch specifier, *hatchpattern*, generates a Path that
+        can be used in a repeated hatching pattern.  *density* is the
+        number of lines per unit square.
+        """
+        from matplotlib.hatch import get_path
+
+        if hatchpattern is None:
+            return None
+
+        hatch_path = cls._hatch_dict.get((hatchpattern, density))
+        if hatch_path is not None:
+            return hatch_path
+
+        hatch_path = get_path(hatchpattern, density)
+        cls._hatch_dict[(hatchpattern, density)] = hatch_path
+        return hatch_path
+
+    def clip_to_bbox(self, bbox, inside=True):
+        """
+        Clip the path to the given bounding box.
+
+        The path must be made up of one or more closed polygons.  This
+        algorithm will not behave correctly for unclosed paths.
+
+        If *inside* is `True`, clip to the inside of the box, otherwise
+        to the outside of the box.
+        """
+        # Use make_compound_path_from_polys
+        verts = clip_path_to_rect(self, bbox, inside)
+        paths = [Path(poly) for poly in verts]
+        return self.make_compound_path(*paths)
+
 
 _get_path_collection_extents = get_path_collection_extents
-def get_path_collection_extents(*args):
+def get_path_collection_extents(
+        master_transform, paths, transforms, offsets, offset_transform):
     """
-    Given a sequence of :class:`Path` objects, returns the bounding
-    box that encapsulates all of them.
+    Given a sequence of :class:`Path` objects,
+    :class:`~matplotlib.transforms.Transform` objects and offsets, as
+    found in a :class:`~matplotlib.collections.PathCollection`,
+    returns the bounding box that encapsulates all of them.
+
+    *master_transform* is a global transformation to apply to all paths
+
+    *paths* is a sequence of :class:`Path` instances.
+
+    *transforms* is a sequence of
+    :class:`~matplotlib.transforms.Affine2D` instances.
+
+    *offsets* is a sequence of (x, y) offsets (or an Nx2 array)
+
+    *offset_transform* is a :class:`~matplotlib.transforms.Affine2D`
+    to apply to the offsets before applying the offset to the path.
+
+    The way that *paths*, *transforms* and *offsets* are combined
+    follows the same method as for collections.  Each is iterated over
+    independently, so if you have 3 paths, 2 transforms and 1 offset,
+    their combinations are as follows:
+
+        (A, A, A), (B, B, A), (C, A, A)
     """
     from transforms import Bbox
-    if len(args[1]) == 0:
+    if len(paths) == 0:
         raise ValueError("No paths provided")
-    return Bbox.from_extents(*_get_path_collection_extents(*args))
+    return Bbox.from_extents(*_get_path_collection_extents(
+        master_transform, paths, transforms, offsets, offset_transform))
+
+def get_paths_extents(paths, transforms=[]):
+    """
+    Given a sequence of :class:`Path` objects and optional
+    :class:`~matplotlib.transforms.Transform` objects, returns the
+    bounding box that encapsulates all of them.
+
+    *paths* is a sequence of :class:`Path` instances.
+
+    *transforms* is an optional sequence of
+    :class:`~matplotlib.transforms.Affine2D` instances to apply to
+    each path.
+    """
+    from transforms import Bbox, Affine2D
+    if len(paths) == 0:
+        raise ValueError("No paths provided")
+    return Bbox.from_extents(*_get_path_collection_extents(
+        Affine2D(), paths, transforms, [], Affine2D()))
