@@ -12,6 +12,7 @@ that actually reflects the values given here. Any additions or deletions to the
 parameter set listed here should also be visited to the
 :file:`matplotlibrc.template` in matplotlib's root source directory.
 """
+from __future__ import print_function
 
 import os
 import warnings
@@ -24,11 +25,12 @@ from matplotlib.colors import is_color_like
 # change for later versions.
 
 interactive_bk = ['GTK', 'GTKAgg', 'GTKCairo', 'FltkAgg', 'MacOSX',
-                  'QtAgg', 'Qt4Agg', 'TkAgg', 'WX', 'WXAgg', 'CocoaAgg']
+                  'QtAgg', 'Qt4Agg', 'TkAgg', 'WX', 'WXAgg', 'CocoaAgg',
+                  'GTK3Cairo', 'GTK3Agg']
 
 
 non_interactive_bk = ['agg', 'cairo', 'emf', 'gdk',
-                      'pdf', 'ps', 'svg', 'template']
+                      'pdf', 'pgf', 'ps', 'svg', 'template']
 all_backends = interactive_bk + non_interactive_bk
 
 
@@ -92,11 +94,11 @@ def validate_fonttype(s):
     try:
         fonttype = validate_int(s)
     except ValueError:
-        if s.lower() in fonttypes.keys():
+        if s.lower() in fonttypes.iterkeys():
             return fonttypes[s.lower()]
         raise ValueError('Supported Postscript/PDF font types are %s' % fonttypes.keys())
     else:
-        if fonttype not in fonttypes.values():
+        if fonttype not in fonttypes.itervalues():
             raise ValueError('Supported Postscript/PDF font types are %s' % fonttypes.values())
         return fonttype
 
@@ -106,17 +108,29 @@ def validate_backend(s):
     if s.startswith('module://'): return s
     else: return _validate_standard_backends(s)
 
-validate_numerix = ValidateInStrings('numerix',[
-    'Numeric','numarray','numpy',
-    ], ignorecase=True)
+validate_qt4 = ValidateInStrings('backend.qt4', ['PyQt4', 'PySide'])
 
-validate_toolbar = ValidateInStrings('toolbar',[
-    'None','classic','toolbar2',
-    ], ignorecase=True)
+def validate_toolbar(s):
+    validator = ValidateInStrings(
+                'toolbar',
+                ['None','classic','toolbar2'],
+                ignorecase=True)
+    s = validator(s)
+    if s.lower == 'classic':
+        warnings.warn("'classic' Navigation Toolbar "
+                      "is deprecated in v1.2.x and will be "
+                      "removed in v1.3")
+    return s
 
-def validate_autolayout(v):
-    if v:
-        warnings.warn("figure.autolayout is not currently supported")
+def validate_maskedarray(v):
+    # 2008/12/12: start warning; later, remove all traces of maskedarray
+    try:
+        if v == 'obsolete':
+            return v
+    except ValueError:
+        pass
+    warnings.warn('rcParams key "maskedarray" is obsolete and has no effect;\n'
+                  ' please delete it from your matplotlibrc file')
 
 class validate_nseq_float:
     def __init__(self, n):
@@ -159,8 +173,11 @@ class validate_nseq_int:
 
 def validate_color(s):
     'return a valid color arg'
-    if s.lower() == 'none':
-        return 'None'
+    try:
+        if s.lower() == 'none':
+            return 'None'
+    except AttributeError:
+        pass
     if is_color_like(s):
         return s
     stmp = '#' + s
@@ -186,6 +203,14 @@ def validate_color(s):
 
     raise ValueError('%s does not look like a color arg%s'%(s, msg))
 
+def validate_colorlist(s):
+    'return a list of colorspecs'
+    if type(s) is str:
+        return [validate_color(c.strip()) for c in s.split(',')]
+    else:
+        assert type(s) in [list, tuple]
+        return [validate_color(c) for c in s]
+
 def validate_stringlist(s):
     'return a list'
     if type(s) is str:
@@ -223,13 +248,24 @@ def validate_font_properties(s):
 
 validate_fontset = ValidateInStrings('fontset', ['cm', 'stix', 'stixsans', 'custom'])
 
+validate_mathtext_default = ValidateInStrings(
+    'default', "rm cal it tt sf bf default bb frak circled scr regular".split())
+
 validate_verbose = ValidateInStrings('verbose',[
     'silent', 'helpful', 'debug', 'debug-annoying',
     ])
 
-validate_cairo_format = ValidateInStrings('cairo_format',
-                            ['png', 'ps', 'pdf', 'svg'],
-                            ignorecase=True)
+def deprecate_savefig_extension(value):
+    warnings.warn("savefig.extension is deprecated.  Use savefig.format instead.")
+    return value
+
+def update_savefig_format(value):
+    # The old savefig.extension could also have a value of "auto", but
+    # the new savefig.format does not.  We need to fix this here.
+    value = str(value)
+    if value == 'auto':
+        value = 'png'
+    return value
 
 validate_ps_papersize = ValidateInStrings('ps_papersize',[
     'auto', 'letter', 'legal', 'ledger',
@@ -265,6 +301,11 @@ def validate_negative_linestyle_legacy(s):
         warnings.warn("Deprecated negative_linestyle specification; use 'solid' or 'dashed'")
         return (0, dashes)  # (offset, (solid, blank))
 
+def validate_tkpythoninspect(s):
+    # Introduced 2010/07/05
+    warnings.warn("tk.pythoninspect is obsolete, and has no effect")
+    return validate_bool(s)
+
 validate_legend_loc = ValidateInStrings('legend_loc',[
   'best',
   'upper right',
@@ -279,6 +320,39 @@ validate_legend_loc = ValidateInStrings('legend_loc',[
   'center',
 ], ignorecase=True)
 
+def deprecate_svg_embed_char_paths(value):
+    warnings.warn("svg.embed_char_paths is deprecated.  Use svg.fonttype instead.")
+
+validate_svg_fonttype = ValidateInStrings('fonttype', ['none', 'path', 'svgfont'])
+
+def validate_hinting(s):
+    if s in (True, False):
+        return s
+    if s.lower() in ('auto', 'native', 'either', 'none'):
+        return s.lower()
+    raise ValueError("hinting should be 'auto', 'native', 'either' or 'none'")
+
+validate_pgf_texsystem = ValidateInStrings('pgf.texsystem',
+                                           ['xelatex', 'lualatex', 'pdflatex'])
+
+validate_movie_writer = ValidateInStrings('animation.writer',
+    ['ffmpeg', 'ffmpeg_file', 'mencoder', 'mencoder_file',
+     'imagemagick', 'imagemagick_file'])
+
+validate_movie_frame_fmt = ValidateInStrings('animation.frame_format',
+    ['png', 'jpeg', 'tiff', 'raw', 'rgba'])
+
+def validate_bbox(s):
+    if type(s) is str:
+        s = s.lower()
+        if s == 'tight':
+            return s
+        if s == 'standard':
+            return None
+        raise ValueError("bbox should be 'tight' or 'standard'")
+
+
+
 class ValidateInterval:
     """
     Value must be in interval
@@ -310,11 +384,9 @@ class ValidateInterval:
 defaultParams = {
     'backend'           : ['Agg', validate_backend], # agg is certainly present
     'backend_fallback'  : [True, validate_bool], # agg is certainly present
-    'numerix'           : ['numpy', validate_numerix],
-    'maskedarray'       : [False, validate_bool],
+    'backend.qt4'       : ['PyQt4', validate_qt4],
     'toolbar'           : ['toolbar2', validate_toolbar],
     'datapath'          : [None, validate_path_exists],   # handled by _get_data_path_cached
-    'units'             : [False, validate_bool],
     'interactive'       : [False, validate_bool],
     'timezone'          : ['UTC', str],
 
@@ -330,8 +402,8 @@ defaultParams = {
     'lines.markeredgewidth' : [0.5, validate_float],
     'lines.markersize'      : [6, validate_float],       # markersize, in points
     'lines.antialiased'     : [True, validate_bool],     # antialised (no jaggies)
-    'lines.dash_joinstyle'  : ['miter', validate_joinstyle],
-    'lines.solid_joinstyle' : ['miter', validate_joinstyle],
+    'lines.dash_joinstyle'  : ['round', validate_joinstyle],
+    'lines.solid_joinstyle' : ['round', validate_joinstyle],
     'lines.dash_capstyle'   : ['butt', validate_capstyle],
     'lines.solid_capstyle'  : ['projecting', validate_capstyle],
 
@@ -348,7 +420,7 @@ defaultParams = {
     'font.variant'      : ['normal', str],           #
     'font.stretch'      : ['normal', str],           #
     'font.weight'       : ['normal', str],           #
-    'font.size'         : [12.0, validate_float], #
+    'font.size'         : [12, validate_float],      # Base font size in points
     'font.serif'        : [['Bitstream Vera Serif', 'DejaVu Serif',
                             'New Century Schoolbook', 'Century Schoolbook L',
                             'Utopia', 'ITC Bookman', 'Bookman',
@@ -373,12 +445,19 @@ defaultParams = {
     'text.usetex'         : [False, validate_bool],
     'text.latex.unicode'  : [False, validate_bool],
     'text.latex.preamble' : [[''], validate_stringlist],
+    'text.latex.preview' : [False, validate_bool],
     'text.dvipnghack'     : [None, validate_bool_maybe_none],
-    'text.fontstyle'      : ['normal', str],
-    'text.fontangle'      : ['normal', str],
-    'text.fontvariant'    : ['normal', str],
-    'text.fontweight'     : ['normal', str],
-    'text.fontsize'       : ['medium', validate_fontsize],
+    'text.hinting'        : [True, validate_hinting],
+    'text.hinting_factor' : [8, validate_int],
+    'text.antialiased'    : [True, validate_bool],
+
+    # The following are deprecated and replaced by, e.g., 'font.style'
+    #'text.fontstyle'      : ['normal', str],
+    #'text.fontangle'      : ['normal', str],
+    #'text.fontvariant'    : ['normal', str],
+    #'text.fontweight'     : ['normal', str],
+    #'text.fontsize'       : ['medium', validate_fontsize],
+
 
     'mathtext.cal'        : ['cursive', validate_font_properties],
     'mathtext.rm'         : ['serif', validate_font_properties],
@@ -387,6 +466,7 @@ defaultParams = {
     'mathtext.bf'         : ['serif:bold', validate_font_properties],
     'mathtext.sf'         : ['sans\-serif', validate_font_properties],
     'mathtext.fontset'    : ['cm', validate_fontset],
+    'mathtext.default'    : ['it', validate_mathtext_default],
     'mathtext.fallback_to_cm' : [True, validate_bool],
 
     'image.aspect'        : ['equal', validate_aspect],  # equal, auto, a number
@@ -407,14 +487,22 @@ defaultParams = {
     'axes.titlesize'        : ['large', validate_fontsize], # fontsize of the axes title
     'axes.grid'             : [False, validate_bool],   # display grid or not
     'axes.labelsize'        : ['medium', validate_fontsize], # fontsize of the x any y labels
+    'axes.labelweight'      : ['normal', str], # fontsize of the x any y labels
     'axes.labelcolor'       : ['k', validate_color],    # color of axis label
     'axes.formatter.limits' : [[-7, 7], validate_nseq_int(2)],
                                # use scientific notation if log10
                                # of the axis range is smaller than the
                                # first or larger than the second
+    'axes.formatter.use_locale' : [False, validate_bool],
+                               # Use the current locale to format ticks
+    'axes.formatter.use_mathtext' : [False, validate_bool],
     'axes.unicode_minus'        : [True, validate_bool],
+    'axes.color_cycle'      : [['b','g','r','c','m','y','k'],
+                                    validate_colorlist], # cycle of plot
+                                                         # line colors
 
     'polaraxes.grid'        : [True, validate_bool],   # display polar grid or not
+    'axes3d.grid'           : [True, validate_bool],   # display 3d grid
 
     #legend properties
     'legend.fancybox'         : [False,validate_bool],
@@ -422,20 +510,16 @@ defaultParams = {
     'legend.isaxes'      : [True,validate_bool],  # this option is internally ignored - it never served any useful purpose
     'legend.numpoints'   : [2, validate_int],     # the number of points in the legend line
     'legend.fontsize'    : ['large', validate_fontsize],
-    'legend.pad'         : [0,   validate_float], # was 0.2, deprecated; the fractional whitespace inside the legend border
-    'legend.borderpad'   : [0.4, validate_float], # units are fontsize
     'legend.markerscale' : [1.0, validate_float], # the relative size of legend markers vs. original
-
-    # the following dimensions are in axes coords
-    'legend.labelsep'      : [0.010, validate_float], # the vertical space between the legend entries
-    'legend.handlelen'     : [0.05, validate_float], # the length of the legend lines
-    'legend.handletextsep' : [0.02, validate_float], # the space between the legend line and legend text
-    'legend.axespad'       : [0.02, validate_float], # the border between the axes and legend edge
     'legend.shadow'        : [False, validate_bool],
+    'legend.frameon'     : [True, validate_bool], # whether or not to draw a frame around legend
 
 
+    # the following dimensions are in fraction of the font size
+    'legend.borderpad'   : [0.4, validate_float], # units are fontsize
     'legend.labelspacing'      : [0.5, validate_float], # the vertical space between the legend entries
     'legend.handlelength'     : [2., validate_float], # the length of the legend lines
+    'legend.handleheight'     : [0.7, validate_float], # the length of the legend lines
     'legend.handletextpad' : [.8, validate_float], # the space between the legend line and legend text
     'legend.borderaxespad'       : [0.5, validate_float], # the border between the axes and legend edge
     'legend.columnspacing'       : [2., validate_float], # the border between the axes and legend edge
@@ -443,19 +527,13 @@ defaultParams = {
 
     'legend.markerscale' : [1.0, validate_float], # the relative size of legend markers vs. original
 
-    # the following dimensions are in axes coords
-    'legend.labelsep'      : [0.010, validate_float], # the vertical space between the legend entries
-    'legend.handlelen'     : [0.05, validate_float], # the length of the legend lines
-    'legend.handletextsep' : [0.02, validate_float], # the space between the legend line and legend text
-    'legend.axespad'       : [0.5, validate_float], # the border between the axes and legend edge
     'legend.shadow'        : [False, validate_bool],
 
-
-
-
     # tick properties
     'xtick.major.size' : [4, validate_float],      # major xtick size in points
     'xtick.minor.size' : [2, validate_float],      # minor xtick size in points
+    'xtick.major.width': [0.5, validate_float],      # major xtick width in points
+    'xtick.minor.width': [0.5, validate_float],      # minor xtick width in points
     'xtick.major.pad'  : [4, validate_float],      # distance to label in points
     'xtick.minor.pad'  : [4, validate_float],      # distance to label in points
     'xtick.color'      : ['k', validate_color],    # color of the xtick labels
@@ -464,6 +542,8 @@ defaultParams = {
 
     'ytick.major.size' : [4, validate_float],      # major ytick size in points
     'ytick.minor.size' : [2, validate_float],      # minor ytick size in points
+    'ytick.major.width': [0.5, validate_float],      # major ytick width in points
+    'ytick.minor.width': [0.5, validate_float],      # minor ytick width in points
     'ytick.major.pad'  : [4, validate_float],      # distance to label in points
     'ytick.minor.pad'  : [4, validate_float],      # distance to label in points
     'ytick.color'      : ['k', validate_color],    # color of the ytick labels
@@ -473,6 +553,7 @@ defaultParams = {
     'grid.color'       : ['k', validate_color],       # grid color
     'grid.linestyle'   : [':', str],       # dotted
     'grid.linewidth'   : [0.5, validate_float],     # in points
+    'grid.alpha'       : [1.0, validate_float],
 
 
     # figure props
@@ -481,7 +562,7 @@ defaultParams = {
     'figure.dpi'        : [ 80, validate_float],   # DPI
     'figure.facecolor'  : [ '0.75', validate_color], # facecolor; scalar gray
     'figure.edgecolor'  : [ 'w', validate_color],  # edgecolor; white
-    'figure.autolayout' : [ False, validate_autolayout],
+    'figure.autolayout' : [ False, validate_bool],
 
     'figure.subplot.left'   : [0.125, ValidateInterval(0, 1, closedmin=True, closedmax=True)],
     'figure.subplot.right'  : [0.9, ValidateInterval(0, 1, closedmin=True, closedmax=True)],
@@ -494,10 +575,13 @@ defaultParams = {
     'savefig.facecolor'   : ['w', validate_color],  # facecolor; white
     'savefig.edgecolor'   : ['w', validate_color],  # edgecolor; white
     'savefig.orientation' : ['portrait', validate_orientation],  # edgecolor; white
+    'savefig.extension'   : ['png', deprecate_savefig_extension], # what to add to extensionless filenames
+    'savefig.format'      : ['png', update_savefig_format], # value checked by backend at runtime
+    'savefig.bbox'        : [None, validate_bbox], # options are 'tight', or 'standard'. 'standard' validates to None.
+    'savefig.pad_inches'  : [0.1, validate_float],
 
-    'cairo.format'       : ['png', validate_cairo_format],
     'tk.window_focus'    : [False, validate_bool],  # Maintain shell focus for TkAgg
-    'tk.pythoninspect'   : [False, validate_bool],  # Set PYTHONINSPECT
+    'tk.pythoninspect'   : [False, validate_tkpythoninspect],  # obsolete
     'ps.papersize'       : ['letter', validate_ps_papersize], # Set the papersize/type
     'ps.useafm'          : [False, validate_bool],  # Set PYTHONINSPECT
     'ps.usedistiller'    : [False, validate_ps_distiller], # use ghostscript or xpdf to distill ps output
@@ -508,17 +592,52 @@ defaultParams = {
     'pdf.use14corefonts' : [False, validate_bool],  # use only the 14 PDF core fonts
                                                     # embedded in every PDF viewing application
     'pdf.fonttype'      : [3, validate_fonttype],  # 3 (Type3) or 42 (Truetype)
+
+    'pgf.debug'         : [False, validate_bool],  # output debug information
+    'pgf.texsystem'     : ['xelatex', validate_pgf_texsystem], # choose latex application for creating pdf files (xelatex/lualatex)
+    'pgf.rcfonts'       : [True, validate_bool],   # use matplotlib rc settings for font configuration
+    'pgf.preamble'      : [[''], validate_stringlist], # provide a custom preamble for the latex process
+
     'svg.image_inline'  : [True, validate_bool],    # write raster image data directly into the svg file
     'svg.image_noscale' : [False, validate_bool],  # suppress scaling of raster data embedded in SVG
-    'svg.embed_char_paths' : [True, validate_bool],  # True to save all characters as paths in the SVG
+    'svg.embed_char_paths' : [True, deprecate_svg_embed_char_paths],  # True to save all characters as paths in the SVG
+    'svg.fonttype' : ['path', validate_svg_fonttype],
 
     'docstring.hardcopy' : [False, validate_bool],  # set this when you want to generate hardcopy docstring
     'plugins.directory' : ['.matplotlib_plugins', str], # where plugin directory is locate
 
-    'path.simplify' : [False, validate_bool],
-    'agg.path.chunksize' : [0, validate_int]       # 0 to disable chunking;
-                                                   # recommend about 20000 to
-                                                   # enable. Experimental.
+    'path.simplify' : [True, validate_bool],
+    'path.simplify_threshold' : [1.0 / 9.0, ValidateInterval(0.0, 1.0)],
+    'path.snap' : [True, validate_bool],
+    'agg.path.chunksize' : [0, validate_int],       # 0 to disable chunking;
+                                                    # recommend about 20000 to
+                                                    # enable. Experimental.
+    # key-mappings (multi-character mappings should be a list/tuple)
+    'keymap.fullscreen' : [('f', 'ctrl+f'), validate_stringlist],
+    'keymap.home' : [['h', 'r', 'home'], validate_stringlist],
+    'keymap.back' : [['left', 'c', 'backspace'], validate_stringlist],
+    'keymap.forward' : [['right', 'v'], validate_stringlist],
+    'keymap.pan' : ['p', validate_stringlist],
+    'keymap.zoom' : ['o', validate_stringlist],
+    'keymap.save' : [('s', 'ctrl+s'), validate_stringlist],
+    'keymap.quit' : [('ctrl+w', ), validate_stringlist],
+    'keymap.grid' : ['g', validate_stringlist],
+    'keymap.yscale' : ['l', validate_stringlist],
+    'keymap.xscale' : [['k', 'L'], validate_stringlist],
+    'keymap.all_axes' : ['a', validate_stringlist],
+
+    # sample data
+    'examples.directory' : ['', str],
+
+    # Animation settings
+    'animation.writer' : ['ffmpeg', validate_movie_writer],
+    'animation.codec' : ['mpeg4', str],
+    'animation.bitrate' : [-1, validate_int],
+    'animation.frame_format' : ['png', validate_movie_frame_fmt], # Controls image format when frames are written to disk
+    'animation.ffmpeg_path' : ['ffmpeg', str], # Path to FFMPEG binary. If just binary name, subprocess uses $PATH.
+    'animation.ffmpeg_args' : ['', validate_stringlist], # Additional arguments for ffmpeg movie writer (using pipes)
+    'animation.mencoder_path' : ['mencoder', str], # Path to FFMPEG binary. If just binary name, subprocess uses $PATH.
+    'animation.mencoder_args' : ['', validate_stringlist], # Additional arguments for mencoder movie writer (using pipes)
 }
 
 if __name__ == '__main__':
@@ -526,4 +645,4 @@ if __name__ == '__main__':
     rc['datapath'][0] = '/'
     for key in rc:
         if not rc[key][1](rc[key][0]) == rc[key][0]:
-            print "%s: %s != %s"%(key, rc[key][1](rc[key][0]), rc[key][0])
+            print("%s: %s != %s"%(key, rc[key][1](rc[key][0]), rc[key][0]))
