@@ -1,9 +1,9 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 
 """
-The new ticker code was designed to explicity support user customized
+The new ticker code was designed to explicitly support user customized
 ticking.  The documentation
-http://matplotlib.sourceforge.net/matplotlib.ticker.html details this
+http://matplotlib.org/matplotlib.ticker.html details this
 process.  That code defines a lot of preset tickers but was primarily
 designed to be user extensible.
 
