@@ -1,12 +1,14 @@
-#!/usr/bin/env python2.7
+#!/usr/local/bin/python2.7
+
 """
 Show how to make date plots in matplotlib using date tick locators and
 formatters.  See major_minor_demo1.py for more information on
 controlling major and minor ticks
 """
+from __future__ import print_function
 import datetime
 from pylab import figure, show
-from matplotlib.dates import MONDAY, SATURDAY
+from matplotlib.dates import MONDAY
 from matplotlib.finance import quotes_historical_yahoo
 from matplotlib.dates import MonthLocator, WeekdayLocator, DateFormatter
 
@@ -23,8 +25,8 @@ monthsFmt = DateFormatter("%b '%y")
 
 
 quotes = quotes_historical_yahoo('INTC', date1, date2)
-if not quotes:
-    print 'Found no quotes'
+if len(quotes) == 0:
+    print ('Found no quotes')
     raise SystemExit
 
 dates = [q[0] for q in quotes]
