@@ -1,9 +1,8 @@
-#!/usr/bin/env python2.5
+#!/usr/local/bin/python2.7
 import matplotlib
 matplotlib.rcParams['legend.fancybox'] = True
 import matplotlib.pyplot as plt
 import numpy as np
-import pylab
 
 def myplot(ax):
     t1 = np.arange(0.0, 1.0, 0.1)
@@ -18,16 +17,15 @@ ax1.legend(loc=1, ncol=3, shadow=True)
 
 ax2 = plt.subplot(3,1,2)
 myplot(ax2)
-ax2.legend(loc=1, ncol=2, shadow=True)
-
+ax2.legend(loc="center left", bbox_to_anchor=[0.5, 0.5],
+           ncol=2, shadow=True, title="Legend")
+ax2.get_legend().get_title().set_color("red")
 
 ax3 = plt.subplot(3,1,3)
 myplot(ax3)
-ax3.legend(loc=1, ncol=4, mode="expand", shadow=True)
+ax3.legend(shadow=True, fancybox=True)
 
 
-#title('Damped oscillation')
-
 plt.draw()
 plt.show()
 
