@@ -1,6 +1,8 @@
-#!/usr/bin/env python2.5
+#!/usr/local/bin/python2.7
 # -*- noplot -*-
 
+from __future__ import print_function
+
 import os, sys
 from pylab import *
 
@@ -11,11 +13,11 @@ for i in range(50):  # 50 frames
     cla()
     imshow(rand(5,5), interpolation='nearest')
     fname = '_tmp%03d.png'%i
-    print 'Saving frame', fname
+    print('Saving frame', fname)
     savefig(fname)
     files.append(fname)
 
-print 'Making movie animation.mpg - this make take a while'
+print('Making movie animation.mpg - this make take a while')
 os.system("mencoder 'mf://_tmp*.png' -mf type=png:fps=10 -ovc lavc -lavcopts vcodec=wmv2 -oac copy -o animation.mpg")
 #os.system("convert _tmp*.png animation.mng")
 
