@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.7
+#!/usr/local/bin/python2.7
 """
 See pcolor_demo2 for an alternative way of generating pcolor plots
 using imshow that is likely faster for large grids
@@ -19,7 +19,7 @@ y = arange(-3.0, 3.0001, dy)
 X,Y = meshgrid(x, y)
 
 Z = func3(X, Y)
-pcolor(X, Y, Z)
+pcolor(X, Y, Z, cmap=cm.RdBu, vmax=abs(Z).max(), vmin=-abs(Z).max())
 colorbar()
 axis([-3,3,-3,3])
 
