@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 """
 See pcolor_demo2 for a much faster way of generating pcolor plots
 """
@@ -20,7 +20,7 @@ Z = func3(X, Y)
 
 
 ax = subplot(111)
-im = imshow(Z, cmap=cm.jet)
+im = imshow(Z, cmap=cm.RdBu, vmax=abs(Z).max(), vmin=-abs(Z).max())
 #im.set_interpolation('nearest')
 #im.set_interpolation('bicubic')
 im.set_interpolation('bilinear')
