@@ -1,9 +1,11 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
+
 """
 Example: simple line plot.
 Show how to make and save a simple line plot with labels, title and grid
 """
 # -*- noplot -*-
+from __future__ import print_function
 from pylab import *
 
 ion()
@@ -22,11 +24,11 @@ import time
 frames = 100.0
 t = time.time()
 c = time.clock()
-for i in xrange(int(frames)):
+for i in range(int(frames)):
     part = i / frames
     axis([0.0, 1.0 - part, -1.0 + part, 1.0 - part])
 wallclock = time.time() - t
 user = time.clock() - c
-print "wallclock:", wallclock
-print "user:", user
-print "fps:", frames / wallclock
+print ("wallclock:", wallclock)
+print ("user:", user)
+print ("fps:", frames / wallclock)
