@@ -1,20 +1,21 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 """
 Use backend agg to access the figure canvas as an RGB string and then
 convert it to an array and pass the string it to PIL for
 rendering
 """
 
-from pylab import *
+import pylab
 from matplotlib.backends.backend_agg import FigureCanvasAgg
+
 try:
-    import Image
-except ImportError, exc:
+    from PIL import Image
+except ImportError:
     raise SystemExit("PIL must be installed to run this example")
 
-plot([1,2,3])
+pylab.plot([1,2,3])
 
-canvas = get_current_fig_manager().canvas
+canvas = pylab.get_current_fig_manager().canvas
 
 agg = canvas.switch_backends(FigureCanvasAgg)
 agg.draw()
@@ -25,7 +26,7 @@ l,b,w,h = agg.figure.bbox.bounds
 w, h = int(w), int(h)
 
 
-X = fromstring(s, uint8)
+X = pylab.fromstring(s, pylab.uint8)
 X.shape = h, w, 3
 
 im = Image.fromstring( "RGB", (w,h), s)
